Package: CarsAnalyser
Title: Motor Trend Cars Analyser
Version: 1.0.0
RDeskVersion: 0.9.0
Depends: ggplot2, dplyr, broom
Description: A professional RDesk showcase demonstrating statistical modeling and native UX.
Author: Janakiraman
License: MIT

Package: RDesk
Title: Native Desktop App Framework for 'R'
Version: 1.0.5
Authors@R: c(
    person("Janakiraman", "G",
           email = "janakiraman.bt@gmail.com",
           role  = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-6050-2318")),
    person("Serge", "Zaitsev",
           role  = "cph",
           comment = "Author of webview library (src/webview/webview.h)"),
    person(given   = "Niels",
           family  = "Lohmann",
           role    = "cph",
           comment = "Author of nlohmann/json (inst/include/nlohmann/json.hpp)"),
    person(given   = "Microsoft Corporation",
           role    = "cph",
           comment = "WebView2 SDK (src/webview2_sdk/)")
  )
Description: Build native Windows desktop applications using 'R' and 
    'WebView2'. Provides a robust 'R6'-based event loop, asynchronous 
    background task management via 'mirai' and 'callr', and a native 
    'Win32' message bridge for seamless 'R'-to-user-interface 
    communication without listening ports or network overhead. Allows 'R' 
    developers to create professional, standalone desktop tools with modern 
    web-based user interfaces while maintaining a pure 'R' backend.
OS_type: windows
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
URL: https://github.com/Janakiraman-311/RDesk,
        https://janakiraman-311.github.io/RDesk/
BugReports: https://github.com/Janakiraman-311/RDesk/issues
SystemRequirements: Windows 10 or later, 'Rtools44' or later
        (https://cran.r-project.org/bin/windows/Rtools/), Microsoft
        'WebView2' Runtime
        (https://developer.microsoft.com/microsoft-edge/webview2/)
RoxygenNote: 7.3.3
Imports: R6, jsonlite, digest, processx, callr, mirai (>= 1.0.0),
        base64enc, zip, stats, utils, parallel
Suggests: testthat (>= 3.0.0), withr, fs, knitr, rmarkdown, devtools,
        renv, broom, ggplot2, dplyr, rstudioapi, pkgdown
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-04-22 09:34:40 UTC; Janak
Author: Janakiraman G [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6050-2318>),
  Serge Zaitsev [cph] (Author of webview library (src/webview/webview.h)),
  Niels Lohmann [cph] (Author of nlohmann/json
    (inst/include/nlohmann/json.hpp)),
  Microsoft Corporation [cph] (WebView2 SDK (src/webview2_sdk/))
Maintainer: Janakiraman G <janakiraman.bt@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-22 13:00:02 UTC

# Generated by roxygen2: do not edit by hand

export(App)
export(async)
export(build_app)
export(rdesk_async)
export(rdesk_auto_update)
export(rdesk_cancel_job)
export(rdesk_create_app)
export(rdesk_df_to_list)
export(rdesk_error_plot)
export(rdesk_is_bundle)
export(rdesk_jobs_list)
export(rdesk_jobs_pending)
export(rdesk_message)
export(rdesk_parse_message)
export(rdesk_plot_to_base64)
export(rdesk_service)
importFrom(R6,R6Class)
importFrom(base64enc,base64encode)
importFrom(callr,r_bg)
importFrom(digest,digest)
importFrom(stats,runif)

# RDesk

[![CRAN status](https://www.r-pkg.org/badges/version/RDesk)](https://CRAN.R-project.org/package=RDesk)
[![R-CMD-check](https://github.com/Janakiraman-311/RDesk/actions/workflows/R-CMD-check.yml/badge.svg)](https://github.com/Janakiraman-311/RDesk/actions/workflows/R-CMD-check.yml)
[![build-app](https://github.com/Janakiraman-311/RDesk/actions/workflows/build-app.yml/badge.svg)](https://github.com/Janakiraman-311/RDesk/actions/workflows/build-app.yml)
[![pkgdown](https://github.com/Janakiraman-311/RDesk/actions/workflows/pkgdown.yaml/badge.svg)](https://janakiraman-311.github.io/RDesk/)

![RDesk Dashboard](man/figures/landing.png)

**RDesk** is a framework for building native Windows desktop applications with R. It turns your logic into a standalone `.exe` that runs on any machine—no R installation required by the end-user.

## Quick Start

Get a professional dashboard running in seconds:

```r
# 1. Install RDesk from CRAN
install.packages("RDesk")

# 2. Create a working dashboard
RDesk::rdesk_create_app("MyDashboard")
```

> **Install the development version from GitHub:**
> ```r
> devtools::install_github("Janakiraman-311/RDesk")
> ```

## Why RDesk?

RDesk solves the "Last Mile" problem of R deployment. Instead of a browser URL, you give your users a familiar Windows tool.

| Feature | Shiny | RDesk |
|:---|:---|:---|
| **Delivery** | Browser + Server | **Native .exe** |
| **Network Ports** | Yes (httpuv) | **Zero (Native Pipe)** |
| **Offline Use** | No | **Yes** |
| **Distribution** | Deploy to Cloud | **Single ZIP or Installer** |
| **User Experience** | Website-like | **Desktop Native** |

## Who uses RDesk?

- Data analysts building internal tools that cannot live on a server
- Consultants distributing one-off analysis tools to clients
- Teams replacing Excel macros with proper R-powered apps
- Organisations that need offline, zero-IT-involvement deployment

Not the right fit for web applications, cross-platform needs, 
or real-time collaborative tools — use Shiny for those.

## Core Benefits

*   **🔒 Zero-Port IPC**: Native bidirectional pipes between R and the UI. No firewall issues or port conflicts.
*   **⚡ Async by Default**: Built-in background task processing via `mirai`. The UI never freezes, even during heavy R computations.
*   **📦 Portable Runtime**: Packages a minimal R distribution into your `.exe`. Your users don't need to install R.
*   **🎨 Modern Web UI**: Use HTML/JS/CSS for the interface while keeping 100% of your logic in R.
*   **🛠 Professional Scaffolding**: Generate dashboards with sidebar navigation, Dark Mode, and auto-wired charts in one command.

## Distribute Your Work

Building a professional installer is a single command away:

```r
RDesk::build_app(
  app_dir         = "MyDashboard",
  app_name        = "SalesTool",
  build_installer = TRUE
)
# Output: dist/SalesTool-1.0.0-setup.exe
```

## Learn More

Visit the full documentation at **[janakiraman-311.github.io/RDesk](https://janakiraman-311.github.io/RDesk/)**

*   [**Getting Started**](https://janakiraman-311.github.io/RDesk/articles/getting-started.html) — From zero to your first native app.
*   [**Coming from Shiny**](https://janakiraman-311.github.io/RDesk/articles/shiny-migration.html) — A side-by-side guide to mapping your Shiny knowledge to RDesk.
*   [**Async Guide**](https://janakiraman-311.github.io/RDesk/articles/async-guide.html) — Mastering background tasks and progress overlays.
*   [**Cookbook**](https://janakiraman-311.github.io/RDesk/articles/cookbook.html) — Copy-paste recipes for common desktop patterns.

## License

MIT © Janakiraman G.

# Phase 21 Pre-condition Verification Script
devtools::load_all(".")

# 1. Verify mirai daemon pool lifecycle
cat("--- Testing mirai lifecycle ---\n")
RDesk:::rdesk_start_daemons()
cat("Daemons started. Pending jobs:", RDesk::rdesk_jobs_pending(), "\n")

# 2. Verify async() wrapper logic (Simulated App context)
cat("\n--- Testing async() wrapper ---\n")
# Create a dummy app-like object in global env so async() can find it via parent.frame lookup
app <- list(
  loading_start = function(...) cat("[MockApp] loading_start called\n"),
  loading_done  = function(...) cat("[MockApp] loading_done called\n"),
  send          = function(type, payload) cat("[MockApp] send called with type:", type, "\n"),
  toast         = function(msg, type) cat("[MockApp] toast called:", msg, "(", type, ")\n")
)

# Create a handler using async()
handler <- RDesk::async(function(p) {
  list(res = p$x * 2)
})

# Manually inject type (as App$on_message would do)
attr_env <- attr(handler, "rdesk_msg_type_env")
attr_env$type <- "test_task"

# Run the handler
cat("Running async handler...\n")
job_id <- handler(list(x = 21))
cat("Job launched with ID:", job_id, "\n")

# Wait a bit for result
cat("Waiting for task completion...\n")
Sys.sleep(1.5)
RDesk:::rdesk_poll_jobs() # Trigger polling logic

# 3. Cleanup
RDesk:::rdesk_stop_daemons()
cat("\nDaemons stopped. Pending jobs:", RDesk::rdesk_jobs_pending(), "\n")
cat("--- Verification Complete ---\n")

# tests/manual/test_phase18.R
# Manual verification for Phase 18: Loading & Toasts

# Find project root (assume it's 2 levels up from here)
script_dir <- getwd()
if (basename(script_dir) == "manual") script_dir <- dirname(dirname(script_dir))
if (file.exists("DESCRIPTION")) {
  devtools::load_all(".")
} else {
  # Try to find RDesk folder if run from USER folder
  potential_root <- file.path(getwd(), "OneDrive/Documents/RDesk")
  if (dir.exists(potential_root)) {
    setwd(potential_root)
    devtools::load_all(".")
  } else {
    stop("Could not find RDesk project root. Please cd to the project directory.")
  }
}

# Create a demo app instance
app <- App$new(title = "Phase 18 Verification")

app$on_ready(function() {
  # Small delay for bridge to be ready
  Sys.sleep(1)
  
  rscript <- file.path(R.home("bin"), "Rscript.exe")
  if (!file.exists(rscript)) rscript <- "Rscript"

  app$toast("Phase 18 Verified: UI Bridge is Live!", type = "info")
  Sys.sleep(1)
  
  dummy_job <- "verify_job_123"
  app$loading_start(
    message     = "Processing background task...", 
    cancellable = TRUE, 
    job_id      = dummy_job
  )
  
  processx::run(rscript, c("-e", "Sys.sleep(1)"))
  app$loading_progress(30, "Still working...")
  
  processx::run(rscript, c("-e", "Sys.sleep(1)"))
  app$loading_progress(70, "Almost there...")
  
  processx::run(rscript, c("-e", "Sys.sleep(1)"))
  
  app$loading_done()
  app$toast("Verification sequence complete!", type = "success")
})

# Run the app
# Note: In a real terminal, this will open the window.
app$run()

# tests/manual/test_phase2.R
# Run with: source("tests/manual/test_phase2.R")

library(devtools)
load_all("c:/Users/Janak/OneDrive/Documents/RDesk/RDesk", recompile = FALSE)

app <- App$new(
  title  = "Phase 2 IPC Test",
  width  = 800,
  height = 600
)

# Wire up all three message handlers
app$on_message("ping", function(msg) {
  cat("[R] Received ping, sending pong\n")
  app$send("pong", list(r_ts = format(Sys.time()), js_ts = msg$ts))
})

app$on_message("get_data", function(msg) {
  cat("[R] Received get_data request\n")
  df <- mtcars
  app$send("data_result", list(
    nrow = nrow(df),
    cols = names(df),
    rows = head(df, 5)
  ))
})

app$on_message("get_time", function(msg) {
  app$send("time_result", list(time = format(Sys.time(), "%H:%M:%S")))
})

app$on_ready(function() {
  cat("[R] App ready. Window opened.\n")
})

app$run()

#' @importFrom base64enc base64encode
NULL

# R/zzz.R
# Package initialization logic

.onLoad <- function(libname, pkgname) {
  # Set default IPC version for the contract
  options(rdesk.ipc_version = "1.0")

  # CI Guard: Detect if running in GitHub Actions and set async backend
  if (Sys.getenv("GITHUB_ACTIONS") == "true") {
    options(rdesk.ci_mode    = TRUE)
    options(rdesk.async_backend = "callr")  # mirai daemons fail in CI headless
  } else {
    # Default to mirai if available, fallback to callr
    backend <- if (requireNamespace("mirai", quietly = TRUE)) "mirai" else "callr"
    options(rdesk.async_backend = backend)
  }

  # Ensure clean job registry on load to prevent stale state across sessions
  rm(list = ls(envir = .rdesk_jobs), envir = .rdesk_jobs)
  rm(list = ls(envir = .rdesk_apps), envir = .rdesk_apps)
}

.onAttach <- function(libname, pkgname) {
  # Platform Guard: Provide a clear message on non-Windows platforms
  if (.Platform$OS.type != "windows") {
    packageStartupMessage(
      "[RDesk] RDesk requires Windows 10 or later.\n",
      "  macOS and Linux support is planned for v2.0.\n",
      "  See: https://github.com/Janakiraman-311/RDesk"
    )
  }

  packageStartupMessage(
    "[RDesk] v", utils::packageVersion("RDesk"), " ready."
  )
  
  # Check for launcher presence
  # Note: rdesk_launcher_path() would stop() here, so we use file.exists manually
  path <- system.file("bin", "rdesk-launcher.exe", package = "RDesk")
  if (path == "" || !file.exists(path)) {
     packageStartupMessage(
       "\n[WARNING] Native launcher binary not found in the installed package library.\n",
       "Tip: If you installed RDesk from source, ensure you have Rtools installed and\n",
       "run `devtools::install(pkg = '.', upgrade = FALSE, quick = TRUE)` to build it."
     )
  }
}

#' Create a new RDesk application
#'
#' @description
#' Scaffolds a professional RDesk application with a modern dashboard layout.
#' The app includes a sidebar for filters, KPI cards, and an asynchronous
#' ggplot2 charting engine fueled by mtcars (default).
#'
#' @param name App name. Must be a valid directory name.
#' @param path Directory to create the app in. Default is current directory.
#' @param theme One of "light", "dark", "system". Default "system".
#' @param open Logical. If TRUE and in RStudio, opens the new project in a new session.
#' @param data_source Internal use. Defaults to "builtin".
#' @param viz_type Internal use. Defaults to "mixed".
#' @param use_async Internal use. Defaults to TRUE.
#'
#' @return Path to the created app directory, invisibly.
#' @examples
#' if (interactive()) {
#'   # Create the Professional Hero Dashboard in a temporary directory
#'   rdesk_create_app("MyDashboard", path = tempdir())
#' }
#' 
#' # The following demonstrates just the return value without opening a window
#' # (Fast and safe - no \dontrun needed for this specific logical check)
#' path <- file.path(tempdir(), "TestLogic")
#' if (!dir.exists(path)) {
#'   # This is just a placeholder example of how to call the function safely
#'   message("Scaffold path will be: ", path)
#' }
#' @export
rdesk_create_app <- function(name,
                              path        = tempdir(),
                              data_source = NULL,
                              viz_type    = NULL,
                              use_async   = NULL,
                              theme       = "light",
                              open        = TRUE) {

  # Validate name
  if (missing(name) || !nzchar(trimws(name))) {
    stop("[RDesk] App name is required. Example: rdesk_create_app('MyApp')")
  }
  
  name <- trimws(name)
  if (!grepl("^[A-Za-z][A-Za-z0-9._-]*$", name)) {
    stop("[RDesk] App name must start with a letter and contain only ",
         "letters, numbers, dots, hyphens, or underscores.")
  }

  # Simply default to the Hero (Advanced Dashboard) automatically
  if (is.null(data_source)) data_source <- "builtin"
  if (is.null(viz_type))    viz_type    <- "mixed"
  if (is.null(use_async))   use_async   <- TRUE
  if (is.null(theme))       theme       <- "system"

  # Ensure length 1 for all parameters
  data_source <- as.character(data_source)[1]
  viz_type    <- as.character(viz_type)[1]
  use_async   <- isTRUE(use_async[1])
  theme       <- as.character(theme)[1]

  # Create app directory
  app_dir <- normalizePath(file.path(path, name), mustWork = FALSE)
  if (dir.exists(app_dir)) {
    stop("[RDesk] Directory already exists: ", app_dir,
         "\nChoose a different name or delete the existing directory.")
  }

  message("\n[RDesk] Generating ", name, "...")

  # Generate from template
  rdesk_scaffold_files(
    app_dir     = app_dir,
    name        = name,
    data_source = data_source,
    viz_type    = viz_type,
    use_async   = isTRUE(use_async),
    theme       = theme
  )

  # Success message
  rdesk_scaffold_success_msg(name, app_dir, data_source, viz_type, use_async)

  # Open in RStudio if available
  if (open && requireNamespace("rstudioapi", quietly = TRUE)) {
    if (rstudioapi::isAvailable()) {
      rstudioapi::openProject(app_dir, newSession = TRUE)
    }
  }

  invisible(app_dir)
}


#' Internal success message
#' @keywords internal
rdesk_scaffold_success_msg <- function(name, app_dir, data_source, viz_type, use_async) {
  message("\n[RDesk] Successfully created: ", app_dir, "\n")
  message("[RDesk] Your Professional Dashboard includes:")
  message("  - Mixed visualization (Charts + Tables)")
  message("  - Built-in KPI cards system")
  message("  - Sidebar filtering engine")
  message("  - Background processing (Async Workers)")
  message("\n[RDesk] Run it now:")
  message(sprintf("  setwd(%s)\n", shQuote(normalizePath(app_dir, winslash = "\\"))))
  message("  source(\"app.R\")\n\n")
  message("[RDesk] Build your executable when ready:")
  message(sprintf("  RDesk::build_app(app_dir = %s, app_name = %s)\n\n",
              shQuote(normalizePath(app_dir, winslash = "\\")), shQuote(name)))
}


#' @keywords internal
rdesk_scaffold_files <- function(app_dir, name, data_source,
                                  viz_type, use_async, theme) {
  # Directory structure
  dirs <- c(app_dir,
            file.path(app_dir, "R"),
            file.path(app_dir, "www"),
            file.path(app_dir, "www", "css"),
            file.path(app_dir, "www", "js"),
            file.path(app_dir, "data"))
  lapply(dirs, dir.create, recursive = TRUE, showWarnings = FALSE)

  # Template variables available to all templates
  vars <- list(
    APP_NAME    = name,
    APP_TITLE   = gsub("[._-]", " ", name),
    DATA_SOURCE = data_source,
    VIZ_TYPE    = viz_type,
    USE_ASYNC   = use_async,
    THEME       = theme,
    RDESK_VER   = as.character(utils::packageVersion("RDesk")),
    DATE        = format(Sys.Date(), "%Y-%m-%d")
  )

  # Write each file from its template
  rdesk_write_template("app.R",           file.path(app_dir, "app.R"),           vars)
  rdesk_write_template("DESCRIPTION",     file.path(app_dir, "DESCRIPTION"),     vars)
  
  # Server variant based on async
  rdesk_write_template("R/server.R",      file.path(app_dir, "R", "server.R"),   vars,
                        variant = if (use_async) "async" else "sync")
                        
  rdesk_write_template("R/data.R",        file.path(app_dir, "R", "data.R"),     vars,
                        variant = data_source)
  rdesk_write_template("R/plots.R",       file.path(app_dir, "R", "plots.R"),    vars,
                        variant = viz_type)
  rdesk_write_template("www/index.html",  file.path(app_dir, "www", "index.html"), vars,
                        variant = viz_type)
  rdesk_write_template("www/css/style.css", file.path(app_dir, "www", "css", "style.css"), vars,
                        variant = theme)
  rdesk_write_template("www/js/app.js",   file.path(app_dir, "www", "js", "app.js"), vars,
                        variant = viz_type)
  rdesk_write_template("data/README.md",  file.path(app_dir, "data", "README.md"), vars)

  # Copy rdesk.js from package
  rdesk_js_src <- system.file("www", "rdesk.js", package = "RDesk")
  if (!nzchar(rdesk_js_src)) {
    # Fallback for dev mode
    rdesk_js_src <- file.path(find.package("RDesk"), "inst", "www", "rdesk.js")
  }
  
  if (!nzchar(rdesk_js_src) || !file.exists(rdesk_js_src)) {
    warning("[RDesk] Could not find rdesk.js - copy it manually from inst/www/rdesk.js")
  } else {
    file.copy(rdesk_js_src, file.path(app_dir, "www", "js", "rdesk.js"))
  }

  message("[RDesk]   Created ", length(dirs), " directories and 8 files")
  invisible(app_dir)
}


#' @keywords internal
rdesk_write_template <- function(template_name, dest_path, vars, variant = NULL) {
  # Look for variant-specific template first, fall back to base template
  template_dir <- system.file("templates/app_skeleton", package = "RDesk")
  if (!nzchar(template_dir)) {
    # Fallback for dev mode
    template_dir <- file.path(find.package("RDesk"), "inst", "templates", "app_skeleton")
  }

  candidates <- c(
    if (!is.null(variant) && nzchar(variant))
      file.path(template_dir, paste0(template_name, ".", variant)),
    file.path(template_dir, template_name)
  )

  template_path <- candidates[file.exists(candidates)][1]
  if (is.na(template_path)) {
    # One more try - default to the non-variant file if it exists
    template_path <- file.path(template_dir, template_name)
  }

  if (!file.exists(template_path)) {
    warning("[RDesk] Template not found: ", template_name, " (variant: ", variant, ") - skipping")
    return(invisible(NULL))
  }

  content <- paste(readLines(template_path, warn = FALSE), collapse = "\n")

  # Replace all {{VAR_NAME}} placeholders
  for (nm in names(vars)) {
    val     <- vars[[nm]]
    if (is.logical(val)) {
        val <- tolower(as.character(val))
    } else {
        val <- as.character(val)
    }
    # Ensure length 1 to avoid gsub warnings/errors
    if (length(val) > 1) val <- paste(val, collapse = ", ")
    
    content <- gsub(paste0("\\{\\{", nm, "\\}\\}"), val, content)
  }

  writeLines(content, dest_path)
  invisible(dest_path)
}

library(testthat)
library(RDesk)

test_that("rdesk_create_app validates app name", {
  expect_error(rdesk_create_app("", open = FALSE), "App name is required")
  expect_error(rdesk_create_app("123App", open = FALSE), "must start with a letter")
  expect_error(rdesk_create_app("App!", open = FALSE), "must start with a letter and contain only")
})

test_that("rdesk_create_app creates directory structure", {
  withr::with_tempdir({
    app_dir <- rdesk_create_app("TestApp", data_source = "builtin", viz_type = "mixed", open = FALSE)
    
    expect_true(dir.exists(app_dir))
    expect_true(dir.exists(file.path(app_dir, "R")))
    expect_true(dir.exists(file.path(app_dir, "www")))
    expect_true(dir.exists(file.path(app_dir, "www", "css")))
    expect_true(dir.exists(file.path(app_dir, "www", "js")))
    
    expect_true(file.exists(file.path(app_dir, "app.R")))
    expect_true(file.exists(file.path(app_dir, "DESCRIPTION")))
    expect_true(file.exists(file.path(app_dir, "R", "server.R")))
    expect_true(file.exists(file.path(app_dir, "www", "index.html")))
    expect_true(file.exists(file.path(app_dir, "www", "js", "rdesk.js")))
  })
})

test_that("template placeholder replacement works", {
  withr::with_tempdir({
    app_dir <- rdesk_create_app("PlaceholderTest", open = FALSE)
    
    app_r <- readLines(file.path(app_dir, "app.R"), warn = FALSE)
    expect_true(any(grepl("PlaceholderTest", app_r)))
    expect_false(any(grepl("\\{\\{", app_r)))
    
    desc <- readLines(file.path(app_dir, "DESCRIPTION"), warn = FALSE)
    expect_true(any(grepl("DataSource: builtin", desc)))
  })
})

test_that("scaffolded app is correctly structured", {
  withr::with_tempdir({
    app_dir <- rdesk_create_app("ModernDash", open = FALSE)
    
    # Check default async backend use
    server_R <- readLines(file.path(app_dir, "R", "server.R"), warn = FALSE)
    expect_true(any(grepl("async\\(function", server_R)))
    
    # Check KPI presence
    plots_R <- readLines(file.path(app_dir, "R", "plots.R"), warn = FALSE)
    expect_true(any(grepl("get_app_kpis", plots_R)))
  })
})

test_that("duplicate directory prevention works", {
  withr::with_tempdir({
    dir.create("DuplicateApp")
    expect_error(rdesk_create_app("DuplicateApp", open = FALSE), "already exists")
  })
})

test_that("scaffolded R code is syntactically valid and runnable", {
  withr::with_tempdir({
    # 1. Create the app
    RDesk::rdesk_create_app("VerifyCode", path = ".", open = FALSE)
    app_dir <- "VerifyCode"
    
    # 2. Verify files exist
    expect_true(dir.exists(file.path(app_dir, "R")))
    expect_true(file.exists(file.path(app_dir, "R", "plots.R")))
    expect_true(file.exists(file.path(app_dir, "R", "data.R")))
    expect_true(file.exists(file.path(app_dir, "R", "server.R")))
    
    # 3. Source the files in a clean environment to check for syntax errors
    # Note: we need ggplot2 and RDesk in search path because template uses them
    library(ggplot2)
    library(RDesk)
    
    env <- new.env(parent = .GlobalEnv)
    expect_no_error(source(file.path(app_dir, "R", "data.R"), local = env))
    expect_no_error(source(file.path(app_dir, "R", "plots.R"), local = env))
    expect_no_error(source(file.path(app_dir, "R", "server.R"), local = env))
    
    # 4. Functional test of the template code
    # init_data() should return mtcars
    expect_true(exists("init_data", envir = env))
    df <- env$init_data()
    expect_s3_class(df, "data.frame")
    
    # make_chart() should return a base64 string
    expect_true(exists("make_chart", envir = env))
    chart <- env$make_chart(df)
    expect_true(is.character(chart))
    expect_true(grepl("^data:image/png;base64,", chart))
  })
})

#' @importFrom digest digest
#' @importFrom stats runif
NULL

#' Construct a standard RDesk IPC message envelope
#'
#' @param type The message type/action name
#' @param payload A list representing the message data
#' @param version The contract version (default "1.0")
#' @return A list representing the standard JSON envelope
#' @export
rdesk_message <- function(type, payload = list(), version = getOption("rdesk.ipc_version", "1.0")) {
  msg <- list(
    id = paste0("msg_", format(Sys.time(), "%s%OS3"), "_", sample.int(9999, 1)),
    type = type,
    version = version,
    payload = payload,
    timestamp = as.numeric(Sys.time())
  )

  msg_json <- jsonlite::toJSON(msg, auto_unbox = TRUE, null = "null")
  msg_size <- nchar(msg_json, type = "bytes")
  if (msg_size > 1e6) {
    warning(
      "[RDesk] Large IPC payload (",
      round(msg_size / 1e6, 1),
      " MB). Consider chunking."
    )
  }

  msg
}

#' Parse and validate an incoming RDesk IPC message
#'
#' @param raw_json The raw JSON string from the frontend
#' @return A list containing the validated message components, or NULL if invalid
#' @export
rdesk_parse_message <- function(raw_json) {
  msg <- tryCatch(jsonlite::fromJSON(raw_json, simplifyVector = FALSE), 
                  error = function(e) NULL)
  
  if (is.null(msg)) return(NULL)

  # Launcher/native events have their own schema
  if (!is.null(msg$event)) return(msg)
  
  # Structural validation
  required <- c("type", "payload")
  if (!all(required %in% names(msg))) {
    warning("[RDesk] IPC: Incoming message missing required fields: ", 
            paste(setdiff(required, names(msg)), collapse = ", "))
    return(NULL)
  }
  
  # Ensure payload is a list
  if (!is.list(msg$payload)) msg$payload <- list()
  
  msg
}

#' Create an IPC message router
#'
#' @return A list with register() and dispatch() methods
#' @keywords internal
rdesk_make_router <- function() {
  handlers <- new.env(parent = emptyenv())
  
  list(
    register = function(type, fn) {
      handlers[[type]] <- fn
    },
    dispatch = function(type, payload) {
      fn <- handlers[[type]]
      if (is.function(fn)) {
        tryCatch(fn(payload), 
                 error = function(e) warning("[RDesk] handler error for ", type, ": ", e$message))
      } else {
        # Silent ignore for unknown types (common in JS events)
      }
    }
  )
}

test_that("rdesk_message() produces correct envelope", {
  msg <- rdesk_message("test_type", list(x = 1))

  expect_named(msg, c("id", "type", "version", "payload", "timestamp"),
               ignore.order = TRUE)
  expect_equal(msg$type, "test_type")
  expect_equal(msg$version, getOption("rdesk.ipc_version", "1.0"))
  expect_equal(msg$payload, list(x = 1))
  expect_true(is.numeric(msg$timestamp))
  expect_true(grepl("^msg_", msg$id))
})

test_that("rdesk_message() generates unique IDs", {
  ids <- replicate(50, rdesk_message("t", list())$id)
  expect_equal(length(unique(ids)), 50)
})

test_that("rdesk_parse_message() returns NULL on bad JSON", {
  expect_null(rdesk_parse_message("not json"))
  expect_null(rdesk_parse_message(""))
  expect_null(rdesk_parse_message("{bad}"))
})

test_that("rdesk_parse_message() returns NULL on missing fields", {
  expect_warning(expect_null(rdesk_parse_message('{"type":"x"}')), "missing required fields")
  expect_warning(expect_null(rdesk_parse_message('{"payload":{}}')), "missing required fields")
})

test_that("rdesk_parse_message() parses valid envelope", {
  json <- jsonlite::toJSON(
    rdesk_message("ping", list(value = 42)),
    auto_unbox = TRUE
  )
  result <- rdesk_parse_message(json)
  expect_false(is.null(result))
  expect_equal(result$type, "ping")
  expect_equal(result$payload$value, 42)
})

test_that("rdesk_parse_message() coerces non-list payload to list", {
  json <- '{"type":"x","payload":"not_a_list"}'
  result <- rdesk_parse_message(json)
  expect_false(is.null(result))
  expect_true(is.list(result$payload))
})

# {{APP_NAME}} — built with RDesk {{RDESK_VER}}
# Generated: {{DATE}}

# Resolve app directory
app_dir <- getwd()

# Load RDesk - dev mode or installed
pkg_root <- dirname(dirname(dirname(app_dir)))
is_dev   <- file.exists(file.path(pkg_root, "DESCRIPTION")) &&
            file.exists(file.path(pkg_root, "R", "App.R"))

if (!nzchar(Sys.getenv("R_BUNDLE_APP")) && is_dev) {
  devtools::load_all(pkg_root, quiet = TRUE)
} else {
  library(RDesk)
}

# Source all R/ modules
lapply(
  list.files(file.path(app_dir, "R"), pattern = "\\.R$", full.names = TRUE),
  source
)

# Launch
app <- App$new(
  title  = "{{APP_TITLE}}",
  width  = 1100L,
  height = 740L,
  www    = file.path(app_dir, "www")
)

init_handlers(app)
app$run()

# inst/apps/mtcars_dashboard/R/data.R
# Data loading and KPI logic

init_data <- function(env) {
  env$df       <- mtcars %>% dplyr::mutate(model = rownames(mtcars), .before = 1)
  env$filtered <- env$df
  env$x_var    <- "wt"
  env$y_var    <- "mpg"
  env$cyl_filter <- c(4, 6, 8)
  env$plot_type  <- "scatter"
}

apply_filters <- function(env) {
  env$filtered <- env$df %>%
    dplyr::filter(cyl %in% env$cyl_filter)
}

kpis <- function(df) {
  list(
    n        = nrow(df),
    mean_mpg = round(mean(df$mpg, na.rm = TRUE), 1),
    mean_hp  = round(mean(df$hp, na.rm = TRUE),  1),
    mean_wt  = round(mean(df$wt, na.rm = TRUE) * 1000, 0)  # lbs
  )
}

summary_stats <- function(df) {
  df %>%
    dplyr::group_by(cyl) %>%
    dplyr::summarise(
      count    = dplyr::n(),
      avg_mpg  = round(mean(mpg, na.rm = TRUE), 1),
      avg_hp   = round(mean(hp, na.rm = TRUE), 1),
      avg_wt   = round(mean(wt, na.rm = TRUE), 2),
      .groups  = "drop"
    )
}

# R/window.R
# Bridge to the standalone native window process

.rdesk_window_buffers <- new.env(parent = emptyenv())
.rdesk_req_id_next <- local({
  n <- 0L
  function() {
    n <<- n + 1L
    paste0("req_", n)
  }
})

#' @keywords internal
rdesk_launcher_path <- function() {
  # Check if we are in development mode or installed
  # Source-built launcher is in bin/ after install
  path <- system.file("bin", "rdesk-launcher.exe", package = "RDesk")
  if (path == "") {
    # Fallback for dev: check inst/bin (built by Makevars or build scripts)
    path <- file.path(getwd(), "inst", "bin", "rdesk-launcher.exe")
  }
  
  if (!file.exists(path)) {
    # Check src/ for development convenience if not yet copied to inst/bin
    alt_path <- file.path(getwd(), "src", "rdesk-launcher.exe")
    if (file.exists(alt_path)) return(alt_path)

    stop("[RDesk] Native launcher not found at: ", path, 
         "\nTip: If installing from source, ensure Rtools is installed and Makevars.win executed.")
  }
  path
}

#' Open a native window pointing to a URL
#'
#' @param url The target URL to load
#' @param title Window title
#' @param width Window width
#' @param height Window height
#' @param www_path Path to the local assets directory
#' @return A processx process object
#' @keywords internal
rdesk_open_window <- function(url, title = "RDesk", width = 1200, height = 800, www_path = "") {
  launcher <- rdesk_launcher_path()

  message("[RDesk] Window opened: ", url)

  proc <- processx::process$new(
    command = launcher,
    args    = c(url, title, as.character(width), as.character(height), www_path, as.character(Sys.getpid())),
    stdin   = "|",   # allow writing QUIT and other commands
    stdout  = "|",   # pipe so we can read READY signal and events
    stderr  = "|",
    cleanup = TRUE   # kill window if R session exits
  )

  # Wait for READY signal from launcher stdout
  deadline <- Sys.time() + getOption("rdesk.launcher_timeout", 10)
  ready <- FALSE
  buffered <- character(0)
  while (Sys.time() < deadline) {
    lines <- proc$read_output_lines()
    if (length(lines) > 0) {
      trimmed <- trimws(lines)
      ready_idx <- match("READY", trimmed, nomatch = 0L)
      if (ready_idx > 0) {
        if (ready_idx > 1) {
          buffered <- c(buffered, lines[seq_len(ready_idx - 1L)])
        }
        trailing <- lines[seq.int(ready_idx + 1L, length(lines))]
        if (length(trailing) > 0) {
          buffered <- c(buffered, trailing)
        }
        ready <- TRUE
        break
      }
      buffered <- c(buffered, lines)
    }
    if (!proc$is_alive()) break
    Sys.sleep(0.05)
  }

  if (!ready) {
    err <- paste(proc$read_error_lines(), collapse = "\n")
    proc$kill()
    stop("[RDesk] Launcher failed to start correctly: ", err)
  }

  .rdesk_window_buffers[[as.character(proc$get_pid())]] <- buffered
  proc
}

#' Close the native window process
#'
#' @param proc The processx process object returned by rdesk_open_window
#' @keywords internal
rdesk_close_window <- function(proc) {
  if (is.null(proc) || !proc$is_alive()) return()
  # Send QUIT command via stdin
  rdesk_send_cmd(proc, "QUIT")
  # QUIT should be fast; this short wait is only a safety net before forcing a kill.
  proc$wait(timeout = 500)
  if (proc$is_alive()) proc$kill()
  rm(list = as.character(proc$get_pid()), envir = .rdesk_window_buffers, inherits = FALSE)
}

# ---- Command sender ----------------------------------------------------------

#' Send a JSON command to the launcher process over stdin
#'
#' @param proc Process object
#' @param cmd Command string (e.g., "QUIT", "SET_MENU")
#' @param payload Data to send as JSON
#' @param id Optional request ID for async responses
#' @keywords internal
rdesk_send_cmd <- function(proc, cmd, payload = list(), id = NULL) {
  if (is.null(proc) || !proc$is_alive()) return(invisible(NULL))
  msg        <- list(cmd = cmd, payload = payload)
  if (!is.null(id)) msg$id <- id
  line       <- jsonlite::toJSON(msg, auto_unbox = TRUE, null = "null")
  proc$write_input(paste0(line, "\n"))
  invisible(NULL)
}

#' Generate a unique request ID for dialog round-trips
#'
#' @return A character string ID
#' @keywords internal
rdesk_req_id <- function() {
  .rdesk_req_id_next()
}

# ---- Stdout event reader -----------------------------------------------------

#' Read all pending stdout lines from the launcher without blocking
#'
#' @param proc Process object
#' @return A list of parsed JSON events
#' @keywords internal
rdesk_read_events <- function(proc) {
  if (is.null(proc) || !proc$is_alive()) return(list())
  key <- as.character(proc$get_pid())
  buffered <- if (exists(key, envir = .rdesk_window_buffers, inherits = FALSE)) {
    .rdesk_window_buffers[[key]]
  } else {
    character(0)
  }
  if (exists(key, envir = .rdesk_window_buffers, inherits = FALSE)) {
    rm(list = key, envir = .rdesk_window_buffers, inherits = FALSE)
  }

  lines  <- c(buffered, proc$read_output_lines())
  events <- list()
  for (line in lines) {
    if (length(line) == 0 || is.na(line)) next
    line <- trimws(line)
    if (nchar(line) == 0 || line == "READY" || line == "CLOSED") next
    tryCatch({
      # Any valid JSON from the launcher is an "event" or "message"
      parsed <- jsonlite::fromJSON(line, simplifyVector = FALSE)
      events <- c(events, list(parsed))
    }, error = function(e) NULL)
  }
  events
}

library(testthat)
library(RDesk)

test_that("rdesk_launcher_path handles fallbacks", {
  # In this test environment, it might not find the binary, 
  # but it should at least stop() with a message containing 'not found'
  # rather than a syntax error.
  expect_type(rdesk_launcher_path(), "character")
})

test_that("rdesk_resolve_launcher_bin_dir handles source tree", {
  # Mock a source tree
  withr::with_tempdir({
    dir.create("src")
    file.create("src/rdesk-launcher.exe")
    
    bin_dir <- RDesk:::rdesk_resolve_launcher_bin_dir(getwd())
    expect_true(dir.exists(bin_dir))
    expect_true(file.exists(file.path(bin_dir, "rdesk-launcher.exe")))
    expect_true(grepl("rdesk-launcher-bin", bin_dir)) # Should be in tempdir
  })
})

test_that("internal helpers work as expected", {
  # Check rdesk_df_to_list (internal but used in templates)
  df <- data.frame(a = 1:2, b = c("x", "y"))
  res <- RDesk:::rdesk_df_to_list(df)
  expect_named(res, c("rows", "cols"))
  expect_equal(length(res$rows), 2)
  expect_equal(res$cols, c("a", "b"))
})

# R/utils.R

#' Check if the app is running in a bundled (standalone) environment
#' @return TRUE if running in a bundle, FALSE otherwise
#' @export
rdesk_is_bundle <- function() {
  # This environment variable is set by stub.cpp
  Sys.getenv("R_BUNDLE_APP") == "1"
}

#' Sanitize an app name for filesystem-safe bundled log paths
#' @keywords internal
rdesk_sanitize_log_component <- function(x) {
  x <- gsub("[^[:alnum:]_.-]+", "_", x, perl = TRUE)
  x <- trimws(x)
  if (!nzchar(x)) "RDeskApp" else x
}

#' Resolve the bundled log directory for an app
#' @keywords internal
rdesk_log_dir <- function(app_name = Sys.getenv("R_APP_NAME", "RDeskApp")) {
  # If running as a standalone bundle, use LocalAppData
  if (rdesk_is_bundle()) {
    base_dir <- Sys.getenv("LOCALAPPDATA")
    if (nzchar(base_dir)) {
      return(file.path(base_dir, "RDesk", rdesk_sanitize_log_component(app_name)))
    }
  }
  
  # Default/Fallback: Always use tempdir() per CRAN policy
  # for non-bundled or check environments
  file.path(tempdir(), "RDesk", rdesk_sanitize_log_component(app_name))
}

#' Log a message to the app's log file
#'
#' @param message Message to log
#' @param level Log level ("INFO", "WARN", "ERROR")
#' @param app_name Optional app name to determine log file
#' @keywords internal
rdesk_log <- function(message, level = "INFO", app_name = Sys.getenv("R_APP_NAME", "RDeskApp")) {
  log_dir <- rdesk_log_dir(app_name)
  if (!dir.exists(log_dir)) dir.create(log_dir, recursive = TRUE, showWarnings = FALSE)
  
  log_file <- file.path(log_dir, "app.log")
  timestamp <- format(Sys.time(), "%Y-%m-%d %H:%M:%OS3")
  line <- sprintf("[%s] [%s] %s\n", timestamp, level, message)
  
  cat(line, file = log_file, append = TRUE)
}

#' Resolve the www directory for an app
#'
#' @param www_dir User-provided path to www directory (character)
#'   Passing an explicit absolute path is the most reliable option and skips
#'   the best-effort call-stack search.
#' @return Normalized absolute path to a valid www directory
#' @keywords internal
rdesk_resolve_www <- function(www_dir) {
  # 1. Default to built-in template if NULL
  if (is.null(www_dir)) {
    path <- system.file("templates", "hello", "www", package = "RDesk")
    if (path == "" || !dir.exists(path)) {
      path <- file.path(getwd(), "inst", "templates", "hello", "www")
    }
    www_dir <- path
  }

  # 2. Ensure rdesk.js is present and up-to-date in the target www directory
  path <- normalizePath(www_dir, mustWork = FALSE)
  if (dir.exists(path)) {
    target_js <- file.path(path, "rdesk.js")
    
    # In dev mode, always copy to reflect library changes
    src_js <- system.file("www", "rdesk.js", package = "RDesk")
    if (src_js == "" || !file.exists(src_js)) {
      src_js <- file.path(getwd(), "inst", "www", "rdesk.js")
    }
    
    should_copy <- file.exists(src_js) && (
      !file.exists(target_js) ||
        !identical(unname(tools::md5sum(src_js)), unname(tools::md5sum(target_js)))
    )

    if (should_copy) {
      file.copy(src_js, target_js, overwrite = TRUE)
    }
    return(path)
  }

  # 3. Best-effort search for the calling script.
  # This relies on source() implementation details and is intentionally a fallback
  # when the caller did not provide an explicit path.
  frames <- sys.frames()
  calls <- sys.calls()
  
  # Method A: Look for 'ofile' or 'file' in frames (standard source() behavior)
  for (f in rev(frames)) {
    for (var in c("ofile", "file")) {
      if (exists(var, envir = f)) {
        val <- get(var, envir = f)
        if (is.character(val) && length(val) == 1 && file.exists(val)) {
          script_dir <- dirname(normalizePath(val))
          p <- normalizePath(file.path(script_dir, www_dir), mustWork = FALSE)
          if (dir.exists(p)) return(p)
          
          # Fallback: Is the user just saying "www" but it's in a sibling folder?
          p_alt <- normalizePath(file.path(script_dir, "www"), mustWork = FALSE)
          if (dir.exists(p_alt)) return(p_alt)
        }
      }
    }
  }

  # Method B: Regex the call stack for source("...") calls
  for (cl in rev(as.character(calls))) {
     # Use a flexible regex for source(file="...") or source("...")
     m <- regmatches(cl, regexec("source\\s*\\(\\s*(?:file\\s*=\\s*)?[\"'](.+?)[\"']", cl))
     if (length(m[[1]]) >= 2) {
        potential_script <- m[[1]][2]
        if (file.exists(potential_script)) {
           script_dir <- dirname(normalizePath(potential_script))
           p <- normalizePath(file.path(script_dir, www_dir), mustWork = FALSE)
           if (dir.exists(p)) return(p)
        }
     }
  }

  # 4. INST/APPS SCAN (Dev fallback)
  # If we provide "ts_creator" or "www", look inside the project structure
  apps_root <- file.path(getwd(), "inst", "apps")
  if (dir.exists(apps_root)) {
    # Check if www_dir IS one of the apps (e.g. App$new(www="ts_creator"))
    app_p <- file.path(apps_root, www_dir, "www")
    if (dir.exists(app_p)) return(app_p)
    
    # Recursive search for any folder named 'www' that has an index.html.
    # Refuse to guess if there is more than one candidate.
    all_wwws <- list.dirs(apps_root, recursive = TRUE)
    all_wwws <- all_wwws[basename(all_wwws) == "www"]
    all_wwws <- all_wwws[file.exists(file.path(all_wwws, "index.html"))]
    if (length(all_wwws) == 1) {
      return(all_wwws)
    }
    if (length(all_wwws) > 1) {
      stop("[RDesk] Multiple candidate www directories were found under inst/apps.\n",
           "Input provided: ", www_dir, "\n",
           "Candidates:\n  - ", paste(normalizePath(all_wwws), collapse = "\n  - "), "\n",
           "Tip: Pass an explicit absolute path to the correct www directory.")
    }
  }

  stop("[RDesk] www directory not found.\n",
       "Input provided: ", www_dir, "\n",
       "Working Directory: ", getwd(), "\n",
       "Tip: Try using an absolute path or ensure your 'www' folder is next to your script.")
}

#' Convert a data frame to a list suitable for JSON serialization
#'
#' @param df Data frame to convert
#' @return A list with 'rows' (list of lists) and 'cols' (character vector)
#' @export
rdesk_df_to_list <- function(df) {
  if (is.null(df) || nrow(df) == 0) {
    return(list(rows = list(), cols = character(0)))
  }
  list(
    rows = lapply(seq_len(nrow(df)), function(i) as.list(df[i, ])),
    cols = names(df)
  )
}

#' Convert a ggplot2 object to a base64-encoded PNG string
#'
#' @param plot A ggplot2 object
#' @param width Width in inches (default 6)
#' @param height Height in inches (default 4)
#' @param dpi DPI resolution (default 96)
#' @return A base64-encoded PNG string or a fallback error plot
#' @export
rdesk_plot_to_base64 <- function(plot, width = 6, height = 4, dpi = 96) {
  tmp <- tempfile(fileext = ".png")
  on.exit(unlink(tmp), add = TRUE)
  
  res <- tryCatch({
    ggplot2::ggsave(tmp, plot = plot, width = width, height = height, dpi = dpi)
    TRUE
  }, error = function(e) {
    warning("[RDesk] Failed to save plot: ", e$message)
    FALSE
  })
  
  if (isTRUE(res) && file.exists(tmp)) {
    raw <- readBin(tmp, "raw", file.info(tmp)$size)
    return(paste0("data:image/png;base64,", base64enc::base64encode(raw)))
  }
  
  # Fallback to error plot
  rdesk_error_plot("Plot generation failed")
}

#' Generate a base64-encoded error plot
#'
#' @param message Error message to display (optional)
#' @return A base64-encoded PNG string
#' @export
rdesk_error_plot <- function(message = "Error generating plot") {
  tmp <- tempfile(fileext = ".png")
  on.exit(unlink(tmp), add = TRUE)
  
  p <- ggplot2::ggplot() +
    ggplot2::annotate("text", x = 1, y = 1, label = message, color = "red", size = 5) +
    ggplot2::theme_void()
    
  ggplot2::ggsave(tmp, plot = p, width = 4, height = 2, dpi = 72)
  
  if (file.exists(tmp)) {
    raw <- readBin(tmp, "raw", file.info(tmp)$size)
    return(paste0("data:image/png;base64,", base64enc::base64encode(raw)))
  }
  NULL
}

#' Parse a hotkey string into modifiers and virtual key codes
#' @param keys String like "Ctrl+Shift+A" or "Alt+F4"
#' @return List with modifiers and vk
#' @keywords internal
rdesk_parse_hotkey <- function(keys) {
  parts <- trimws(tolower(strsplit(keys, "[+ ]")[[1]]))
  mod <- 0
  vk  <- 0
  
  if ("alt"   %in% parts) mod <- mod + 1
  if ("ctrl"  %in% parts || "control" %in% parts) mod <- mod + 2
  if ("shift" %in% parts) mod <- mod + 4
  if ("win"   %in% parts) mod <- mod + 8
  
  # Remove modifiers to find the main key
  main <- setdiff(parts, c("alt", "ctrl", "control", "shift", "win"))
  if (length(main) == 0) return(list(modifiers = mod, vk = 0))
  
  key <- main[1]
  if (nchar(key) == 1) {
    vk <- as.integer(charToRaw(toupper(key)))
  } else if (grepl("^f[0-9]+$", key)) {
    f_num <- as.integer(substring(key, 2))
    vk <- 0x70 + (f_num - 1) # F1 is 0x70
  } else {
    # Common special keys
    vk <- switch(key,
      "space"  = 0x20,
      "enter"  = 0x0D,
      "return" = 0x0D,
      "escape" = 0x1B,
      "tab"    = 0x09,
      "backspace" = 0x08,
      "delete" = 0x2E,
      "insert" = 0x2D,
      "home"   = 0x24,
      "end"    = 0x23,
      "pageup" = 0x21,
      "pagedown" = 0x22,
      "left"   = 0x25,
      "up"     = 0x26,
      "right"  = 0x27,
      "down"   = 0x28,
      0
    )
  }
  
  list(modifiers = mod, vk = vk)
}

library(testthat)
library(RDesk)

test_that("rdesk_plot_to_base64 handles basic ggplot", {
  p <- ggplot2::ggplot(mtcars, ggplot2::aes(x = mpg, y = wt)) + ggplot2::geom_point()
  b64 <- rdesk_plot_to_base64(p)
  
  expect_true(is.character(b64))
  expect_true(grepl("^data:image/png;base64,", b64))
})

test_that("rdesk_plot_to_base64 falls back to error plot on failure", {
  p <- "not a plot"
  
  # Should still return a base64 string because it fails over to rdesk_error_plot
  expect_warning(b64 <- rdesk_plot_to_base64(p), "Failed to save plot")
  expect_true(is.character(b64))
  expect_true(grepl("^data:image/png;base64,", b64))
})

test_that("rdesk_error_plot generates valid base64", {
  b64 <- rdesk_error_plot("Custom Error")
  expect_true(is.character(b64))
  expect_true(grepl("^data:image/png;base64,", b64))
})

# Manual performance test — run locally, not in CI
devtools::load_all(".")

# Simple task function
task_fn <- function(x) {
  # Simulate very light work
  x * 2
}

message("\n--- RDesk Async Performance Benchmark ---")
message("Testing 5 sequential tasks to measure startup overhead difference.\n")

# 1. Benchmark callr
message("[1/2] Benchmarking 'callr' backend (spawning per task)...")
options(rdesk.async_backend = "callr")
t_callr <- system.time({
  for (i in 1:5) {
    done <- FALSE
    rdesk_async(task_fn, args = list(x = i),
      on_done = function(r) { done <<- TRUE },
      on_error = function(e) { message("Error: ", e$message); done <<- TRUE }
    )
    # Busy wait for polling (simulating the app event loop)
    while (!done) { 
      rdesk_poll_jobs()
      Sys.sleep(0.01) 
    }
  }
})

# 2. Benchmark mirai
message("[2/2] Benchmarking 'mirai' backend (persistent daemon pool)...")
options(rdesk.async_backend = "mirai")
# Manually start daemons for the test (usually handled by App$run)
mirai::daemons(2) 
t_mirai <- system.time({
  for (i in 1:5) {
    done <- FALSE
    rdesk_async(task_fn, args = list(x = i),
      on_done = function(r) { done <<- TRUE },
      on_error = function(e) { message("Error: ", e$message); done <<- TRUE }
    )
    while (!done) { 
      rdesk_poll_jobs()
      Sys.sleep(0.01) 
    }
  }
})
mirai::daemons(0) # Shutdown daemons

# Results
callr_elapsed <- t_callr["elapsed"]
mirai_elapsed <- t_mirai["elapsed"]
speedup       <- callr_elapsed / mirai_elapsed

message("\n--- RESULTS ---")
message("callr 5 tasks: ", round(callr_elapsed, 2), "s")
message("mirai 5 tasks: ", round(mirai_elapsed, 2), "s")
message("Startup Speedup: ", round(speedup, 1), "x")
message("------------------------------------------\n")

if (speedup > 2) {
  message("SUCCESS: mirai significantly reduces task latency.")
} else {
  message("NOTE: Speedup less than 2x, check system load or config.")
}

# R/App.R
# RDesk App R6 class - the complete public API
 
# Global registry for multi-window management
.rdesk_apps <- new.env(parent = emptyenv())
 
#' @importFrom R6 R6Class
NULL

#' Create and launch a native desktop application window from R.
#'
#' @description
#' Provides bidirectional native pipe communication between R and the UI.
#'
#' @examples
#' # Safe logical check (unwrapped)
#' app_dir <- system.file("templates/hello", package = "RDesk")
#' if (nzchar(app_dir)) {
#'   message("Built-in app directory: ", app_dir)
#' }
#' 
#' if (interactive()) {
#'   app <- App$new(title = "Car Visualizer", width = 1200, height = 800)
#'   
#'   app$on_ready(function() {
#'     message("App is ready!")
#'   })
#'   
#'   # Handle messages from UI
#'   app$on_message("get_data", function(payload) {
#'     list(cars = mtcars[1:5, ])
#'   })
#'   
#'   # Start the app
#'   app$run()
#' }
#' @export
App <- R6::R6Class("App",
 
  public = list(
 
    #' @description Create a new RDesk application
    #' @param title Window title string
    #' @param width Window width in pixels (default 1200)
    #' @param height Window height in pixels (default 800)
    #' @param www Directory containing HTML/CSS/JS assets (default: built-in template)
    #' @param icon Path to window icon file
    #' @return A new App instance
    initialize = function(title,
                          width  = 1200L,
                          height = 800L,
                          www    = NULL,
                          icon   = NULL) {
      if (missing(title)) stop("Title is mandatory")
      if (missing(width)) stop("Width is mandatory")
      if (missing(height)) stop("Height is mandatory")

      private$.title  <- title
      private$.width  <- as.integer(width)
      private$.height <- as.integer(height)
      private$.www    <- rdesk_resolve_www(www)
      private$.icon   <- icon
      private$.router <- rdesk_make_router()
      private$.id     <- paste0("app_", digest::digest(list(
        Sys.time(),
        proc.time(),
        runif(1)
      ), algo = "crc32"))
      private$.hotkey_callbacks <- new.env(parent = emptyenv())

      # System handler for UI-initiated job cancellation
      private$.router$register("__cancel_job__", function(payload) {
        if (!is.null(payload$job_id)) {
          rdesk_cancel_job(payload$job_id)
          self$loading_done()
          self$toast("Operation cancelled.", type = "warning")
        }
      })
    },

    #' @description Register a callback to fire when the window is ready
    #' @param fn A zero-argument function called after the server starts and window opens
    #' @return The App instance (invisible)
    on_ready = function(fn) {
      if (!is.function(fn)) stop("on_ready() requires a function")
      private$.ready_fn <- fn
      invisible(self)
    },

    #' @description Register a callback to fire when the user attempts to close the window
    #' @param fn A zero-argument function. Should return TRUE to allow closing, FALSE to cancel.
    #' @return The App instance (invisible)
    on_close = function(fn) {
      if (!is.function(fn)) stop("on_close() requires a function")
      private$.on_close_fn <- fn
      # Tell the launcher to start intercepting WM_CLOSE
      private$.send_launcher_cmd("INTERCEPT_CLOSE", list(enabled = TRUE))
      invisible(self)
    },

    #' @description Check for application updates from a remote URL
    #' @param version_url URL to a JSON metadata file (e.g. \verb{\{"version": "1.1.0", "url": "http://..."\}})
    #' @param current_version Optional version string to compare against. Defaults to app description version.
    #' @return A list with update status and metadata
    check_update = function(version_url, current_version = NULL) {
      if (is.null(current_version)) {
        desc_path <- file.path(self$get_dir(), "DESCRIPTION")
        if (file.exists(desc_path)) {
          desc <- read.dcf(desc_path)
          current_version <- as.character(desc[1, "Version"])
        } else {
          current_version <- "1.0.0"
        }
      }

      tmp <- tempfile(fileext = ".json")
      on.exit(unlink(tmp), add = TRUE)

      tryCatch({
        utils::download.file(version_url, tmp, mode = "wb", quiet = TRUE)
        latest <- jsonlite::fromJSON(tmp)

        has_update <- utils::compareVersion(latest$version, current_version) > 0

        list(
          update_available = has_update,
          current_version  = current_version,
          latest_version   = latest$version,
          download_url     = latest$url
        )
      }, error = function(e) {
        warning("[RDesk] Update check failed: ", e$message)
        list(update_available = FALSE, error = e$message)
      })
    },

    #' @description Register a global keyboard shortcut (hotkey)
    #' @param keys Character string representing the key combination (e.g., "Ctrl+Shift+A")
    #' @param fn A zero-argument function to be called when the hotkey is pressed
    #' @return The App instance (invisible)
    register_hotkey = function(keys, fn) {
      if (!is.character(keys) || length(keys) != 1) stop("keys must be a single string")
      if (!is.function(fn)) stop("fn must be a function")

      parsed <- rdesk_parse_hotkey(keys)
      if (is.null(parsed) || parsed$vk == 0) stop("Invalid hotkey format: ", keys)

      hotkey_id <- as.integer(sample.int(9999, 1))
      private$.hotkey_callbacks[[as.character(hotkey_id)]] <- fn
      
      private$.send_launcher_cmd("REGISTER_HOTKEY", payload = list(
        id = hotkey_id,
        modifiers = parsed$modifiers,
        vk = parsed$vk,
        label = keys
      ))
      invisible(self)
    },

    #' @description Set the native system tray menu
    #' @param items A named list of lists defining the menu structure
    #' @return The App instance (invisible)
    set_tray_menu = function(items) {
      # Convert R named list to JSON array the launcher understands
      menu_json <- private$.build_menu_json(items, prefix = "tray_menu")

      private$.send_launcher_cmd("SET_TRAY_MENU", payload = menu_json, queue_if_unavailable = TRUE)

      invisible(self)
    },

    #' @description Write text to the system clipboard
    #' @param text Character string to copy
    #' @return The App instance (invisible)
    clipboard_write = function(text) {
      private$.send_launcher_cmd("CLIPBOARD_WRITE", payload = list(text = as.character(text)))
      invisible(self)
    },

    #' @description Read text from the system clipboard
    #' @return Character string from clipboard or NULL
    clipboard_read = function() {
      req_id <- rdesk_req_id()
      private$.send_launcher_cmd("CLIPBOARD_READ", id = req_id)
      private$.wait_dialog_result(req_id)
    },

    #' @description Register a handler for a UI -> R message type
    #' @param type Unique message identifier string
    #' @param fn A function(payload) called when this message type arrives
    #' @return The App instance (invisible)
    on_message = function(type, fn) {
      if (!is.character(type) || length(type) != 1) stop("type must be a single string")
      if (!is.function(fn)) stop("fn must be a function")

      # If fn is an async() wrapper, inject the message type
      # so it can auto-route results as <type>_result
      if (isTRUE(attr(fn, "rdesk_async_wrapper"))) {
        type_env <- attr(fn, "rdesk_msg_type_env")
        if (!is.null(type_env)) type_env$type <- type
      }

      private$.router$register(type, fn)
      invisible(self)
    },

    #' @description Send a message from R to the UI
    #' @param type Character string message type (received by rdesk.on() in JS)
    #' @param payload A list or data.frame to serialise as JSON payload
    #' @return The App instance (invisible)
    send = function(type, payload = list()) {
      # Construct the standard envelope
      msg_envelope <- rdesk_message(type, payload)
      msg_json <- jsonlite::toJSON(msg_envelope, auto_unbox = TRUE)

      if (!is.null(private$.window_proc) && private$.window_proc$is_alive()) {
        # Use internal command SEND_MSG to bridge to PostWebMessage
        rdesk_send_cmd(private$.window_proc, "SEND_MSG", payload = msg_json)
      } else if (rdesk_is_bundle()) {
        # Legacy fallback if a bundled app is ever hosted by an external launcher
        cat(msg_json, "\n", sep = "")
        flush(stdout())
      } else {
        # Queue message if launcher not yet ready
        private$.send_queue[[length(private$.send_queue) + 1]] <- msg_envelope
      }
      invisible(self)
    },

    #' @description Load an HTML file into the window
    #' @param path Path relative to the www directory (e.g. "index.html")
    #' @return The App instance (invisible)
    load_ui = function(path = "index.html") {
      self$send("__navigate__", list(path = path))
      invisible(self)
    },

    #' @description Set the window size dynamically
    #' @param width New width (pixels)
    #' @param height New height (pixels)
    #' @return The App instance (invisible)
    set_size = function(width, height) {
      private$.send_launcher_cmd("SET_SIZE", list(width = as.integer(width), height = as.integer(height)))
      private$.width  <- as.integer(width)
      private$.height <- as.integer(height)
      invisible(self)
    },

    #' @description Set the window position dynamically
    #' @param x Horizontal position from left (pixels)
    #' @param y Vertical position from top (pixels)
    #' @return The App instance (invisible)
    set_position = function(x, y) {
      private$.send_launcher_cmd("SET_POS", list(x = as.integer(x), y = as.integer(y)))
      invisible(self)
    },

    #' @description Set the window title dynamically
    #' @param title New title
    #' @return The App instance (invisible)
    set_title = function(title) {
      private$.send_launcher_cmd("SET_TITLE", list(title = as.character(title)))
      private$.title <- as.character(title)
      invisible(self)
    },

    #' @description Minimize the window to the taskbar
    #' @return The App instance (invisible)
    minimize = function() {
      private$.send_launcher_cmd("MINIMIZE")
      invisible(self)
    },

    #' @description Maximize the window to fill the screen
    #' @return The App instance (invisible)
    maximize = function() {
      private$.send_launcher_cmd("MAXIMIZE")
      invisible(self)
    },

    #' @description Restore the window from minimize/maximize
    #' @return The App instance (invisible)
    restore = function() {
      private$.send_launcher_cmd("RESTORE")
      invisible(self)
    },

    #' @description Toggle fullscreen mode
    #' @param enabled If TRUE, enters fullscreen. If FALSE, exits.
    #' @return The App instance (invisible)
    fullscreen = function(enabled = TRUE) {
      private$.send_launcher_cmd("FULLSCREEN", list(enabled = isTRUE(enabled)))
      invisible(self)
    },

    #' @description Set the window to stay always on top of others
    #' @param enabled If TRUE, always on top.
    #' @return The App instance (invisible)
    always_on_top = function(enabled = TRUE) {
      private$.send_launcher_cmd("TOPMOST", list(enabled = isTRUE(enabled)))
      invisible(self)
    },

    #' @description Set the native window menu
    #' @param items A named list of lists defining the menu structure
    #' @return The App instance (invisible)
    set_menu = function(items) {
      # Convert R named list to JSON array the launcher understands
      menu_json <- private$.build_menu_json(items)

      private$.send_launcher_cmd("SET_MENU", payload = menu_json, queue_if_unavailable = TRUE)

      private$.menu_callbacks <- items
      invisible(self)
    },

    #' @description Open a native file-open dialog
    #' @param title Dialog title
    #' @param filters List of file filters, e.g. list("CSV files" = "*.csv")
    #' @return Selected file path (character) or NULL if cancelled
    dialog_open = function(title = "Open File", filters = NULL) {
      filter_str <- private$.build_filter_str(filters)
      req_id     <- rdesk_req_id()
      private$.send_launcher_cmd(
        "DIALOG_OPEN",
        payload = list(title = title, filters = filter_str),
        id = req_id
      )
      private$.wait_dialog_result(req_id)
    },

    #' @description Open a native file-save dialog
    #' @param title Dialog title
    #' @param default_name Initial filename
    #' @param filters List of file filters
    #' @return Selected file path (character) or NULL if cancelled
    dialog_save = function(title = "Save File", default_name = "",
                            filters = NULL) {
      filter_str <- private$.build_filter_str(filters)

      # Extract default extension (e.g. "csv" from "*.csv")
      def_ext <- NULL
      if (!is.null(filters) && length(filters) > 0) {
        f <- filters[[1]]
        def_ext <- gsub("^.*\\.", "", f)
      }

      req_id     <- rdesk_req_id()
      private$.send_launcher_cmd(
        "DIALOG_SAVE",
        payload = list(title        = title,
                       default_name = default_name,
                       filters      = filter_str,
                       default_ext  = def_ext),
        id = req_id
      )
      private$.wait_dialog_result(req_id)
    },

    #' @description Open a native folder selection dialog
    #' @param title Dialog title
    #' @return Selected directory path (character) or NULL if cancelled
    dialog_folder = function(title = "Select Folder") {
      req_id <- rdesk_req_id()
      private$.send_launcher_cmd(
        "DIALOG_FOLDER",
        payload = list(title = title),
        id = req_id
      )
      private$.wait_dialog_result(req_id)
    },

    #' @description Show a native message box / alert
    #' @param message The message text
    #' @param title The dialog title
    #' @param type One of "ok", "okcancel", "yesno", "yesnocancel"
    #' @param icon One of "info", "warning", "error", "question"
    #' @return The button pressed (character: "ok", "cancel", "yes", "no")
    message_box = function(message, title = "RDesk", type = "ok", icon = "info") {
      req_id <- rdesk_req_id()
      private$.send_launcher_cmd(
        "MESSAGE_BOX",
        payload = list(message = message, title = title, type = type, icon = icon),
        id = req_id
      )
      private$.wait_dialog_result(req_id)
    },

    #' @description Open a native color selection dialog
    #' @param initial_color Optional hex color to start with (e.g. "#FF0000")
    #' @return Selected hex color code or NULL if cancelled
    dialog_color = function(initial_color = "#FFFFFF") {
      req_id <- rdesk_req_id()
      private$.send_launcher_cmd(
        "DIALOG_COLOR",
        payload = list(color = initial_color),
        id = req_id
      )
      private$.wait_dialog_result(req_id)
    },

    #' @description Send a native desktop notification
    #' @param title Notification title
    #' @param body Notification body text
    #' @return The App instance (invisible)
    notify = function(title, body = "") {
      private$.send_launcher_cmd(
        "NOTIFY",
        payload = list(title = title, body = body),
        queue_if_unavailable = TRUE
      )
      invisible(self)
    },

    #' @description Show a loading state in the UI
    #' @param message Text shown under the spinner
    #' @param progress Optional numeric 0-100 for a progress bar
    #' @param cancellable If TRUE, shows a cancel button in the UI
    #' @param job_id Optional job_id from rdesk_async() to wire cancel button
    loading_start = function(message     = "Loading...",
                             progress    = NULL,
                             cancellable = FALSE,
                             job_id      = NULL) {
      self$send("__loading__", list(
        active      = TRUE,
        message     = message,
        progress    = progress,
        cancellable = cancellable,
        job_id      = job_id
      ))
      invisible(self)
    },

    #' @description Update progress on an active loading state
    #' @param value Numeric 0-100
    #' @param message Optional updated message
    loading_progress = function(value, message = NULL) {
      payload <- list(active = TRUE, progress = value)
      if (!is.null(message)) payload$message <- message
      self$send("__loading__", payload)
      invisible(self)
    },

    #' @description Hide the loading state in the UI
    loading_done = function() {
      self$send("__loading__", list(active = FALSE, message = "", progress = NULL))
      invisible(self)
    },

    #' @description Show a non-blocking toast notification in the UI
    #' @param message Text to show
    #' @param type One of "info", "success", "warning", "error"
    #' @param duration_ms How long to show it (default 3000ms)
    toast = function(message, type = "info", duration_ms = 3000L) {
      self$send("__toast__", list(
        message     = message,
        type        = type,
        duration_ms = as.integer(duration_ms)
      ))
      invisible(self)
    },

    #' @description Set or update the system tray icon
    #' @param label Tooltip text for the tray icon
    #' @param icon Path to .ico file (optional)
    #' @param on_click Character "left" or "right" or callback function(button)
    #' @return The App instance (invisible)
    set_tray = function(label = "RDesk App", icon = NULL, on_click = NULL) {
      private$.tray_callback <- on_click
      private$.send_launcher_cmd(
        "SET_TRAY",
        payload = list(label = label, icon = icon),
        queue_if_unavailable = TRUE
      )
      invisible(self)
    },

    #' @description Remove the system tray icon
    #' @return The App instance (invisible)
    remove_tray = function() {
      private$.tray_callback <- NULL
      private$.send_launcher_cmd("REMOVE_TRAY", queue_if_unavailable = TRUE)
      invisible(self)
    },

    #' @description Service this app's pending native events
    #' @return The App instance (invisible)
    service = function() {
      private$.poll_events()
      invisible(self)
    },

    #' @description Close the window and stop the app's event loop.
    #' @return The App instance (invisible)
    quit = function() {
      private$.running <- FALSE
      # Also remove from global registry if present
      if (exists(as.character(private$.id), envir = .rdesk_apps)) {
        rm(list = as.character(private$.id), envir = .rdesk_apps)
      }
      invisible(self)
    },

    #' @description Get the application root directory (where www/ and R/ are located).
    #' @return Character string path.
    get_dir = function() {
      dirname(private$.www)
    },

    #' @description Start the application - opens the window
    #' @param block If TRUE (default), blocks with an event loop until the window is closed.
    run = function(block = TRUE) {
      # CI Guard: Skip initialization if running in a headless environment
      if (getOption("rdesk.ci_mode", FALSE)) {
        message("[RDesk] CI Mode: Skipping native window initialization.")
        return(invisible(self))
      }

      rdesk_log(sprintf("Starting application: %s", private$.title), level = "INFO", app_name = private$.title)

      private$.running <- TRUE
      if (getOption("rdesk.async_backend", "callr") == "mirai") {
        rdesk_start_daemons()  # Pre-warm worker pool
      }

      url <- "https://app.rdesk/index.html"

      private$.window_proc <- rdesk_open_window(
        url      = url,
        title    = private$.title,
        width    = private$.width,
        height   = private$.height,
        www_path = private$.www
      )

      # Flush queued messages
      private$.flush_send_queue()
      private$.flush_command_queue()

      if (!is.null(private$.ready_fn)) {
        tryCatch(private$.ready_fn(), error = function(e) {
          rdesk_log(sprintf("Startup Error (on_ready): %s", e$message), level = "ERROR", app_name = private$.title)
          warning("[RDesk] on_ready error: ", e$message)
        })
      }

      assign(as.character(private$.id), self, envir = .rdesk_apps)

      if (block) {
        on.exit(private$.cleanup())
        tryCatch({
          while (private$.running) {
            rdesk_service()
            if (!private$.running) break
            Sys.sleep(0.01)
          }
        }, error = function(e) {
          rdesk_log(sprintf("Application CRASHED: %s", e$message), level = "ERROR", app_name = private$.title)
          warning("[RDesk] Fatal error: ", e$message)
          private$.running <- FALSE
        })
        message("[RDesk] App closed.")
      }

      invisible(self)
    }
  ),

  private = list(
    .id          = NULL,
    .title       = NULL,
    .width       = NULL,
    .height      = NULL,
    .www         = NULL,
    .icon        = NULL,
    .ready_fn    = NULL,
    .on_close_fn = NULL,
    .running     = FALSE,
    .window_proc = NULL,
    .router      = NULL,
    .send_queue  = list(),
    .command_queue = list(),
    .menu_actions  = new.env(parent = emptyenv()), # Stores the action ID -> function mapping for menus and tray menus
    .pending_dialogs = list(),  # req_id -> result or NULL
    .tray_callback = NULL,      # Function(button)
    .hotkey_callbacks = NULL,
    .bundle_con = NULL,         # Non-blocking stdin connection in hosted bundle mode

    .cleanup = function() {
      if (getOption("rdesk.async_backend", "callr") == "mirai") {
        rdesk_stop_daemons()  # Shut down worker pool cleanly
      }
      if (!is.null(private$.window_proc)) {
        rdesk_close_window(private$.window_proc)
        private$.window_proc <- NULL
      }
      rdesk_log(sprintf("Application closed: %s", private$.title), level = "INFO", app_name = private$.title)
      private$.bundle_con <- NULL
      private$.running <- FALSE
    },

    .send_launcher_cmd = function(cmd, payload = list(), id = NULL, queue_if_unavailable = FALSE) {
      if (!is.null(private$.window_proc) && private$.window_proc$is_alive()) {
        rdesk_send_cmd(private$.window_proc, cmd, payload = payload, id = id)
        return(invisible(TRUE))
      }

      msg <- list(cmd = cmd, payload = payload)
      if (!is.null(id)) msg$id <- id

      if (rdesk_is_bundle()) {
        cat(jsonlite::toJSON(msg, auto_unbox = TRUE, null = "null"), "\n", sep = "")
        flush(stdout())
        return(invisible(TRUE))
      }

      if (isTRUE(queue_if_unavailable)) {
        private$.command_queue[[length(private$.command_queue) + 1]] <- msg
        return(invisible(TRUE))
      }

      stop("[RDesk] Launcher is not available for command: ", cmd)
    },

    .flush_send_queue = function() {
      if (length(private$.send_queue) == 0) return(invisible(NULL))
      for (msg_envelope in private$.send_queue) {
        msg_json <- jsonlite::toJSON(msg_envelope, auto_unbox = TRUE)
        if (!is.null(private$.window_proc) && private$.window_proc$is_alive()) {
          rdesk_send_cmd(private$.window_proc, "SEND_MSG", payload = msg_json)
        } else if (rdesk_is_bundle()) {
          cat(msg_json, "\n", sep = "")
          flush(stdout())
        }
      }
      private$.send_queue <- list()
      invisible(NULL)
    },

    .flush_command_queue = function() {
      if (length(private$.command_queue) == 0) return(invisible(NULL))
      pending <- private$.command_queue
      private$.command_queue <- list()
      for (cmd_msg in pending) {
        private$.send_launcher_cmd(
          cmd = cmd_msg$cmd,
          payload = if (is.null(cmd_msg$payload)) list() else cmd_msg$payload,
          id = cmd_msg$id
        )
      }
      invisible(NULL)
    },

    .ensure_bundle_conn = function() {
      if (is.null(private$.bundle_con)) {
        private$.bundle_con <- processx::conn_create_fd(0L, encoding = "")
      }
      private$.bundle_con
    },

    .poll_bundle_input = function() {
      if (!rdesk_is_bundle()) return(invisible(NULL))
      con <- private$.ensure_bundle_conn()
      lines <- tryCatch(
        processx::conn_read_lines(con, n = 100, timeout = 0),
        error = function(e) character(0)
      )
      if (length(lines) == 0) return(invisible(NULL))

      for (line in lines) {
        msg <- rdesk_parse_message(line)
        if (!is.null(msg)) {
          if (!is.null(msg$event)) private$.handle_launcher_event(msg)
          else private$.router$dispatch(msg$type, msg$payload)
        }
      }
      invisible(NULL)
    },
 
    .build_menu_json = function(items, prefix = "menu") {
      # Use an environment to ensure we don't clear existing actions from other menus (e.g. tray vs bar)
      if (is.null(private$.menu_actions)) private$.menu_actions <- new.env(parent = emptyenv())
      
      recurse <- function(it, parent_label = prefix) {
        res <- list()
        for (i in seq_along(it)) {
          val  <- it[[i]]
          lbl  <- if (is.null(names(it)) || names(it)[i] == "") as.character(val) else names(it)[i]
          
          # Support both simple list("L"=fn) and detailed list("L"=list(checked=T, callback=fn))
          item_callback <- val
          item_checked  <- FALSE
          is_submenu    <- is.list(val) && !is.function(val) && ("items" %in% names(val))
          is_detailed   <- is.list(val) && !is.function(val) && (any(c("callback", "checked") %in% names(val)))

          if (lbl == "---" || identical(val, "---")) {
            res <- c(res, list(list(label = "---")))
          } else if (is_submenu) {
            # Submenu with items
            sub_items <- if (is.list(val$items)) val$items else val
            res <- c(res, list(list(label = lbl, items = recurse(sub_items, paste0(parent_label, "_", lbl)))))
          } else if (is.list(val) && !is.function(val) && !is_detailed) {
            # Default to treat any list without 'callback'/'checked' as a submenu (backward compat)
            res <- c(res, list(list(label = lbl, items = recurse(val, paste0(parent_label, "_", lbl)))))
          } else {
            # Menu item
            if (is_detailed) {
               item_callback <- val$callback
               item_checked  <- isTRUE(val$checked)
            }
            item_id <- paste0(parent_label, "_", i, "_", digest::digest(lbl, algo="crc32"))
            private$.menu_actions[[item_id]] <- item_callback
            res <- c(res, list(list(label = lbl, id = item_id, checked = item_checked)))
          }
        }
        res
      }

      # Top level is special - it's a list of top-level menus
      final <- list()
      for (top_label in names(items)) {
        sub_items <- items[[top_label]]
        final <- c(final, list(list(label = top_label, items = recurse(sub_items, paste0(prefix, "_", top_label)))))
      }
      final
    },
 
    .build_filter_str = function(filters) {
      # Convert list("CSV Files"="*.csv") to
      # "CSV Files|*.csv|All Files|*.*|" (launcher converts | to \0)
      if (is.null(filters)) return("All Files|*.*|")
      parts <- character(0)
      for (nm in names(filters)) {
        parts <- c(parts, nm, filters[[nm]])
      }
      parts <- c(parts, "All Files", "*.*")
      paste0(paste(parts, collapse = "|"), "||")
    },
 
    .wait_dialog_result = function(req_id, timeout_sec = 60) {
      private$.pending_dialogs[[req_id]] <- NULL
      deadline <- Sys.time() + timeout_sec
      while (Sys.time() < deadline) {
        if (!is.null(private$.window_proc) && private$.window_proc$is_alive()) {
          events <- rdesk_read_events(private$.window_proc)
          for (evt in events) private$.handle_launcher_event(evt)
        } else if (rdesk_is_bundle()) {
          private$.poll_bundle_input()
        }
        
        result <- private$.pending_dialogs[[req_id]]
        if (!is.null(result)) {
          private$.pending_dialogs[[req_id]] <- NULL
          return(if (result == "__CANCEL__") NULL else result)
        }
        Sys.sleep(0.05)
      }
      NULL  # timeout
    },
 
    .handle_launcher_event = function(evt) {
      if (!is.null(evt$event)) {
        if (evt$event == "MENU_CLICK") {
          callback <- private$.menu_actions[[evt$id]]
          if (is.function(callback)) {
            tryCatch(callback(),
                     error = function(e) warning("[RDesk] menu handler error: ", e$message))
          }
        } else if (evt$event == "DIALOG_RESULT") {
          private$.pending_dialogs[[evt$id]] <- if (!is.null(evt$path)) evt$path else evt$result
        } else if (evt$event == "DIALOG_CANCEL") {
          private$.pending_dialogs[[evt$id]] <- "__CANCEL__"
        } else if (evt$event == "TRAY_CLICK") {
          if (is.function(private$.tray_callback)) {
            tryCatch(private$.tray_callback(evt$button),
                     error = function(e) warning("[RDesk] tray handler error: ", e$message))
          }
        } else if (evt$event == "WINDOW_CLOSING") {
          res <- TRUE
          if (is.function(private$.on_close_fn)) {
            res <- tryCatch(private$.on_close_fn(), error = function(e) {
              warning("[RDesk] on_close error: ", e$message)
              TRUE 
            })
          }
          if (isTRUE(res)) {
            self$quit()
          }
        } else if (evt$event == "HOTKEY") {
          callback <- private$.hotkey_callbacks[[as.character(evt$id)]]
          if (is.function(callback)) {
            tryCatch(callback(), error = function(e) warning("[RDesk] hotkey handler error: ", e$message))
          }
        }
      } else if (!is.null(evt$type)) {
        # It's a JS -> R message forwarded by launcher
        private$.router$dispatch(evt$type, evt$payload)
      }
    },
 
    .poll_events = function() {
      if (!is.null(private$.window_proc)) {
        events <- rdesk_read_events(private$.window_proc)
        for (evt in events) {
          private$.handle_launcher_event(evt)
        }
        if (!private$.window_proc$is_alive()) {
          self$quit()
        }
      }
    }
  )
)
 
#' Service all active RDesk applications
#' 
#' Processes native OS events for all open windows.
#' Call this periodically if you are running apps with \code{block = FALSE}.
#' 
#' @export
rdesk_service <- function() {
  # 1. Poll any background jobs
  rdesk_poll_jobs()

  # 2. Service each registered app
  app_ids <- ls(.rdesk_apps)
  for (id in app_ids) {
    app <- .rdesk_apps[[id]]
    app$service()
  }
}#' Automatically check for and install app updates
#'
#' @description
#' \code{rdesk_auto_update} is a high-level function designed for bundled (standalone)
#' applications. It checks a remote version string, compares it with the current
#' version, and if a newer version is found, it downloads and executes the installer
#' silently before quitting the current application.
#'
#' @param version_url URL to a plain text file containing the latest version string (e.g., "1.1.0")
#' @param download_url URL to the latest installer .exe
#' @param current_version Current app version string e.g. "1.0.0"
#' @param silent If TRUE, downloads and installs without prompting. Default FALSE.
#' @param app Optional App instance for showing toast notifications.
#' @return Invisible TRUE if update was applied, FALSE otherwise.
#' @export
rdesk_auto_update <- function(version_url,
                               download_url,
                               current_version,
                               silent  = FALSE,
                               app     = NULL) {
  if (!rdesk_is_bundle()) {
    # Only acts in bundled mode
    return(invisible(FALSE))
  }

  latest <- tryCatch({
    con <- url(version_url)
    on.exit(close(con))
    trimws(readLines(con, n = 1, warn = FALSE))
  }, error = function(e) {
    warning("[RDesk] Could not fetch remote version: ", e$message)
    return(invisible(NULL))
  })

  if (is.null(latest) || !nzchar(latest)) {
    return(invisible(FALSE))
  }

  if (utils::compareVersion(latest, current_version) <= 0) {
    # No update needed
    return(invisible(FALSE))
  }

  # Update available
  if (!is.null(app) && !silent) {
    app$toast(
      paste0("Version ", latest, " is available. Downloading..."),
      type = "info"
    )
  }

  # Temporary path for the new installer
  dest <- file.path(tempdir(), paste0("update-", latest, "-setup.exe"))

  tryCatch({
    message("[RDesk] Downloading update version ", latest, "...")
    utils::download.file(download_url, dest, mode = "wb", quiet = TRUE)

    if (!is.null(app) && !silent) {
      app$toast("Update downloaded. Restarting...", type = "success")
      Sys.sleep(1.5)
    }

    # Launch installer silently and quit
    # /SILENT /SUPPRESSMSGBOXES /NORESTART is standard for InnoSetup
    message("[RDesk] Executing silent installer: ", dest)
    system2(dest, args = c("/SILENT", "/SUPPRESSMSGBOXES", "/NORESTART"),
            wait = FALSE)

    if (!is.null(app)) {
      app$quit()
    } else {
      # Fallback if no app object - just tell the user we are exiting
      message("[RDesk] Update triggered. Closing application.")
      quit(save = "no")
    }

    invisible(TRUE)

  }, error = function(e) {
    if (!is.null(app)) {
      app$toast(paste("Update failed:", e$message), type = "error")
    }
    warning("[RDesk] Update failed: ", e$message)
    invisible(FALSE)
  })
}

# tests/manual/test_phase3.R
library(devtools)
load_all("c:/Users/Janak/OneDrive/Documents/RDesk/RDesk", recompile = FALSE)

app <- App$new(title = "Phase 3 Native Features", width = 900, height = 650)

app$on_ready(function() {
  message("[R] Setting up menu...")
  
  # Define the menu structure
  # Note: The launcher handles list(Label = list("Item" = fn, ...))
  app$set_menu(list(
    File = list(
      "Open file" = function() {
        message("[R] Opening dialog...")
        path <- app$dialog_open(
          title   = "Choose a CSV",
          filters = list("CSV files" = "*.csv", "Text files" = "*.txt")
        )
        if (!is.null(path)) {
          message("[R] User chose: ", path)
          app$send("log", list(msg = paste("Loaded:", path)))
        } else {
          message("[R] User cancelled dialog")
        }
      },
      "Save output" = function() {
        path <- app$dialog_save(
          title        = "Save result",
          default_name = "output.csv"
        )
        if (!is.null(path)) {
          message("[R] Save to: ", path)
          app$send("log", list(msg = paste("Saving to:", path)))
        }
      },
      "---",
      "Exit" = app$quit
    ),
    Help = list(
      "About" = function() {
        app$notify("RDesk", "Version 0.1.0 — the first native R desktop framework")
      }
    )
  ))

  app$notify("RDesk", "Phase 3 test app is ready")
  message("[R] App ready. Test the menu bar and dialogs.")
})

# Add a simple handler for logging messages from R to UI if needed
# For now we'll just use the default hello template which shows messages in log
app$on_message("ping", function(msg) {
  app$send("log", list(msg = "Pong from R"))
})

app$on_message("get_data", function(msg) {
  app$send("data_result", list(nrow = 32, cols = c("mpg", "cyl")))
})

app$on_message("get_time", function(msg) {
  app$send("time_result", list(time = format(Sys.time())))
})

app$run()

# tests/manual/verify_server.R
library(devtools)
load_all("c:/Users/Janak/OneDrive/Documents/RDesk/RDesk", recompile = FALSE)

tryCatch({
  port <- RDesk:::rdesk_free_port()
  message("Found free port: ", port)
  
  www_path <- RDesk:::rdesk_resolve_www(NULL)
  message("Resolved www path: ", www_path)
  
  if (!dir.exists(www_path)) stop("www path does not exist!")
  
  server <- RDesk:::rdesk_start_server(port, www_path, function(ws) {
    message("WS handler initialized")
  })
  
  message("Server started successfully at http://127.0.0.1:", port)
  
  # Brief delay to ensure server is receptive
  Sys.sleep(1)
  
  httpuv::stopServer(server)
  message("Server stopped successfully")
  
}, error = function(e) {
  message("ERROR: ", e$message)
  quit(status = 1)
})

# inst/apps/mtcars_dashboard/R/plots.R
# Visualization logic

plot_to_b64 <- function(p, w = 820, h = 400) {
  tmp <- tempfile(fileext = ".png")
  on.exit(unlink(tmp))
  ggplot2::ggsave(tmp, plot = p, width = w/96, height = h/96,
                  dpi = 96, device = "png", bg = "white")
  raw  <- readBin(tmp, "raw", file.info(tmp)$size)
  paste0("data:image/png;base64,", base64enc::base64encode(raw))
}

make_plot <- function(df, x, y, type = "scatter") {
  p <- ggplot2::ggplot(df, ggplot2::aes(
      x     = if (type == "boxplot") factor(cyl) else .data[[x]],
      y     = .data[[y]],
      color = factor(cyl),
      fill  = if (type == "boxplot") factor(cyl) else NULL,
      label = model
    ))
  
  if (type == "boxplot") {
    p <- p + 
      ggplot2::geom_boxplot(alpha = 0.6, outlier.size = 2) +
      ggplot2::geom_jitter(width = 0.2, alpha = 0.5)
  } else {
    p <- p + 
      ggplot2::geom_point(size = 3.5, alpha = 0.85) +
      ggplot2::geom_smooth(ggplot2::aes(label = NULL, color = NULL, fill = NULL), 
                            method = "lm", se = TRUE,
                            color = "#2E6DA4", fill = "#D6E8F7", alpha = 0.3)
  }

  p +
    ggplot2::scale_color_manual(
      name   = "Cylinders",
      values = c("4" = "#0F6E56", "6" = "#2E6DA4", "8" = "#993C1D")
    ) +
    ggplot2::scale_fill_manual(
      name   = "Cylinders",
      values = c("4" = "#0F6E56", "6" = "#2E6DA4", "8" = "#993C1D")
    ) +
    ggplot2::labs(
      x       = if (type == "boxplot") "CYLINDERS" else toupper(x),
      y       = toupper(y),
      caption = paste0("n = ", nrow(df), " vehicles")
    ) +
    ggplot2::theme_minimal(base_size = 13) +
    ggplot2::theme(
      plot.background  = ggplot2::element_rect(fill = "white", color = NA),
      panel.grid.minor = ggplot2::element_blank(),
      legend.position  = "right"
    )
}

# R/build.R
# rdesk::build_app() - packages an RDesk app into a self-contained distributable

#' Build a self-contained distributable from an RDesk application
#'
#' @param app_dir Path to the app directory (must contain app.R and www/)
#' @param out_dir Output directory for the zip file (created if not exists)
#' @param app_name Name of the application. Defaults to name in DESCRIPTION or "MyRDeskApp".
#' @param version Version string. Defaults to version in DESCRIPTION or "1.0.0".
#' @param r_version R version to bundle e.g. "4.4.2". Defaults to current R version.
#' @param include_packages Character vector of extra CRAN packages to bundle.
#'   RDesk's own dependencies are always included automatically.
#' @param portable_r_method How to provision the bundled R runtime when
#'   `runtime_dir = "download"` is used explicitly. `"extract_only"` requires
#'   standalone 7-Zip and never launches the R installer. `"installer"` allows
#'   the legacy silent installer path explicitly.
#' @param runtime_dir Controls which R runtime is bundled with the app.
#'   \itemize{
#'     \item `NULL` (default) -- copies the developer's currently running R
#'       installation. Guarantees the bundled packages and the runtime are the
#'       same R version, eliminating renv version-mismatch crashes.
#'     \item A filesystem path -- copies that R installation root directly.
#'     \item `"download"` -- downloads a portable R installer from CRAN
#'       (legacy behaviour; requires network; risks version mismatch with renv).
#'   }
#' @param overwrite If TRUE, overwrite existing output. Default FALSE.
#' @param build_installer If TRUE, also build a Windows installer (.exe) using InnoSetup.
#' @param publisher Documentation for the application publisher (used in installer).
#' @param website URL for the application website (used in installer).
#' @param license_file Path to a license file (.txt or .rtf) to include in the installer.
#' @param icon_file Path to an .ico file for the installer and application shortcut.
#' @param prune_runtime If TRUE, remove unnecessary files (Tcl/Tk, docs, tests) from 
#'   the bundled R runtime to reduce size (~15-20MB saving). Default TRUE.
#' @param dry_run If TRUE, performs a quick validation of the app structure and 
#'   environment without performing the full build. Default FALSE.
#' @return Path to the created zip file, invisibly.
#' @examples
#' # Prepare an app directory (following scaffold example)
#' app_path <- file.path(tempdir(), "MyApp")
#' rdesk_create_app("MyApp", path = tempdir())
#' 
#' # Perform a dry-run build (fast, no external binaries downloaded)
#' build_app(app_path, out_dir = tempdir(), dry_run = TRUE)
#' @export
build_app <- function(app_dir = ".",
                      out_dir  = file.path(tempdir(), "dist"),
                      app_name = NULL,
                      version  = NULL,
                      r_version = NULL,
                      include_packages = character(0),
                      portable_r_method = c("extract_only", "installer"),
                      runtime_dir = NULL,
                      overwrite = FALSE,
                      build_installer = FALSE,
                      publisher = "RDesk User",
                      website   = "https://github.com/Janakiraman-311/RDesk",
                      license_file = NULL,
                      icon_file    = NULL,
                      prune_runtime = TRUE,
                      dry_run       = FALSE) {

  # Restore options on exit
  old_opts <- options(timeout = max(1200, getOption("timeout")))
  on.exit(options(old_opts), add = TRUE)
  portable_r_method <- match.arg(portable_r_method)
  app_dir <- normalizePath(app_dir, mustWork = TRUE)
  # Normalise runtime_dir:
  #   NULL      -> auto-detect (copy developer's R)
  #   "download" -> legacy download behaviour
  #   <path>    -> copy that path directly
  use_download <- identical(runtime_dir, "download")
  user_runtime_dir <- if (use_download || is.null(runtime_dir)) NULL else
    normalizePath(path.expand(runtime_dir), mustWork = TRUE)

  if (dry_run) {
    message("\n[RDesk] DRY RUN: Validating app structure...")
    if (!file.exists(file.path(app_dir, "app.R"))) stop("[dry_run] Missing app.R")
    if (!dir.exists(file.path(app_dir, "www"))) stop("[dry_run] Missing www/")
    message("[RDesk]   V Structure OK")
    
    # Check RTools
    rtools_path <- Sys.getenv("RTOOLS45_HOME", Sys.getenv("RTOOLS44_HOME", ""))
    if (nzchar(rtools_path)) {
      message("[RDesk]   V RTools found: ", rtools_path)
    } else {
      message("[RDesk]   ! RTools not found (Optional if using pre-built binaries)")
    }
    
    message("[RDesk] DRY RUN: All checks passed.")
    return(invisible(TRUE))
  }

  # Auto-detect metadata from DESCRIPTION if possible
  desc_path <- file.path(app_dir, "DESCRIPTION")
  if (file.exists(desc_path)) {
    desc <- read.dcf(desc_path)
    if (is.null(app_name)) {
      if ("Package" %in% colnames(desc)) app_name <- as.character(desc[1, "Package"])
      else if ("AppName" %in% colnames(desc)) app_name <- as.character(desc[1, "AppName"])
    }
    if (is.null(version) && "Version" %in% colnames(desc)) {
      version <- as.character(desc[1, "Version"])
    }
  }

  # Fallbacks
  if (is.null(app_name)) app_name <- "MyRDeskApp"
  if (is.null(version))  version  <- "1.0.0"

  # ---- Pre-flight Validation -----------------------------------------------
  rdesk_validate_build_inputs(
    app_dir = app_dir,
    extra_pkgs = include_packages,
    build_installer = build_installer,
    portable_r_method = portable_r_method,
    runtime_dir = user_runtime_dir,
    use_download = use_download
  )

  if (is.null(r_version))
    r_version <- paste0(R.version$major, ".", R.version$minor)

  # ---- Staging directory ----------------------------------------------------
  dist_name  <- paste0(app_name, "-", version, "-windows")
  stage_root <- file.path(tempdir(), dist_name)
  if (dir.exists(stage_root)) unlink(stage_root, recursive = TRUE)
  dir.create(stage_root, recursive = TRUE)
  on.exit(unlink(stage_root, recursive = TRUE, force = TRUE), add = TRUE)

  message("[RDesk] Building: ", dist_name)
  message("[RDesk] Staging in: ", stage_root)

  # ---- Step 1: Copy app files ----------------------------------------------
  message("[RDesk] Step 1/6 - copying app files...")
  app_stage <- file.path(stage_root, "app")
  dir.create(app_stage)
  rdesk_copy_dir(app_dir, app_stage)

  # ---- Step 2: Copy RDesk binaries -----------------------------------------
  message("[RDesk] Step 2/6 - copying launcher binaries...")
  bin_src   <- system.file("bin", package = "RDesk")
  if (bin_src == "" || !dir.exists(bin_src)) {
    bin_src <- rdesk_resolve_launcher_bin_dir(getwd())
  }
  if (!dir.exists(bin_src)) {
    stop("[build_app] Could not locate launcher binaries under installed package or source tree.")
  }
  bin_stage <- file.path(stage_root, "bin")
  dir.create(bin_stage)
  rdesk_copy_dir(bin_src, bin_stage)

  # ---- Step 3: Provision R runtime -----------------------------------------
  stage_runtime_dir <- file.path(stage_root, "runtime", "R")
  dir.create(stage_runtime_dir, recursive = TRUE)

  if (!is.null(user_runtime_dir)) {
    # Explicit path supplied by developer
    message("[RDesk] Step 3/6 - copying provided R runtime: ", user_runtime_dir)
    rdesk_copy_r_runtime(user_runtime_dir, stage_runtime_dir)
    if (prune_runtime) rdesk_prune_runtime(stage_runtime_dir)

  } else if (use_download) {
    # Explicit "download" sentinel — legacy behaviour
    message("[RDesk] Step 3/6 - downloading portable R ", r_version, " (legacy mode)...")
    message("[RDesk]   NOTE: Consider using the default (runtime_dir = NULL) to avoid")
    message("[RDesk]   version-mismatch crashes between the downloaded R and your renv packages.")
    r_version <- rdesk_fetch_portable_r(
      r_version = r_version,
      dest_dir  = stage_runtime_dir,
      prune     = prune_runtime,
      method    = portable_r_method
    )

  } else {
    # Default: copy the developer's own R installation
    r_home <- rdesk_detect_r_home()
    message("[RDesk] Step 3/6 - copying your R ", r_version, " installation as app runtime...")
    message("[RDesk]   This guarantees renv packages and the runtime are the same version.")
    rdesk_copy_r_runtime(r_home, stage_runtime_dir)
    if (prune_runtime) rdesk_prune_runtime(stage_runtime_dir)
  }

  # ---- Step 4: Bundle packages ---------------------------------------------
  message("[RDesk] Step 4/6 - bundling R packages...")
  pkg_lib <- file.path(stage_root, "packages", "library")
  dir.create(pkg_lib, recursive = TRUE)

  # Always include RDesk and its hard deps that might not be on CRAN
  core_pkgs <- c("RDesk", "R6", "jsonlite", "processx", "base64enc", 
                 "ggplot2", "dplyr", "digest", "zip", "callr", "httpuv", 
                 "mirai", "nanonext", "rcmdcheck", "renv", "rstudioapi")
  all_pkgs  <- unique(c(core_pkgs, include_packages))

  rdesk_install_packages_to(all_pkgs, pkg_lib, r_version)

  # Install RDesk separately from the local source tree or the installed package.
  message("[RDesk]   Bundling RDesk package...")
  rdesk_src <- normalizePath(getwd(), mustWork = FALSE)
  is_rdesk_source <- FALSE
  if (file.exists(file.path(rdesk_src, "DESCRIPTION"))) {
    desc_check <- read.dcf(file.path(rdesk_src, "DESCRIPTION"))
    if ("Package" %in% colnames(desc_check) && desc_check[1, "Package"] == "RDesk") {
      is_rdesk_source <- TRUE
    }
  }

  if (is_rdesk_source) {
    message("[RDesk]     Source tree detected.")
    # Build to binary zip to avoid 'in use' installation errors
    tmp_bin <- file.path(tempdir(), "RDesk_bundle.zip")
    suppressMessages(devtools::build(rdesk_src, binary = TRUE, path = tempdir(), quiet = TRUE))
    # devtools::build returns the path, but find it just in case
    zip_files <- list.files(tempdir(), pattern = "^RDesk_.*\\.zip$", full.names = TRUE)
    if (length(zip_files) > 0) {
      zip::unzip(zip_files[1], exdir = pkg_lib)
      file.remove(zip_files)
    } else {
      # Fallback to direct library copy if build fails
      installed_rdesk <- system.file(package = "RDesk")
      if (nzchar(installed_rdesk)) {
        rdesk_copy_dir(installed_rdesk, file.path(pkg_lib, "RDesk"))
      } else {
        stop("[build_app] Failed to build RDesk binary for bundling.")
      }
    }
  } else {
    installed_rdesk <- system.file(package = "RDesk")
    if (!nzchar(installed_rdesk)) {
      stop("[build_app] Could not locate the installed RDesk package to bundle.")
    }
    message("[RDesk]     Installed package detected.")
    rdesk_copy_dir(installed_rdesk, file.path(pkg_lib, "RDesk"))
  }

  # ---- Step 4b: Snapshot package versions into bundle ---------------------
  message("[RDesk] Step 4b/6 - snapshotting package versions...")
  rdesk_snapshot_bundle(pkg_lib, stage_root)
  
  # ---- Step 5: Build the launcher stub ------------------------------------
  message("[RDesk] Step 5/6 - building launcher stub...")
  # In development, system.file might not work correctly if not installed
  stub_src <- system.file("stub", "stub.cpp", package = "RDesk")
  if (stub_src == "") {
    stub_src <- file.path(getwd(), "inst/stub/stub.cpp")
  }
  
  stub_exe <- file.path(stage_root, paste0(app_name, ".exe"))
  rdesk_build_stub(stub_src, stub_exe, app_name)

  # ---- Step 6: Zip everything ----------------------------------------------
  message("[RDesk] Step 6/6 - creating zip archive...")
  if (!dir.exists(out_dir)) dir.create(out_dir, recursive = TRUE)
  zip_path <- file.path(normalizePath(out_dir), paste0(dist_name, ".zip"))
  if (file.exists(zip_path)) {
    if (!overwrite) stop("[build_app] Output already exists: ", zip_path,
                         "\nUse overwrite=TRUE to replace.")
    file.remove(zip_path)
  }

  old_wd <- getwd()
  on.exit(setwd(old_wd), add = TRUE)
  setwd(dirname(stage_root))
  zip::zip(zip_path, files = basename(stage_root), recurse = TRUE)

  size_mb <- round(file.info(zip_path)$size / 1024^2, 1)
  message("[RDesk] Done! ", zip_path, " (", size_mb, " MB)")

  # ---- Step 7: Build installer (Optional) ----------------------------------
  if (build_installer) {
    message("[RDesk] Step 7/7 - building Windows setup executable...")
    rdesk_build_installer(
      stage_root = stage_root,
      out_dir    = out_dir,
      app_name   = app_name,
      version    = version,
      publisher  = publisher,
      website    = website,
      license_file = license_file,
      icon_file    = icon_file
    )
  }

  message("[RDesk] Distribute the output - no R installation needed on the target machine.")

  invisible(zip_path)
}

#' Validate build inputs before starting the process
#' @keywords internal
#' @param app_dir Path to app directory.
#' @param extra_pkgs Character vector of packages.
#' @param build_installer Logical.
#' @param portable_r_method Method for R portability.
#' @param runtime_dir Path to pre-existing runtime.
rdesk_validate_build_inputs <- function(app_dir,
                                        extra_pkgs,
                                        build_installer = FALSE,
                                        portable_r_method = c("extract_only", "installer"),
                                        runtime_dir = NULL,
                                        use_download = FALSE) {
  portable_r_method <- match.arg(portable_r_method)
  message("[RDesk] Pre-flight validation...")

  # 1. Essential files
  if (!file.exists(file.path(app_dir, "app.R")))
    stop("[Validation Failed] app.R not found in: ", app_dir)
  if (!dir.exists(file.path(app_dir, "www")))
    stop("[Validation Failed] www/ directory not found in: ", app_dir)

  # 2. Package check
  core_pkgs <- c("R6", "jsonlite", "processx", "base64enc", "ggplot2", "dplyr", "zip")
  all_pkgs  <- unique(c(core_pkgs, extra_pkgs))
  missing   <- all_pkgs[!vapply(all_pkgs, requireNamespace, logical(1), quietly = TRUE)]
  if (length(missing) > 0) {
    stop("[Validation Failed] The following required packages are not available in your R library:\n",
         paste("  -", missing, collapse = "\n"),
         "\nPlease install them before building.")
  }

  # 3. Rtools check (needed for stub compilation)
  tryCatch(rdesk_find_gpp(), error = function(e) {
    stop("[Validation Failed] Rtools (g++) is required to build the launcher stub.\n",
         "Error: ", e$message)
  })

  # 3b. Runtime provisioning validation
  if (!is.null(runtime_dir)) {
    # Explicit path supplied — must contain bin/
    if (!dir.exists(file.path(runtime_dir, "bin"))) {
      stop("[Validation Failed] runtime_dir must point to an R installation root containing bin/.\n",
           "Provided path: ", runtime_dir)
    }
  } else if (use_download && portable_r_method == "extract_only") {
    # Download path requested — check for 7-Zip
    sevenzip <- rdesk_find_7zip()
    if (is.null(sevenzip)) {
      message("[RDesk]   Warning: Standalone 7-Zip not found.")
      message("[RDesk]   Switching to portable_r_method='installer' (no extra tools needed).")
      assign("portable_r_method", "installer", envir = parent.frame())
    }
  } else {
    # Default auto-detect path — validate R.home() is accessible
    r_home <- R.home()
    if (!dir.exists(file.path(r_home, "bin"))) {
      stop("[Validation Failed] Cannot locate your R installation at R.home(): ", r_home,
           "\nIf this is unexpected, supply runtime_dir = R.home() explicitly.")
    }
    message("[RDesk]   Runtime: your R ",
            paste0(R.version$major, ".", R.version$minor),
            " at ", r_home)
  }

  # 4. InnoSetup check
  if (build_installer) {
    iscc <- rdesk_find_iscc()
    if (is.null(iscc)) {
      stop("[Validation Failed] InnoSetup (ISCC.exe) not found.\n",
           "It is required to build the .exe installer.\n",
           "Download it from: https://jrsoftware.org/isdl.php")
    }
    message("[RDesk]   InnoSetup found: ", iscc)
  }

  message("[RDesk] Pre-flight check passed.")
}

# ---- Internal helpers --------------------------------------------------------

#' Auto-detect the current R installation root
#' @keywords internal
rdesk_detect_r_home <- function() {
  r_home    <- R.home()
  r_version <- paste0(R.version$major, ".", R.version$minor)
  if (!dir.exists(file.path(r_home, "bin"))) {
    stop("[RDesk] rdesk_detect_r_home: R.home() returned '" , r_home,
         "' but bin/ was not found there. Supply runtime_dir explicitly.")
  }
  message("[RDesk] Detected R ", r_version, " at: ", r_home)
  r_home
}

#' Copy an R installation into the bundle staging directory
#'
#' Copies `bin/`, `library/`, `etc/`, `modules/`, and `include/` from
#' \code{r_home} into \code{dest_dir}. Skips heavyweight directories
#' (Tcl/Tk, docs, tests) that are already pruned by \code{rdesk_prune_runtime()}.
#' @keywords internal
rdesk_copy_r_runtime <- function(r_home, dest_dir) {
  # ---- bin/ : copy only x64 executables, skip i386 -------------------------
  bin_src  <- file.path(r_home, "bin")
  bin_dest <- file.path(dest_dir, "bin")
  if (dir.exists(bin_src)) {
    message("[RDesk]   Copying bin/ (x64 only)")
    dir.create(bin_dest, recursive = TRUE, showWarnings = FALSE)
    # Copy top-level bin files (R.exe, Rscript.exe, etc.)
    top_files <- list.files(bin_src, full.names = TRUE, recursive = FALSE)
    top_files <- top_files[!file.info(top_files)$isdir]
    file.copy(top_files, bin_dest, overwrite = TRUE)
    # Copy only x64 subdirectory, not i386
    x64_src  <- file.path(bin_src, "x64")
    x64_dest <- file.path(bin_dest, "x64")
    if (dir.exists(x64_src)) {
      dir.create(x64_dest, recursive = TRUE, showWarnings = FALSE)
      file.copy(list.files(x64_src, full.names = TRUE), x64_dest, overwrite = TRUE)
    }
  }

  # ---- library/ : copy only base + recommended packages --------------------
  lib_src  <- file.path(r_home, "library")
  lib_dest <- file.path(dest_dir, "library")
  if (dir.exists(lib_src)) {
    message("[RDesk]   Copying library/ (base + recommended only)")
    dir.create(lib_dest, recursive = TRUE, showWarnings = FALSE)
    pkgs <- list.dirs(lib_src, recursive = FALSE, full.names = TRUE)
    for (p in pkgs) {
      desc_path <- file.path(p, "DESCRIPTION")
      keep <- FALSE
      if (file.exists(desc_path)) {
        d <- tryCatch(read.dcf(desc_path), error = function(e) NULL)
        if (!is.null(d) && "Priority" %in% colnames(d)) {
          priority <- trimws(as.character(d[1, "Priority"]))
          keep <- priority %in% c("base", "recommended")
        }
      }
      # Also keep 'translations' which has no Priority but ships with R
      if (!keep && basename(p) == "translations") keep <- TRUE
      if (keep) {
        file.copy(p, lib_dest, recursive = TRUE, overwrite = TRUE)
      }
    }
  }

  # ---- etc/, modules/, include/ : copy fully (small, essential) ------------
  for (d in c("etc", "modules", "include")) {
    src <- file.path(r_home, d)
    if (dir.exists(src)) {
      message("[RDesk]   Copying ", d, "/")
      file.copy(src, dest_dir, recursive = TRUE, overwrite = TRUE)
    }
  }

  invisible(dest_dir)
}

rdesk_copy_dir <- function(from, to) {
  dirs <- list.dirs(from, recursive = TRUE, full.names = TRUE)
  for (d in dirs) {
    rel <- substring(d, nchar(from) + 2)
    if (nzchar(rel)) {
      dir.create(file.path(to, rel), recursive = TRUE, showWarnings = FALSE)
    }
  }

  files <- list.files(from, recursive = TRUE, full.names = TRUE, all.files = TRUE, no.. = TRUE)
  for (f in files) {
    if (dir.exists(f)) next
    rel  <- substring(f, nchar(from) + 2)
    dest <- file.path(to, rel)
    dir.create(dirname(dest), recursive = TRUE, showWarnings = FALSE)
    file.copy(f, dest, overwrite = TRUE)
  }
}

rdesk_fetch_portable_r <- function(r_version,
                                   dest_dir,
                                   prune = TRUE,
                                   method = c("extract_only", "installer")) {
  method <- match.arg(method)
  
  # Try primary first
  url_primary <- paste0("https://cloud.r-project.org/bin/windows/base/R-", r_version, "-win.exe")
  tmp_exe_primary <- file.path(tempdir(), paste0("R-", r_version, "-win.exe"))
  
  success <- FALSE
  actual_v <- r_version
  
  if (file.exists(tmp_exe_primary)) {
    success <- TRUE
    final_exe <- tmp_exe_primary
  } else {
    message("[RDesk]   Downloading R installer (~80MB)...")
    success <- tryCatch({
      suppressWarnings(utils::download.file(url_primary, tmp_exe_primary, mode = "wb", quiet = FALSE, method = "libcurl"))
      final_exe <- tmp_exe_primary
      TRUE
    }, error = function(e) { FALSE })
    
    if (!success && grepl("4.5", r_version)) {
      message("[RDesk]   R 4.5.x not found on mirror. Falling back to stable R 4.4.2...")
      actual_v <- "4.4.2"
      url_fallback <- "https://cloud.r-project.org/bin/windows/base/old/4.4.2/R-4.4.2-win.exe"
      tmp_exe_fallback <- file.path(tempdir(), "R-4.4.2-win.exe")
      
      if (file.exists(tmp_exe_fallback)) {
        final_exe <- tmp_exe_fallback
        success <- TRUE
      } else {
        success <- tryCatch({
          suppressWarnings(utils::download.file(url_fallback, tmp_exe_fallback, mode = "wb", quiet = FALSE, method = "libcurl"))
          final_exe <- tmp_exe_fallback
          TRUE
        }, error = function(e2) FALSE)
      }
    }
  }
  
  if (!success) stop("[build_app] Failed to download R installer (tried 4.5.x and 4.4.2).")
  
  message("[RDesk]   Preparing R runtime (", actual_v, ") (this takes ~60 seconds)...")
  tmp_extract <- file.path(tempdir(), paste0("R-", actual_v, "-extract"))
  if (dir.exists(tmp_extract)) unlink(tmp_extract, recursive = TRUE)
  dir.create(tmp_extract, recursive = TRUE)

  if (method == "extract_only") {
    sevenzip <- rdesk_find_7zip()
    ret <- system2(sevenzip, args = c("x", "-y", paste0("-o", normalizePath(tmp_extract, winslash = "\\")), normalizePath(final_exe, winslash = "\\")), stdout = FALSE, stderr = FALSE)
    if (!identical(ret, 0L)) stop("[build_app] Failed to extract the R installer with standalone 7-Zip.")
  } else {
    install_cmd <- sprintf('"%s" /SILENT /DIR="%s" /COMPONENTS="main,x64"', normalizePath(final_exe), normalizePath(tmp_extract))
    ret <- system(install_cmd, wait = TRUE, show.output.on.console = FALSE)
    if (!identical(ret, 0L)) stop("[build_app] Silent installation of R failed.")
  }

  r_root <- rdesk_find_r_dir(tmp_extract)
  if (is.null(r_root)) stop("[build_app] Could not locate the extracted R runtime.")
  rdesk_copy_dir(r_root, dest_dir)
  if (prune) rdesk_prune_runtime(dest_dir)
  
  return(actual_v)
}

rdesk_prune_runtime <- function(runtime_dir) {
  prune <- c("doc", "tests", "Tcl", "share/locale", "library/tcltk", "library/KernSmooth", "library/spatial")
  for (p in prune) {
    target <- file.path(runtime_dir, p)
    if (dir.exists(target)) unlink(target, recursive = TRUE)
  }
}

rdesk_find_7zip <- function() {
  candidates <- c(Sys.which("7z"), Sys.which("7za"), "C:/Program Files/7-Zip/7z.exe", "C:/Program Files (x86)/7-Zip/7z.exe")
  found <- candidates[nchar(candidates) > 0 & file.exists(candidates)]
  found <- found[!grepl("rtools", found, ignore.case = TRUE)]
  if (length(found) == 0) return(NULL)
  found[1]
}

rdesk_find_r_dir <- function(extracted_root) {
  all_rscripts <- list.files(extracted_root, pattern = "Rscript.exe", recursive = TRUE, full.names = TRUE)
  if (length(all_rscripts) == 0) return(NULL)
  dirname(dirname(all_rscripts[1]))
}

rdesk_install_packages_to <- function(pkgs, lib_dir, r_version) {
  minor <- paste(strsplit(r_version, "\\.")[[1]][1:2], collapse = ".")
  avail <- tryCatch(utils::available.packages(repos = "https://cloud.r-project.org", type = "win.binary", filters = list()), error = function(e) NULL)
  all_deps <- rdesk_resolve_deps(pkgs, avail)
  all_deps <- setdiff(all_deps, "RDesk")
  target_repos <- sprintf("https://cloud.r-project.org/bin/windows/contrib/%s", minor)
  
  if (length(all_deps) > 0) {
    message("[RDesk]   Downloading ", length(all_deps), " packages...")
    utils::install.packages(all_deps, lib = lib_dir, contriburl = target_repos, type = "win.binary", quiet = FALSE, dependencies = FALSE)
  }
  
  # Final verification of bundled critical packages
  critical <- intersect(all_deps, c("callr", "mirai", "nanonext", "ggplot2", "processx"))
  found <- list.dirs(lib_dir, full.names = FALSE, recursive = FALSE)
  missing <- setdiff(critical, found)
  if (length(missing) > 0) {
    stop("[build_app] CRITICAL FAILURE: The following core dependencies failed to bundle:\n",
         paste("  -", missing, collapse = "\n"),
         "\nThis is likely due to a CRAN mirror issue or package availability for R ", r_version)
  }
}

rdesk_resolve_deps <- function(pkgs, avail) {
  base_pkgs <- c("base", "compiler", "datasets", "graphics", "grDevices", "grid", 
                 "methods", "parallel", "splines", "stats", "stats4", "tcltk", 
                 "tools", "utils", "MASS", "lattice", "boot", "class", "cluster", 
                 "codetools", "foreign", "KernSmooth", "mgcv", "nlme", "nnet", 
                 "rpart", "spatial", "survival")
  resolved  <- character(0)
  queue     <- pkgs
  while (length(queue) > 0) {
    pkg <- queue[1]; queue <- queue[-1]
    if (pkg %in% resolved || pkg %in% base_pkgs || pkg == "RDesk") next
    resolved <- c(resolved, pkg)
    if (!is.null(avail) && pkg %in% rownames(avail)) {
      # Check both Depends and Imports for binary transparency
      dep_fields <- avail[pkg, c("Depends", "Imports")]
      dep_fields <- dep_fields[!is.na(dep_fields) & nchar(dep_fields) > 0]
      deps_str <- paste(dep_fields, collapse = ", ")
      
      if (nchar(deps_str) > 0) {
        dep_names <- trimws(gsub("\\s*\\(.*?\\)", "", strsplit(deps_str, ",")[[1]]))
        # Filter out R itself from Depends
        dep_names <- dep_names[dep_names != "R"]
        queue <- c(queue, setdiff(dep_names[nchar(dep_names) > 0], c(resolved, base_pkgs)))
      }
    }
  }
  resolved
}

rdesk_build_stub <- function(stub_cpp, out_exe, app_name) {
  gpp <- rdesk_find_gpp()
  tmp_cpp <- file.path(tempdir(), paste0("stub_", digest::digest(app_name, algo="crc32"), ".cpp"))
  lines <- readLines(stub_cpp); lines <- gsub("{{APP_NAME}}", app_name, lines, fixed = TRUE); writeLines(lines, tmp_cpp)
  inc_path <- system.file("include", package = "RDesk")
  if (inc_path == "") inc_path <- file.path(getwd(), "inst/include")
  src_inc <- dirname(normalizePath(stub_cpp, mustWork = TRUE))
  sdk_inc <- file.path(src_inc, "webview2_sdk", "build", "native", "include")
  system2(gpp, args = c("-std=c++17", "-O2", "-mwindows", "-I", shQuote(inc_path), "-I", shQuote(src_inc), "-I", shQuote(sdk_inc), shQuote(tmp_cpp), "-o", shQuote(out_exe), "-lole32", "-lcomctl32", "-loleaut32", "-luuid", "-lshlwapi", "-lversion", "-lstdc++fs"))
}

rdesk_find_iscc <- function() {
  candidates <- c(Sys.which("ISCC"), file.path(Sys.getenv("LOCALAPPDATA"), "Programs", "Inno Setup 6", "ISCC.exe"), "C:/Program Files (x86)/Inno Setup 6/ISCC.exe")
  found <- candidates[nchar(candidates) > 0 & file.exists(candidates)]
  if (length(found) == 0) return(NULL)
  found[1]
}

rdesk_build_installer <- function(stage_root, out_dir, app_name, version, publisher, website, license_file, icon_file) {
  template_path <- system.file("installer", "template.iss", package = "RDesk")
  if (template_path == "") template_path <- file.path(getwd(), "inst/installer/template.iss")
  iss_content <- readLines(template_path)
  iss_content <- gsub("{{AppName}}", app_name, iss_content, fixed = TRUE)
  iss_content <- gsub("{{AppVersion}}", version, iss_content, fixed = TRUE)
  iss_content <- gsub("{{AppPublisher}}", publisher, iss_content, fixed = TRUE)
  iss_content <- gsub("{{AppURL}}", website, iss_content, fixed = TRUE)
  iss_content <- gsub("{{AppExeName}}", paste0(app_name, ".exe"), iss_content, fixed = TRUE)
  iss_content <- gsub("{{SourceDir}}", normalizePath(stage_root), iss_content, fixed = TRUE)
  iss_content <- gsub("{{OutputDir}}", normalizePath(out_dir), iss_content, fixed = TRUE)
  iss_content <- gsub("{{SetupBaseName}}", paste0(app_name, "-", version, "-setup"), iss_content, fixed = TRUE)
  iss_content <- gsub("{{AppID}}", sprintf("RDesk-App-%s", digest::digest(app_name, algo = "crc32")), iss_content, fixed = TRUE)
  license_path <- if (!is.null(license_file)) normalizePath(license_file) else ""
  iss_content <- gsub("{{LicenseFile}}", license_path, iss_content, fixed = TRUE)
  icon_path <- if (!is.null(icon_file)) normalizePath(icon_file) else ""
  iss_content <- gsub("{{AppIconFile}}", icon_path, iss_content, fixed = TRUE)
  iss_temp <- file.path(tempdir(), "installer.iss"); writeLines(iss_content, iss_temp)
  system2(rdesk_find_iscc(), args = c("/Q", shQuote(iss_temp)))
}

rdesk_find_gpp <- function() {
  candidates <- c(Sys.which("g++"), "C:/rtools45/mingw64/bin/g++.exe", "C:/rtools44/mingw64/bin/g++.exe")
  found <- candidates[nchar(candidates) > 0 & file.exists(candidates)]
  if (length(found) == 0) stop("[build_app] g++ not found.")
  found[1]
}

#' Resolve a source-tree launcher binary directory
#' @keywords internal
rdesk_resolve_launcher_bin_dir <- function(project_root) {
  inst_bin <- file.path(project_root, "inst", "bin")
  if (dir.exists(inst_bin) && file.exists(file.path(inst_bin, "rdesk-launcher.exe"))) return(inst_bin)

  # Check src/ for source-built launcher
  src_bin <- file.path(project_root, "src")
  if (file.exists(file.path(src_bin, "rdesk-launcher.exe"))) {
    temp_bin <- file.path(tempdir(), "rdesk-launcher-bin")
    if (dir.exists(temp_bin)) unlink(temp_bin, recursive = TRUE)
    dir.create(temp_bin, recursive = TRUE)
    file.copy(file.path(src_bin, "rdesk-launcher.exe"), file.path(temp_bin, "rdesk-launcher.exe"))
    return(temp_bin)
  }
  ""
}

rdesk_snapshot_bundle <- function(lib_dir, stage_root) {
  if (!requireNamespace("renv", quietly = TRUE)) return(invisible(NULL))
  pkg_names <- list.dirs(lib_dir, full.names = FALSE, recursive = FALSE)
  if (length(pkg_names) == 0) return(invisible(NULL))
  
  lock_entries <- lapply(pkg_names, function(p_name) {
    ver <- as.character(utils::packageVersion(p_name, lib.loc = lib_dir))
    list(Package = p_name, Version = ver, Source = "Repository", Repository = "CRAN")
  })
  names(lock_entries) <- pkg_names
  lockfile <- list(R = list(Version = paste0(R.version$major, ".", R.version$minor), Repositories = list(list(Name = "CRAN", URL = "https://cloud.r-project.org"))), Packages = lock_entries)
  jsonlite::write_json(lockfile, file.path(stage_root, "renv.lock"), pretty = TRUE, auto_unbox = TRUE)
}

test_that("rdesk_validate_build_inputs() stops on missing app.R", {
  withr::with_tempdir({
    dir.create("www")
    expect_error(
      rdesk_validate_build_inputs(".", character(0), FALSE),
      "app.R not found"
    )
  })
})

test_that("rdesk_validate_build_inputs() stops on missing www/", {
  withr::with_tempdir({
    file.create("app.R")
    expect_error(
      rdesk_validate_build_inputs(".", character(0), FALSE),
      "www/"
    )
  })
})

test_that("rdesk_validate_build_inputs() passes with correct structure", {
  withr::with_tempdir({
    file.create("app.R")
    dir.create("www")
    # Either passes (Rtools present) or fails with a launcher-related message —
    # not an app.R / www missing error.
    result <- tryCatch(
      rdesk_validate_build_inputs(".", character(0), FALSE),
      error = function(e) e$message
    )
    if (is.character(result)) {
      # Structural checks passed; it only failed because g++ is not present
      expect_false(grepl("app.R|www/", result))
    } else {
      succeed("rdesk_validate_build_inputs passed with no error")
    }
  })
})

# inst/apps/mtcars_dashboard/app.R
# Thin entry point

resolve_app_dir <- function() {
  file_arg <- grep("^--file=", commandArgs(trailingOnly = FALSE), value = TRUE)
  if (length(file_arg) > 0) {
    script_path <- normalizePath(sub("^--file=", "", file_arg[1]), winslash = "/", mustWork = FALSE)
    if (nzchar(script_path) && file.exists(script_path)) {
      return(dirname(script_path))
    }
  }

  if (!nzchar(Sys.getenv("R_BUNDLE_APP"))) {
    rstudio_path <- tryCatch(rstudioapi::getActiveDocumentContext()$path, error = function(e) "")
    if (nzchar(rstudio_path) && file.exists(rstudio_path)) {
      return(dirname(normalizePath(rstudio_path, winslash = "/", mustWork = TRUE)))
    }
  }

  normalizePath(getwd(), winslash = "/", mustWork = TRUE)
}

app_dir <- resolve_app_dir()

# Development Guard: If we are inside the RDesk source tree, use load_all()
# instead of library(RDesk) to ensure our latest changes are active.
pkg_root <- dirname(dirname(dirname(app_dir)))
is_dev <- file.exists(file.path(pkg_root, "DESCRIPTION")) &&
  file.exists(file.path(pkg_root, "R", "App.R"))

if (!nzchar(Sys.getenv("R_BUNDLE_APP")) && is_dev) {
  message("[RDesk] Dev mode detected. Loading local source from: ", pkg_root)
  devtools::load_all(pkg_root)
} else {
  suppressPackageStartupMessages(library(RDesk))
}

suppressPackageStartupMessages(library(ggplot2))
suppressPackageStartupMessages(library(dplyr))

# Source all modular logic from R/
r_dir <- file.path(app_dir, "R")
if (!dir.exists(r_dir)) {
  stop("[mtcars_dashboard] R/ directory not found at: ", r_dir)
}

r_files <- sort(list.files(r_dir, pattern = "\\.R$", full.names = TRUE))
if (length(r_files) == 0) {
  stop("[mtcars_dashboard] No R source files found in: ", r_dir)
}

invisible(lapply(r_files, source))

# Handle startup logging for bundled apps
if (nzchar(Sys.getenv("R_BUNDLE_APP"))) {
  app_name <- Sys.getenv("R_APP_NAME", "CarsAnalyser")
  log_dir <- RDesk:::rdesk_log_dir(app_name)
  if (!dir.exists(log_dir)) dir.create(log_dir, recursive = TRUE)

  log_file <- file.path(log_dir, "rdesk_startup.log")
  sink_conn <- file(log_file, open = "wt")
  sink(sink_conn, type = "message")

  cat(sprintf("[%s] RDesk startup initiated (Modular)\n", Sys.time()))
}

cleanup_bundle_logging <- function() {
  if (nzchar(Sys.getenv("R_BUNDLE_APP"))) {
    if (sink.number(type = "message") > 0) sink(type = "message")
    if (exists("sink_conn", inherits = FALSE) && isOpen(sink_conn)) close(sink_conn)
  }
}
on.exit(cleanup_bundle_logging(), add = TRUE)

tryCatch({
  .env <- new.env(parent = .GlobalEnv)
  .env$app_dir <- app_dir
  if (!exists("init_data", mode = "function")) {
    stop("[mtcars_dashboard] init_data() was not loaded from ", r_dir)
  }
  init_data(.env)

  app <- App$new(
    title = "Motor Trend Cars Analyser - RDesk",
    width = 1100,
    height = 740,
    www = file.path(app_dir, "www")
  )

  if (exists("init_handlers")) {
    init_handlers(app, .env)
  }

  app$run()

}, error = function(e) {
  if (nzchar(Sys.getenv("R_BUNDLE_APP"))) {
    cat(sprintf("\n[%s] CRITICAL ERROR:\n%s\n", Sys.time(), e$message))
  }
  stop(e)
})

# R/async.R
# Background task management for RDesk: Dual Backend (callr + mirai)

#' @importFrom digest digest
#' @importFrom callr r_bg
#' @importFrom stats runif
NULL

# Job registry - a private environment to track running jobs
.rdesk_jobs <- new.env(parent = emptyenv())

#' Start the mirai daemon pool
#'
#' Called once at App$run() startup when rdesk.async_backend is "mirai".
#' @return Invisible number of workers started.
#' @keywords internal
rdesk_start_daemons <- function() {
  if (getOption("rdesk.async_backend", "callr") != "mirai") return(invisible(NULL))
  if (!requireNamespace("mirai", quietly = TRUE)) return(invisible(NULL))

  # Cap at 2 workers for CRAN compliance (avoid Note on CPU vs Elapsed time)
  n <- min(2L, max(1L, parallel::detectCores(logical = FALSE) - 1L))
  mirai::daemons(n)
  message("[RDesk] mirai daemon pool started: ", n, " workers")
  invisible(n)
}

#' Stop the mirai daemon pool
#'
#' Called at App$cleanup().
#' @keywords internal
rdesk_stop_daemons <- function() {
  if (getOption("rdesk.async_backend", "callr") != "mirai") return(invisible(NULL))
  if (!requireNamespace("mirai", quietly = TRUE)) return(invisible(NULL))
  
  mirai::daemons(0)
  message("[RDesk] mirai daemon pool stopped")
  invisible(NULL)
}

#' Run a task in the background
#'
#' Automatically switches between 'mirai' (persistent daemons) and 'callr' (on-demand processes).
#'
#' @param task A function to run in the background.
#' @param args A list of arguments to pass to the task.
#' @param on_done Callback function(result) called when the task finishes successfully.
#' @param on_error Callback function(error) called if the task fails.
#' @param timeout_sec Optional timeout in seconds. If exceeded, the job is
#'   cancelled and \code{on_error()} receives a timeout error.
#' @param app_id Optional App ID used to associate a job with a specific app.
#' @return Invisible job ID.
#' @examples
#' # Fast, non-interactive task check (safe to unwrap)
#' rdesk_jobs_pending()
#' 
#' if (interactive()) {
#'   # Run a long-running computation in the background
#'   rdesk_async(
#'     task = function(n) { Sys.sleep(2); sum(runif(n)) },
#'     args = list(n = 1e6),
#'     on_done = function(res) message("Task finished: ", res),
#'     on_error = function(err) message("Task failed: ", err$message)
#'   )
#' }
#' @export
rdesk_async <- function(task, args = list(), on_done = NULL, on_error = NULL,
                        timeout_sec = NULL, app_id = NULL) {
  if (missing(task) || is.null(task)) stop("Missing task function")
  if (!is.function(task)) stop("task must be a function")

  # Generate a unique job ID
  job_id  <- paste0("job_", digest::digest(runif(1), algo = "crc32"))
  backend <- getOption("rdesk.async_backend", "callr")

  if (backend == "mirai" && requireNamespace("mirai", quietly = TRUE)) {
    # mirai path - submit to persistent daemon pool
    # We use .expr to execute the task with provided args
    m <- mirai::mirai(
      .expr = do.call(task, args),
      task  = task,
      args  = args
    )
    .rdesk_jobs[[job_id]] <- list(
      job      = m,
      backend  = "mirai",
      on_done  = on_done,
      on_error = on_error,
      started  = Sys.time(),
      timeout_sec = timeout_sec,
      app_id = app_id
    )
  } else {
    # callr fallback - on-demand process spawning
    job <- callr::r_bg(task, args = args, supervise = TRUE)
    
    .rdesk_jobs[[job_id]] <- list(
      job      = job,
      backend  = "callr",
      on_done  = on_done,
      on_error = on_error,
      started  = Sys.time(),
      timeout_sec = timeout_sec,
      app_id = app_id
    )
  }
  
  invisible(job_id)
}

#' Poll background jobs
#'
#' This is called internally by the main event loop to check if any 
#' background tasks have finished. Handles both mirai and callr backends.
#'
#' @keywords internal
rdesk_poll_jobs <- function() {
  job_ids <- ls(.rdesk_jobs, pattern = "^job_")
  for (id in job_ids) {
    entry   <- .rdesk_jobs[[id]]
    backend <- entry[["backend"]]
    
    # Strictly validate that entry is a list and contains a job
    if (!is.list(entry) || is.null(entry[["job"]])) {
      rm(list = id, envir = .rdesk_jobs)
      next
    }
    
    job <- entry[["job"]]
    timeout_sec <- entry[["timeout_sec"]]

    if (!is.null(timeout_sec) &&
        difftime(Sys.time(), entry[["started"]], units = "secs") > timeout_sec) {
      rdesk_cancel_job(id)
      if (is.function(entry[["on_error"]])) {
        tryCatch(
          entry[["on_error"]](simpleError(paste0("Job timed out after ", timeout_sec, " seconds"))),
          error = function(e) warning("[RDesk] on_error handler failed: ", e$message)
        )
      }
      next
    }

    # Check completion based on backend API
    is_done <- if (backend == "mirai") {
      !mirai::unresolved(job)
    } else {
      # callr path
      status <- tryCatch(job$poll_io(0), error = function(e) NULL)
      if (is.null(status)) FALSE else !job$is_alive()
    }

    if (!is_done) next

    # Job finished - remove from registry first to avoid re-polling
    rm(list = id, envir = .rdesk_jobs)

    # Extract result or error based on backend
    if (backend == "mirai") {
      result <- job$data
      if (inherits(result, "mirai_error")) {
        if (is.function(entry[["on_error"]])) {
          tryCatch(entry[["on_error"]](simpleError(as.character(result))),
            error = function(e) warning("[RDesk] on_error handler failed: ", e$message))
        }
      } else {
        if (is.function(entry[["on_done"]])) {
          tryCatch(entry[["on_done"]](result),
            error = function(e) warning("[RDesk] on_done handler failed: ", e$message))
        }
      }
    } else {
      # callr path
      result <- tryCatch(job$get_result(), error = function(e) e)
      if (inherits(result, "error")) {
        if (is.function(entry[["on_error"]])) {
          tryCatch(entry[["on_error"]](result),
            error = function(e) warning("[RDesk] on_error handler failed: ", e$message))
        }
      } else {
        if (is.function(entry[["on_done"]])) {
          tryCatch(entry[["on_done"]](result),
            error = function(e) warning("[RDesk] on_done handler failed: ", e$message))
        }
      }
    }
  }
}

#' Cancel a running background job
#'
#' @param job_id The ID of the job to cancel.
#' @return Invisible TRUE if cancelled, FALSE if not found.
#' @export
rdesk_cancel_job <- function(job_id) {
  if (exists(job_id, envir = .rdesk_jobs)) {
    entry <- .rdesk_jobs[[job_id]]
    backend <- entry[["backend"]]
    
    if (backend == "mirai") {
      # mirai has no direct 'kill' for tasks already in a persistent daemon.
      # We simply remove from the registry so the callback never fires.
      # The daemon keeps running until completion, so cancellation here only
      # suppresses callbacks and UI follow-up.
      warning("[RDesk] Cancellation requested for a mirai job. The running task cannot be interrupted; only callbacks are suppressed.")
      rm(list = job_id, envir = .rdesk_jobs)
    } else {
      # callr path: kill the process
      if (is.list(entry) && !is.null(entry[["job"]])) {
        if (entry[["job"]]$is_alive()) entry[["job"]]$kill()
      }
      rm(list = job_id, envir = .rdesk_jobs)
    }
    invisible(TRUE)
  } else {
    invisible(FALSE)
  }
}

#' Check if any background jobs are pending
#'
#' @return Number of pending jobs.
#' @export
rdesk_jobs_pending <- function() {
  length(ls(.rdesk_jobs, pattern = "^job_"))
}

#' List currently pending background jobs
#'
#' @return A data.frame with job ID, started time, backend, and app ID.
#' @export
rdesk_jobs_list <- function() {
  job_ids <- ls(.rdesk_jobs, pattern = "^job_")
  if (length(job_ids) == 0) {
    return(data.frame(
      job_id = character(0),
      started = as.POSIXct(character(0)),
      backend = character(0),
      app_id = character(0),
      stringsAsFactors = FALSE
    ))
  }

  rows <- lapply(job_ids, function(id) {
    entry <- .rdesk_jobs[[id]]
    data.frame(
      job_id = id,
      started = as.POSIXct(entry[["started"]], origin = "1970-01-01"),
      backend = if (is.null(entry[["backend"]])) NA_character_ else entry[["backend"]],
      app_id = if (is.null(entry[["app_id"]])) NA_character_ else entry[["app_id"]],
      stringsAsFactors = FALSE
    )
  })

  do.call(rbind, rows)
}

#' Wrap a message handler to run asynchronously with zero configuration
#'
#' @description
#' \code{async()} is the simplest way to make an RDesk message handler
#' non-blocking. Wrap any handler function with \code{async()} and RDesk
#' automatically handles background execution, loading states, error toasts,
#' and result routing.
#'
#' `async()` transforms a standard RDesk message handler into a background task.
#' The UI remains responsive while the task runs. When finished, a result
#' message (e.g., `get_data_result`) is automatically sent back to the UI.
#'
#' @details
#' To ensure the background worker has access to all application logic, RDesk
#' automatically sources every `.R` file in the application's `R/` directory
#' before executing the task. It also snapshots currently loaded packages
#' (excluding system packages) to recreate the environment.
#'
#' @param fn The handler function, taking a `payload` argument.
#' @param app The RDesk `App` instance. If NULL, tries to resolve from the global registry.
#' @param loading_message Message to display in the UI overlay while working.
#' @param cancellable Whether the UI should show a 'Cancel' button.
#' @param error_message Prefix for toast notifications if the task fails.
#' @return A wrapped handler function suitable for `app$on_message()`.
#'
#' @examples
#' if (interactive()) {
#'   app$on_message("filter_cars", async(function(payload) {
#'     mtcars[mtcars$cyl == payload$cylinders, ]
#'   }, app = app))
#' }
#' @export
async <- function(fn,
                  app             = NULL,
                  loading_message = "Working...",
                  cancellable     = TRUE,
                  error_message   = "Error: ") {
  
  if (!is.function(fn)) stop("task must be a function")

  # Capture packages loaded NOW at registration time
  # This is intentional - we snapshot the environment when the
  # developer calls async(), not when the task runs
  base_pkgs <- c("base", "methods", "datasets", "utils",
                 "grDevices", "graphics", "stats", "R6",
                 "jsonlite", "digest", "processx", "callr", "mirai")

  loaded_pkgs <- tryCatch({
    loaded <- search()
    pkg_names <- gsub("^package:", "", grep("^package:", loaded, value = TRUE))
    unique(c(setdiff(pkg_names, base_pkgs), "RDesk"))
  }, error = function(e) character(0))

  # Store the message type for result routing
  # This gets populated when on_message() registers the handler
  msg_type_env <- new.env(parent = emptyenv())
  msg_type_env$type <- NULL

  # Return the actual handler function
  wrapper <- function(payload) {
    # Resolve app reference - try explicit param first, then global registry
    app_obj <- app
    if (is.null(app_obj)) {
      app_ids <- ls(.rdesk_apps)
      if (length(app_ids) > 0) {
        warning("[RDesk] async(): app should be supplied explicitly in multi-window contexts; falling back to the first registered app.")
        app_obj <- .rdesk_apps[[app_ids[1]]]
      }
    }
    if (is.null(app_obj)) {
      warning("[RDesk] async(): could not resolve app reference")
      return(invisible(NULL))
    }

    # Derive result message type from stored type
    result_type <- if (!is.null(msg_type_env$type)) {
      paste0(msg_type_env$type, "_result")
    } else {
      "__async_result__"
    }

    # Launch background task
    job_id <- rdesk_async(
      task = function(.fn, .pkgs, .payload, .app_dir) {
        # Reload packages in isolated worker context
        invisible(lapply(.pkgs, function(p) {
          tryCatch(
            library(p, character.only = TRUE,
                    quietly = TRUE, warn.conflicts = FALSE),
            error = function(e) NULL
          )
        }))
        
        # Source app modules to ensure handlers and helpers (like make_chart) are available
        if (!is.null(.app_dir) && dir.exists(file.path(.app_dir, "R"))) {
          r_files <- list.files(file.path(.app_dir, "R"), pattern = "\\.R$", full.names = TRUE)
          invisible(lapply(r_files, source))
        }
        
        # Run the developer's function
        .fn(.payload)
      },
      args = list(
        .fn      = fn,
        .pkgs    = loaded_pkgs,
        .payload = payload,
        .app_dir = if (!is.null(app_obj)) app_obj$get_dir() else NULL
      ),
      on_done = function(result) {
        app_obj$loading_done()
        app_obj$send(result_type, result)
      },
      on_error = function(err) {
        app_obj$loading_done()
        app_obj$toast(
          paste0(error_message, conditionMessage(err)),
          type = "error"
        )
      },
      app_id = app_obj$.__enclos_env__$private$.id
    )

    # Show loading overlay
    app_obj$loading_start(
      message     = loading_message,
      cancellable = cancellable,
      job_id      = job_id
    )

    invisible(job_id)
  }

  # Tag the wrapper so on_message() can inject the type
  attr(wrapper, "rdesk_async_wrapper") <- TRUE
  attr(wrapper, "rdesk_msg_type_env")  <- msg_type_env
  
  # Metadata for testing and internal identification
  attr(wrapper, "is_rdesk_async")   <- TRUE
  attr(wrapper, "loading_message") <- loading_message

  wrapper
}

library(testthat)
library(RDesk)

test_that("rdesk_async produces valid job IDs", {
  # We can't run it without a master, but we can test the creation logic
  # or mock the master if we wanted to be fancy.
  # For now, let's test that it requires a task.
  expect_error(rdesk_async(), "Missing task function")
})

test_that("async() wrapper returns function with attributes", {
  f <- function(payload) list(result = payload$x * 2)
  w <- async(f, loading_message = "Computing...")
  
  expect_true(is.function(w))
  expect_equal(attr(w, "loading_message"), "Computing...")
  expect_equal(attr(w, "is_rdesk_async"), TRUE)
})

test_that("async() handles default parameters", {
  f <- function(p) list()
  w <- async(f)
  expect_equal(attr(w, "loading_message"), "Working...")
})

test_that("async() fails if task is not a function", {
  expect_error(async("not a function"), "task must be a function")
})

# tests/manual/test_phase5.R
devtools::load_all()

# Build the demo dashboard into a distributable zip
# Use R 4.5.1 since that's what we are using locally
zip_path <- build_app(
  app_dir  = "inst/apps/mtcars_dashboard",
  out_dir  = "dist",
  app_name = "CarsAnalyser",
  version  = "1.0.0",
  r_version = "4.5.1",
  overwrite = TRUE
)

cat("Output:", zip_path, "\n")

# Verify zip contents
# Note: we need the zip package for this
if (!requireNamespace("zip", quietly = TRUE)) install.packages("zip")
contents <- zip::zip_list(zip_path)
cat("Files in zip:", nrow(contents), "\n")

# Check key files exist inside zip
key_files <- c("CarsAnalyser.exe",
                "bin/rdesk-launcher.exe",
                "runtime/R/bin/x64/Rscript.exe",
                "app/app.R")
for (f in key_files) {
  rel_paths <- contents$filename
  # Match either exact or with folder prefix
  found <- any(grepl(f, rel_paths, fixed = TRUE))
  cat(ifelse(found, "[OK]", "[MISSING]"), f, "\n")
}

# Standardize path: resolve the app directory from the running script whenever possible
resolve_app_dir <- function() {
  file_arg <- grep("^--file=", commandArgs(trailingOnly = FALSE), value = TRUE)
  if (length(file_arg) > 0) {
    script_path <- normalizePath(sub("^--file=", "", file_arg[1]), winslash = "/", mustWork = FALSE)
    if (nzchar(script_path) && file.exists(script_path)) {
      return(dirname(script_path))
    }
  }

  if (!nzchar(Sys.getenv("R_BUNDLE_APP"))) {
    rstudio_path <- tryCatch(rstudioapi::getActiveDocumentContext()$path, error = function(e) "")
    if (nzchar(rstudio_path) && file.exists(rstudio_path)) {
      return(dirname(normalizePath(rstudio_path, winslash = "/", mustWork = TRUE)))
    }
  }

  normalizePath(getwd(), winslash = "/", mustWork = TRUE)
}

app_dir <- resolve_app_dir()

# Development Guard
pkg_root <- dirname(dirname(dirname(app_dir)))
is_dev <- file.exists(file.path(pkg_root, "DESCRIPTION")) &&
  file.exists(file.path(pkg_root, "R", "App.R"))

if (!nzchar(Sys.getenv("R_BUNDLE_APP")) && is_dev) {
  message("[RDesk] Dev mode detected. Loading local source...")
  devtools::load_all(pkg_root)
} else {
  suppressPackageStartupMessages(library(RDesk))
}

suppressPackageStartupMessages(library(haven))
suppressPackageStartupMessages(library(Hmisc))
suppressPackageStartupMessages(library(dplyr))

# Handle startup logging for bundled apps
if (nzchar(Sys.getenv("R_BUNDLE_APP"))) {
  app_name <- Sys.getenv("R_APP_NAME", "TSCreator")
  log_dir <- RDesk:::rdesk_log_dir(app_name)
  if (!dir.exists(log_dir)) dir.create(log_dir, recursive = TRUE)

  log_file <- file.path(log_dir, "rdesk_startup.log")
  sink_conn <- file(log_file, open = "wt")
  sink(sink_conn, type = "message")

  cat(sprintf("[%s] RDesk startup initiated (TS Creator)\n", Sys.time()))
  cat("R Version:", R.version.string, "\n")
  cat("libPaths:\n")
  cat(paste("  -", .libPaths(), collapse = "\n"), "\n\n")
}

cleanup_bundle_logging <- function() {
  if (nzchar(Sys.getenv("R_BUNDLE_APP"))) {
    if (sink.number(type = "message") > 0) sink(type = "message")
    if (exists("sink_conn", inherits = FALSE) && isOpen(sink_conn)) close(sink_conn)
  }
}
on.exit(cleanup_bundle_logging(), add = TRUE)

tryCatch({
  app <- App$new(
    title = "TS Domain Creator (Native)",
    width = 1100,
    height = 800,
    www = file.path(app_dir, "www")
  )

  const_parmCD <- "STSTDTC"
  const_filename <- "ts.xpt"

  app$on_message("pick_folder", function(payload) {
    path <- app$dialog_open("Select Export Directory")
    if (!is.null(path)) {
      dir_path <- dirname(path)
      app$send("update_folder", list(path = dir_path))
    }
  })

  app$on_message("export", function(input) {
    if (is.null(input$studyID) || trimws(input$studyID) == "") {
      app$toast("Error: Study ID is mandatory.", type = "error")
      return()
    }

    selected_directory <- input$directoryPath
    if (is.null(selected_directory) || selected_directory == "") {
      app$toast("Error: Invalid directory path.", type = "error")
      return()
    }

    app$loading_start("Generating TS domain...")

    if (!dir.exists(selected_directory)) {
      tryCatch(dir.create(selected_directory, recursive = TRUE), error = function(e) {})
    }

    full_file_path <- file.path(selected_directory, const_filename)

    ts_val <- ""
    ts_val_nf <- "NA"

    if (input$useDate && !is.null(input$studyDate) && input$studyDate != "") {
      ts_val <- input$studyDate
      ts_val_nf <- ""
    }

    data <- data.frame(
      STUDYID = input$studyID,
      TSPARMCD = const_parmCD,
      TSVAL = ts_val,
      TSVALNF = ts_val_nf,
      stringsAsFactors = FALSE
    )

    label(data) <- "Trial Summary"
    label(data[["STUDYID"]]) <- "Study Identifier"
    label(data[["TSPARMCD"]]) <- "Trial Summary Parameter Short Name"
    label(data[["TSVAL"]]) <- "Parameter Value"
    label(data[["TSVALNF"]]) <- "Parameter Null Flavor"

    write_xpt(data, path = full_file_path, version = 5)

    app$loading_done()
    app$toast("Export successful!", type = "success")

    info <- file.info(full_file_path)
    info_str <- paste(
      "File Size: ", info$size, " bytes\n",
      "Last Modified: ", info$mtime, "\n",
      "Source: ", full_file_path
    )

    table_html <- paste0(
      '<table class="w-full text-sm text-left text-slate-500 border-collapse">',
      '<thead class="text-xs text-slate-700 uppercase bg-slate-50 border-b">',
      '<tr><th class="px-4 py-2">STUDYID</th><th class="px-4 py-2">TSPARMCD</th><th class="px-4 py-2">TSVAL</th><th class="px-4 py-2">TSVALNF</th></tr>',
      "</thead><tbody>",
      "<tr>",
      '<td class="px-4 py-3 border-b">', data$STUDYID, "</td>",
      '<td class="px-4 py-3 border-b">', data$TSPARMCD, "</td>",
      '<td class="px-4 py-3 border-b">', data$TSVAL, "</td>",
      '<td class="px-4 py-3 border-b">', data$TSVALNF, "</td>",
      "</tr>",
      "</tbody></table>"
    )

    app$send("results", list(
      info = info_str,
      table_html = table_html
    ))
  })

  app$run()

}, error = function(e) {
  if (nzchar(Sys.getenv("R_BUNDLE_APP"))) {
    cat(sprintf("\n[%s] CRITICAL ERROR:\n%s\n", Sys.time(), e$message))
  }
  stop(e)
})

# tests/manual/verify_ipc_browser.R
library(devtools)
load_all("c:/Users/Janak/OneDrive/Documents/RDesk/RDesk", recompile = FALSE)

# Use a fixed port for the browser test if possible, or just print it
port <- 54321 

app <- App$new(
  title  = "IPC Browser Verification",
  www    = system.file("templates", "hello", "www", package = "RDesk")
)

# During development fallback
if (app$.__enclos_env__$private$.www == "") {
  app$.__enclos_env__$private$.www <- file.path(getwd(), "inst", "templates", "hello", "www")
}

app$on_message("ping", function(msg) {
  message("[R] Received ping: ", jsonlite::toJSON(msg))
  app$send("pong", list(server_time = format(Sys.time())))
})

app$on_message("get_data", function(msg) {
  message("[R] Received get_data")
  app$send("data_result", list(nrow = 32, cols = c("mpg", "cyl")))
})

# We'll run the server for a fixed time or until a signal
app$on_ready(function() {
  message("VERIFY_URL: http://127.0.0.1:", port, "/?__rdesk_port__=", port)
})

# Override the random port for this test
# We'll monkeypatch rdesk_free_port or just set it in run()
# For simplicity, let's just let it be random and I'll catch the output.

app$run()

# tests/manual/test_phase8_multi.R
# Test Multi-Window and System Tray support
 
devtools::load_all(".")
 
# Window 1: Main Control
app1 <- App$new(title = "Primary Window", width = 800, height = 600)
 
app1$on_ready(function() {
  app1$set_tray(
    label = "RDesk Controller",
    on_click = function(btn) {
      message("[Tray] Clicked with: ", btn)
      app1$notify("Tray Click", paste("You used the", btn, "mouse button"))
    }
  )
})
 
# Window 2: Secondary View
app2 <- App$new(title = "Secondary Window", width = 400, height = 300)
 
# Launch both non-blocking
app1$run(block = FALSE)
app2$run(block = FALSE)
 
message("[Test] Both windows launched. Managing from single R thread...")
 
# Run our own service loop for 30 seconds
start_time <- Sys.time()
while (as.numeric(difftime(Sys.time(), start_time, units = "secs")) < 30) {
  rdesk_service()
  Sys.sleep(0.01)
}
 
message("[Test] Done.")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
# Verify RDesk is installed correctly
packageVersion("RDesk")
# Check that the example app is bundled (if existing)
app_path <- system.file("templates/hello", package = "RDesk")
if (nzchar(app_path)) {
  file.exists(app_path)
  list.files(app_path)
}

# inst/apps/mtcars_dashboard/R/server.R
# Message handlers and UI events

push_update <- function(app, env) {
  df <- env$filtered
  # Sourcing local helpers since they aren't in a package
  lapply(list.files(file.path(env$app_dir, "R"), pattern = "\\.R$", full.names = TRUE), source)
  
  app$send("data_update", list(
    kpis    = kpis(df),
    chart   = plot_to_b64(make_plot(df, env$x_var, env$y_var, env$plot_type)),
    summary = summary_stats(df),
    table   = df %>%
      dplyr::select(model, mpg, hp, wt, cyl) %>%
      head(15)
  ))
}

init_handlers <- function(app, env) {
  # -- navigation -----------------------------------------------------------
  # We use a simple section switcher in the UI
  app$on_message("nav", function(msg) {
    app$send("switch_section", list(section = msg$target))
  })

  # -- explorer -------------------------------------------------------------
  app$on_message("ready", function(msg) {
    push_update(app, env)
  })

  app$on_message("set_cyl_filter", async(function(msg) {
    env$cyl_filter <- as.numeric(unlist(msg$cyls))
    apply_filters(env)
    
    # Sourcing local helpers
    lapply(list.files(file.path(env$app_dir, "R"), pattern = "\\.R$", full.names = TRUE), source)
    
    list(
      kpis    = kpis(env$filtered),
      chart   = plot_to_b64(make_plot(env$filtered, env$x_var, env$y_var, env$plot_type)),
      summary = summary_stats(env$filtered),
      table   = env$filtered %>%
        dplyr::select(model, mpg, hp, wt, cyl) %>%
        head(15)
    )
  }, app = app, loading_message = "Filtering cars..."))

  app$on_message("toggle_plot_type", function(msg) {
    env$plot_type <- msg$type
    push_update(app, env)
  })

  # -- models ---------------------------------------------------------------
  app$on_message("run_model", async(function(payload) {
    # Full linear model
    model  <- lm(mpg ~ wt + cyl + hp, data = mtcars)
    model_summary <- summary(model)
    result <- as.data.frame(model_summary$coefficients)
    result$term <- rownames(result)
    rownames(result) <- NULL
    names(result) <- c("estimate", "std.error", "statistic", "p.value", "term")
    
    # Result for UI
    list(
      coefficients = result,
      r_squared    = round(model_summary$r.squared, 3),
      formula      = "mpg ~ wt + cyl + hp"
    )
  }, app = app, loading_message = "Fitting model..."))

  # -- export ---------------------------------------------------------------
  app$on_message("export_csv", function(msg) {
    path <- app$dialog_save(
      title        = "Export mtcars data",
      default_name = "rdesk_export.csv",
      filters      = list("CSV files" = "*.csv")
    )
    if (!is.null(path)) {
      write.csv(env$filtered, path, row.names = FALSE)
      app$toast(paste("Exported to", basename(path)), type = "success")
    }
  })

  # -- menu -----------------------------------------------------------------
  app$on_ready(function() {
    app$set_menu(list(
      File = list(
        "Reset App"     = function() {
          init_data(env)
          push_update(app, env)
          app$toast("App state reset", type = "info")
        },
        "---",
        "Exit"          = app$quit
      ),
      Section = list(
        "Explorer"      = function() app$send("__trigger__", list(action = "nav", target = "explorer")),
        "Models"        = function() app$send("__trigger__", list(action = "nav", target = "models")),
        "Export"        = function() app$send("__trigger__", list(action = "nav", target = "export"))
      ),
      Help = list(
        "About RDesk"   = function() {
          app$toast("RDesk v1.0.0: The native desktop framework for R.", type = "info")
        }
      )
    ))
  })
}

library(testthat)
library(RDesk)

test_that("App constructor validates mandatory arguments", {
  expect_error(App$new(width = 800, height = 600), "Title is mandatory")
  expect_error(App$new(title = "Test"), "Width is mandatory")
  expect_error(App$new(title = "Test", width = 800), "Height is mandatory")
})

test_that("App constructor validates resolution and paths", {
  withr::with_tempdir({
    # Test missing www path
    expect_error(App$new("Test", 800, 600, www = "nonexistent_dir"), "www directory not found")
    
    # Test valid initialization
    dir.create("www")
    app <- App$new("Test", 800, 600, www = "www")
    expect_s3_class(app, "App")

  })
})

test_that("App sets default options on load", {
  # These are set in .onLoad in zzz.R
  expect_false(is.null(getOption("rdesk.ipc_version")))
  expect_false(is.null(getOption("rdesk.async_backend")))
})

---
title: "Architecture"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Architecture}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
# Verify RDesk is installed correctly
packageVersion("RDesk")
# Check that the example app is bundled (if existing)
app_path <- system.file("templates/hello", package = "RDesk")
if (nzchar(app_path)) {
  file.exists(app_path)
  list.files(app_path)
}
```

RDesk is built on a "Source-to-Binary" philosophy that combines the flexibility of R with the security and performance of native Windows APIs.

## The Three Components

### 1. The Native Launcher (`launcher.cpp`)
A lightweight (66.5kb) C++ application that:
- Initializes the **WebView2** runtime.
- Establishes a **Zero-Port IPC** bridge using standard input/output (stdin/stdout).
- Manages the parent window, native menus, and system tray.
- Includes an **Anti-Zombie Watchdog** that ensures R processes are cleaned up when the UI closes.

### 2. The R Backend (`App.R`)
An R6-based event loop that:
- Runs a non-blocking service loop to process desktop messages.
- Manages the **Async Engine** for background task execution.
- Handles high-level desktop operations (File Dialogs, Clipboard, Notifications).

### 3. The Web Frontend (`www/`)
A standard HTML/JS/CSS surface that:
- Communicates with R via the global `window.rdesk` object.
- Pushes events to R using `rdesk.send(type, payload)`.
- Listens for R updates using `rdesk.on(type, callback)`.

## Zero-Port Design

Unlike traditional R web frameworks, RDesk does not open any network ports (`httpuv`, `WebSockets`).

1. **Security**: No firewall prompts or local network exposure.
2. **Reliability**: No "port in use" errors during multi-instance launches.
3. **Performance**: Native pipe communication is faster and more reliable than the loopback network stack.

## Process Lifecycle

When you run `app$run()`:
1. R spawns the native launcher as a child process.
2. The launcher opens the Windows window and starts the Chromium-based WebView2.
3. R enters a `while(private$.running)` loop, servicing the stdin/stdout pipes.
4. When the user closes the window, the launcher signals R to exit, ensuring a clean shutdown of all background worker nodes.

---
title: "Coming from Shiny"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Coming from Shiny}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
# Convert a data frame for JSON serialisation
result <- RDesk::rdesk_df_to_list(head(mtcars, 3))
length(result$rows)   # 3
result$cols[1:3]      # first three column names
```

If you know Shiny, you can learn RDesk in an afternoon.
Your R data logic moves across unchanged.
What changes is the delivery layer -- instead of a browser,
you get a native Windows window.

## The mental model shift

Shiny uses a **reactive graph** -- inputs change, outputs update automatically.
RDesk uses **explicit message passing** -- the UI sends a message,
R handles it and pushes a result back.

This feels more like writing an API handler than a Shiny server.

## Side-by-side patterns

### Responding to user input

**Shiny**
```r
server <- function(input, output, session) {
  output$plot <- renderPlot({
    filtered <- mtcars[mtcars$cyl == input$cyl, ]
    plot(filtered$wt, filtered$mpg)
  })
}
```

**RDesk**
```r
app$on_message("filter", async(function(payload) {
  filtered <- mtcars[mtcars$cyl == payload$cyl, ]
  list(chart = rdesk_plot_to_base64(
    plot(filtered$wt, filtered$mpg)
  ))
}, app = app))
```

The key difference: RDesk does not re-run automatically.
The UI explicitly calls `rdesk.send("filter", {cyl: 6})` and
receives `filter_result` back.

### Sending data to the UI

**Shiny**
```r
output$table <- renderTable({ mtcars })
```

**RDesk**
```r
# In R -- push data explicitly
app$send("table_data", list(
  rows = lapply(seq_len(nrow(mtcars)), function(i) as.list(mtcars[i,])),
  cols = names(mtcars)
))
```

```javascript
// In JavaScript -- receive it
rdesk.on("table_data", function(data) {
  renderTable(data.rows, data.cols);
});
```

### File downloads

**Shiny**
```r
output$download <- downloadHandler(
  filename = "data.csv",
  content  = function(file) write.csv(mtcars, file)
)
```

**RDesk**
```r
app$on_message("save_csv", function(payload) {
  path <- app$dialog_save(
    title   = "Save CSV",
    filters = "CSV files (*.csv)|*.csv"
  )
  if (!is.null(path)) write.csv(mtcars, path, row.names = FALSE)
})
```

### Native menus (no Shiny equivalent)

```r
app$set_menu(list(
  File = list(
    "Open..."  = function() app$dialog_open(),
    "Save..."  = function() app$dialog_save(),
    "---",
    "Exit"     = app$quit
  ),
  Help = list(
    "About"    = function() app$toast("MyApp v1.0", type = "info")
  )
))
```

### Async processing

**Shiny** (with future/promises)
```r
library(future)
plan(multisession)

observeEvent(input$run, {
  future({
    slow_model(input$data)
  }) %...>% (function(result) {
    output$result <- renderText(result)
  })
})
```

**RDesk**
```r
app$on_message("run_model", async(function(payload) {
  slow_model(payload$data)
}, app = app, loading_message = "Running model..."))
```

RDesk's `async()` handles the loading overlay, cancellation,
and result routing automatically.

## What you can reuse unchanged

Everything that does not touch Shiny inputs/outputs moves across directly:

- All data loading and transformation code
- All ggplot2 chart code (render with `rdesk_plot_to_base64()`)
- All statistical modelling code
- All file reading and writing code
- All helper functions

## What you must rewrite

| Shiny pattern | RDesk equivalent |
|---|---|
| `input$x` | `payload$x` inside `on_message()` |
| `reactive({...})` | Call helper functions explicitly |
| `renderPlot({...})` | `rdesk_plot_to_base64()` + `app$send()` |
| `renderTable({...})` | `app$send()` with list of rows |
| `observe({...})` | `app$on_message()` handler |
| `showNotification()` | `app$toast()` |

## Practical migration path

1. Keep all your data and modelling R files unchanged
2. Replace `ui.R` with `www/index.html` (plain HTML -- no Shiny DSL)
3. Replace `server.R` handlers one by one using `app$on_message()`
4. Replace `renderPlot` calls with `rdesk_plot_to_base64() + app$send()`
5. Run `source("app.R")` and test each handler as you migrate

---
title: "Getting Started with RDesk"
date: 2026-04-12
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting Started with RDesk}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r}
# Verify RDesk is installed correctly
packageVersion("RDesk")
# Check that the example app is bundled (if existing)
app_path <- system.file("templates/hello", package = "RDesk")
if (nzchar(app_path)) {
  file.exists(app_path)
}
```

## The "Last Mile" of R Deployment

Data analysts build powerful tools in R, but sharing them with non-technical users is difficult. Shiny is excellent for web dashboards, but it requires a server and a steady network connection.

**RDesk** solves this by turning your R code into a **native Windows desktop application**. It runs as a standalone `.exe` with zero network overhead and no server required.

## 1. Installation

RDesk works out-of-the-box on modern Windows systems. You only need **R** and **Rtools** installed.

```r
# Install from GitHub
# devtools::install_github("Janakiraman-311/RDesk")
library(RDesk)
```

## 2. Your First App (The One-Click Way)

The fastest way to build an RDesk application is using the built-in scaffolding tool. This generates a professional dashboard with a sidebar, KPI cards, and an asynchronous charting engine.

```r
library(RDesk)

# Create a professional dashboard in a new directory
RDesk::rdesk_create_app("MyDashboard")
```

![RDesk Hero Dashboard](../man/figures/landing.png)

Navigate to the `MyDashboard` folder and run `app.R` to see the results.

## 3. Building a Standalone Executable

The core value of RDesk is the ability to turn your R code into a standalone `.exe` that runs on any machine without requiring a pre-installed R version.

> **First build note:** The initial build takes 3-5 minutes as RDesk 
> downloads and bundles a portable R runtime. Subsequent builds 
> complete in under 30 seconds.

```r
# Build a portable ZIP bundle
RDesk::build_app(
  app_dir  = "MyDashboard",
  app_name = "MyProfessionalApp"
)
```

The resulting `.zip` file in the `dist/` folder contains exactly what you need to distribute your tool. The end-user simply unzips and runs `MyProfessionalApp.exe`.

### Using renv for reproducibility

If your project uses `renv`, commit your `renv.lock` in the project root. RDesk's GitHub Actions workflows will automatically restore from that lockfile before building. Additionally, `build_app()` writes a bundle-level `renv.lock` into each distributable when `renv` is active.

## 4. Key Concepts

### Zero-Port IPC
Zero ports, zero firewall issues. Communication happens over standard input/output using a high-speed JSON protocol.

### Async Processing
Use the `async()` wrapper to run heavy R tasks without freezing the UI:

```r
app$on_message("heavy_task", async(function(msg) {
  Sys.sleep(2) # Simulate work
  list(status = "Done")
}, app = app))
```

## Next Steps

*   [**Coming from Shiny**](shiny-migration.html) — Map your existing Shiny knowledge to the RDesk mental model.
*   [**Cookbook**](cookbook.html) — Common recipes for file dialogs, charts, and native menus.
*   [**Async Guide**](async-guide.html) — Deep dive into background task management.

---
title: "Cookbook"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Cookbook}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
# Build a valid IPC message envelope
msg <- RDesk::rdesk_message("get_data", list(filter = "cyl == 6"))
str(msg)
```

Copy-paste recipes for the most common RDesk patterns.

## Load a CSV with a file dialog

```r
app$on_message("load_file", function(payload) {
  path <- app$dialog_open(
    title   = "Open CSV",
    filters = list("CSV files" = "*.csv")
  )
  if (is.null(path)) return(invisible(NULL))

  df <- utils::read.csv(path, stringsAsFactors = FALSE)
  app$send("file_loaded", list(
    rows     = nrow(df),
    cols     = names(df),
    filename = basename(path)
  ))
})
```

## Render a ggplot2 chart

![RDesk Explorer Boxplot](../man/figures/explorer_boxplot.png)
![RDesk Models Page](../man/figures/models.png)

```r
app$on_message("get_chart", async(function(payload) {
  p <- ggplot2::ggplot(mtcars,
         ggplot2::aes(wt, mpg, colour = factor(cyl))) +
       ggplot2::geom_point(size = 3) +
       ggplot2::theme_minimal()

  list(chart = rdesk_plot_to_base64(p))
}, app = app))
```

```javascript
rdesk.on("get_chart_result", function(data) {
  document.getElementById("chart").src =
    "data:image/png;base64," + data.chart;
});
```

## Show a toast notification

```r
app$toast("File saved successfully", type = "success")
app$toast("Could not connect",       type = "error")
app$toast("Update available",        type = "info")
```

## Save a file with a dialog

```r
app$on_message("export_csv", function(payload) {
  path <- app$dialog_save(
    title        = "Save CSV",
    filters      = list("CSV files" = "*.csv"),
    default_name = "export.csv"
  )
  if (is.null(path)) return(invisible(NULL))

  write.csv(mtcars, path, row.names = FALSE)
  app$toast(paste("Saved to", basename(path)), type = "success")
})
```

## Run a slow computation without freezing the UI

```r
app$on_message("run_analysis", async(function(payload) {
  result <- slow_function(payload$data)
  list(
    summary = as.list(summary(result)),
    n       = length(result)
  )
}, app = app, loading_message = "Analysing..."))
```

## Add a native menu

```r
app$on_ready(function() {
  app$set_menu(list(
    File = list(
      "Open..."   = function() app$send("load_file", list()),
      "Export..." = function() app$send("export_csv", list()),
      "---",
      "Exit"      = app$quit
    ),
    View = list(
      "Refresh"   = function() app$send("refresh", list()),
      "Dark mode" = function() app$send("toggle_theme", list())
    ),
    Help = list(
      "About"     = function() {
        app$toast("MyApp v1.0.0 -- built with RDesk", type = "info")
      }
    )
  ))
})
```

## Check for and install updates

```r
# In app.R, before app$run()
rdesk_auto_update(
  current_version = "1.0.0",
  version_url     = "https://yourserver.com/latest.txt",
  download_url    = "https://yourserver.com/MyApp-setup.exe",
  app             = app
)
```

Host a plain text file at `version_url` containing only the latest
version string, e.g. `1.1.0`. RDesk checks it silently on launch
and installs the update if a newer version is available.

## Send data when the app starts

```r
app$on_ready(function() {
  df <- load_initial_data()
  app$send("data_ready", list(
    rows = lapply(seq_len(nrow(df)), function(i) as.list(df[i,])),
    cols = names(df),
    n    = nrow(df)
  ))
})
```

## Handle errors gracefully

```r
app$on_message("risky_operation", async(function(payload) {
  tryCatch({
    result <- operation_that_might_fail(payload)
    list(success = TRUE, data = result)
  }, error = function(e) {
    list(success = FALSE, error = e$message)
  })
}, app = app))
```

```javascript
rdesk.on("risky_operation_result", function(data) {
  if (!data.success) {
    showError(data.error);
    return;
  }
  renderResult(data.data);
});
```

## Detect if running as a built app vs dev mode

```r
if (rdesk_is_bundle()) {
  # Running as distributed exe
  data_path <- file.path(getwd(), "data", "config.json")
} else {
  # Running in development via source("app.R")
  data_path <- file.path(app_dir, "data", "config.json")
}
```

---
title: "AI Coding Assistant"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{AI Coding Assistant}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# RDesk AI Skill File v1.0.4
# For use with AI coding assistants working on RDesk applications
---

## WHAT YOU ARE

You are an expert assistant for the RDesk R package. RDesk builds native
Windows desktop applications using R for logic and HTML/CSS/JS for the
interface. You help developers build, debug, migrate, and extend RDesk apps.

Before writing any code, read this entire file. Every section contains
rules that affect correctness. Violating them produces bugs that are
hard to diagnose.

---

## SECTION 1 — ARCHITECTURE OVERVIEW

RDesk has five layers. Understand all five before touching any code.

```
Layer 5: App code         (R + HTML/JS — what developers write)
Layer 4: Plugin API       (async(), rdesk_auto_update(), rdesk_watch())
Layer 3: R Core API       (App R6 class — app$send, app$on_message)
Layer 2: IPC bridge       (PostWebMessageAsString + stdin/stdout pipe)
Layer 1: Native shell     (C++ launcher + WebView2 — never touch this)
```

RULE: Developers only write Layer 5. Never suggest modifying Layer 1 or 2.
RULE: Layer 1 C++ is compiled automatically during install.packages(). 
      Never suggest manual compilation.
RULE: Communication between R and JavaScript is always explicit. 
      There is no automatic reactivity. The developer must explicitly 
      send and receive every message.

### Process model

```
[rdesk-launcher.exe]          [R process]
  WebView2 window    <-IPC->   App$run() event loop
  HTML/CSS/JS UI               on_message() handlers
  rdesk.js bridge              async() workers (mirai/callr)
```

The launcher and R are separate OS processes. They communicate via:
- R to JS: app$send("type", payload) -> PostWebMessageAsString
- JS to R: rdesk.send("type", payload) -> stdin JSON pipe

Zero TCP ports are opened at any point. No httpuv. No WebSocket.

---

## SECTION 2 — IPC CONTRACT (NEVER BREAK THIS)

Every message has this exact envelope. Both directions. Always.

```json
{
  "id":        "msg_abc123",
  "type":      "action_name",
  "version":   "1.0",
  "payload":   {},
  "timestamp": 1234567890.123
}
```

RULE: Never construct raw JSON manually. Always use:
- R side:  app$send("type", list(...))   or   rdesk_message("type", list(...))
- JS side: rdesk.send("type", {...})

RULE: Handler return values are automatically wrapped in the envelope.
      Return a plain list from on_message() handlers. Never return 
      a pre-wrapped envelope.

RULE: The result of a message of type "foo" arrives in JS as "foo_result".
```javascript
// Send from JS
rdesk.send("get_data", { filter: "cyl == 6" });

// Receive result in JS  
rdesk.on("get_data_result", function(data) {
  renderTable(data.rows, data.cols);
});
```

RULE: System message types start with double underscore: __loading__,
      __progress__, __reload_ui__. Never use __ prefix for app messages.

---

## SECTION 3 — APP STRUCTURE

Every RDesk app has this exact structure. Never deviate.

```
MyApp/
├── app.R              <- entry point, keep thin
├── DESCRIPTION        <- app metadata
├── R/
│   ├── server.R       <- on_message() handlers (edit this most)
│   ├── data.R         <- data loading and transformation
│   └── plots.R        <- chart rendering helpers
└── www/
    ├── index.html     <- UI markup
    ├── css/
    │   └── style.css
    └── js/
        ├── app.js     <- UI event handling
        └── rdesk.js   <- IPC bridge (NEVER edit this file)
```

### Minimal app.R

```r
app_dir <- tryCatch(
  if (nzchar(Sys.getenv("R_BUNDLE_APP"))) getwd()
  else dirname(rstudioapi::getActiveDocumentContext()$path),
  error = function(e) getwd()
)

library(RDesk)

lapply(
  list.files(file.path(app_dir, "R"), pattern = "\\.R$", full.names = TRUE),
  source
)

app <- App$new(
  title  = "My App",
  width  = 1100L,
  height = 740L,
  www    = file.path(app_dir, "www")
)

init_handlers(app)
app$run()
```

RULE: app.R must stay thin. All handler logic goes in R/server.R.
RULE: Always source R/ files with lapply + list.files. Never use 
      source() on individual files by name — it breaks hot reload.
RULE: The function init_handlers(app) is the contract between app.R 
      and server.R. Always define it in server.R.

---

## SECTION 4 — HANDLERS (on_message)

### Basic handler

```r
# In R/server.R
init_handlers <- function(app) {

  app$on_ready(function() {
    # Runs once when window is ready
    # Set menu, send initial data here
    df <- init_data()
    app$send("data_ready", rdesk_df_to_list(df))
  })

  app$on_message("get_data", function(payload) {
    df <- load_data(payload$filter)
    rdesk_df_to_list(df)
  })

}
```

RULE: Every handler must return a value or NULL. 
      The return value is sent back as type_result.
RULE: Never call app$send() inside a plain on_message() handler to 
      return the response. Return the list directly. 
      app$send() is for pushing unsolicited updates.
RULE: Payload fields arrive as R list elements: payload$field_name.
      Never use payload[["field_name"]] — use $ accessor for clarity.

### Accessing payload

```r
# JavaScript sends:
# rdesk.send("filter", { column: "cyl", value: 6, ascending: true })

app$on_message("filter", function(payload) {
  col  <- payload$column     # "cyl"
  val  <- payload$value      # 6
  asc  <- payload$ascending  # TRUE
  
  df <- mtcars[mtcars[[col]] == val, ]
  if (!asc) df <- df[nrow(df):1, ]
  
  rdesk_df_to_list(df)
})
```

### on_ready handler

```r
app$on_ready(function() {
  # Set native menu
  app$set_menu(list(
    File = list(
      "Open..."  = function() app$send("open_file", list()),
      "Save..."  = function() app$send("save_file", list()),
      "---",
      "Exit"     = app$quit
    ),
    Help = list(
      "About"    = function() app$toast("MyApp v1.0", type = "info")
    )
  ))
  
  # Send initial data
  app$send("init_data", list(
    data    = rdesk_df_to_list(init_data()),
    version = "1.0.0"
  ))
})
```

---

## SECTION 5 — ASYNC ENGINE

RDesk has three tiers of async. Use the right one.

### Tier 1 — async() wrapper (use this for 95% of cases)

```r
app$on_message("heavy_task", async(function(payload) {
  # Runs in background mirai/callr worker
  # UI stays responsive
  # Loading overlay shown automatically
  result <- slow_computation(payload$data)
  list(result = result, n = length(result))
}, app = app, loading_message = "Computing..."))
```

RULE: async() captures the packages loaded at registration time and 
      reloads them in the worker. If your handler uses a package, 
      library() it before calling async() or add it to DESCRIPTION Imports.

RULE: Do NOT access app$ methods inside an async() worker. 
      The App object lives in the main process. Workers cannot reach it.
      Return data from the worker. app$send() happens automatically.

RULE: Do NOT use global variables inside async() workers. 
      Pass everything via payload or capture in the closure explicitly.

```r
# WRONG - global variable access in worker
my_data <- load_data()
app$on_message("process", async(function(payload) {
  process(my_data)  # my_data not available in worker
}, app = app))

# CORRECT - pass via payload or capture explicitly
my_data <- load_data()
app$on_message("process", async(function(payload) {
  process(payload$data)  # sent from JS
}, app = app))

# OR - capture explicitly in closure
local_data <- my_data
app$on_message("process", async(function(payload) {
  process(local_data)  # captured in closure, serialised to worker
}, app = app))
```

### Tier 2 — rdesk_async() (explicit control)

```r
job_id <- rdesk_async(
  task     = function(data) slow_model(data),
  args     = list(data = my_data),
  on_done  = function(result) app$send("model_done", result),
  on_error = function(err)    app$toast(err$message, type = "error")
)

app$loading_start("Running model...", cancellable = TRUE, job_id = job_id)
```

### Tier 3 — mirai direct (expert use only)

```r
m <- mirai::mirai(
  slow_fn(x),
  slow_fn = slow_fn,
  x = my_data
)
```

### async_progress() — real-time updates from workers

```r
app$on_message("long_process", async(function(payload) {
  items <- payload$items
  for (i in seq_along(items)) {
    process_item(items[[i]])
    async_progress(
      value   = round(i / length(items) * 100),
      message = paste0("Processing ", i, " of ", length(items))
    )
  }
  list(done = TRUE, processed = length(items))
}, app = app, loading_message = "Starting..."))
```

---

## SECTION 6 — JAVASCRIPT PATTERNS

### rdesk.js API — complete reference

```javascript
// Send message to R, returns Promise
rdesk.send("message_type", { key: "value" })
  .then(function(result) { /* handle result */ })
  .catch(function(err)   { /* handle error */ });

// Listen for messages pushed from R
rdesk.on("message_type", function(data) { /* handle */ });

// Run code when app is ready
rdesk.ready(function() {
  rdesk.send("get_initial_data", {});
});

// Remove a listener
rdesk.off("message_type");
```

### Standard UI patterns

```javascript
// Render a base64 chart from R
rdesk.on("chart_result", function(data) {
  document.getElementById("chart").src =
    "data:image/png;base64," + data.chart;
});

// Render a table from R
rdesk.on("data_result", function(data) {
  var head = document.getElementById("thead");
  var body = document.getElementById("tbody");
  
  head.innerHTML = "<tr>" + 
    data.cols.map(function(c) { return "<th>" + c + "</th>"; }).join("") + 
    "</tr>";
    
  body.innerHTML = data.rows.map(function(row) {
    return "<tr>" + 
      data.cols.map(function(c) { 
        return "<td>" + (row[c] !== undefined ? row[c] : "") + "</td>"; 
      }).join("") + 
      "</tr>";
  }).join("");
});

// Handle loading state
rdesk.on("__loading__", function(state) {
  var overlay = document.getElementById("loading-overlay");
  if (overlay) overlay.style.display = state.active ? "flex" : "none";
});

// Send on button click
document.getElementById("btn-refresh").addEventListener("click", function() {
  rdesk.send("get_data", { filter: getCurrentFilter() });
});
```

RULE: Always use rdesk.ready() to wrap any rdesk.send() calls that 
      fire on page load. Without it the IPC bridge may not be 
      initialised yet.

RULE: Never use fetch(), XMLHttpRequest, or WebSocket to communicate 
      with R. Only rdesk.send() and rdesk.on().

RULE: rdesk.js is in www/js/rdesk.js. Never edit it. 
      Never import it from a CDN — it must be the local copy.

---

## SECTION 7 — CHARTS AND DATA

### Plot to base64

```r
# In R/plots.R
make_scatter <- function(df, x_var = "wt", y_var = "mpg") {
  p <- ggplot2::ggplot(df, ggplot2::aes(.data[[x_var]], .data[[y_var]])) +
    ggplot2::geom_point(size = 3, alpha = 0.8, colour = "#378ADD") +
    ggplot2::geom_smooth(method = "lm", se = FALSE,
                          colour = "#1D9E75", linewidth = 0.8) +
    ggplot2::theme_minimal(base_size = 13)
  rdesk_plot_to_base64(p)
}
```

```r
# In handler
app$on_message("get_chart", async(function(payload) {
  df <- filter_data(payload)
  list(chart = make_scatter(df, payload$x_var, payload$y_var))
}, app = app))
```

```javascript
// In app.js
rdesk.on("get_chart_result", function(data) {
  document.getElementById("main-chart").src =
    "data:image/png;base64," + data.chart;
});
```

### Data frame to list

```r
# rdesk_df_to_list() converts a data frame to JSON-serialisable list
result <- rdesk_df_to_list(mtcars)
# result$rows  -> list of row lists
# result$cols  -> character vector of column names

# In handler
app$on_message("get_table", function(payload) {
  df <- filter_data(payload)
  rdesk_df_to_list(df)
})
```

---

## SECTION 8 — NATIVE FEATURES

### File dialogs

```r
# Open file
app$on_message("open_file", function(payload) {
  path <- app$dialog_open(
    title   = "Open Data File",
    filters = "CSV files (*.csv)|*.csv|All files (*.*)|*.*"
  )
  if (is.null(path)) return(list(cancelled = TRUE))
  
  df <- utils::read.csv(path, stringsAsFactors = FALSE)
  c(list(cancelled = FALSE, filename = basename(path)),
    rdesk_df_to_list(df))
})

# Save file
app$on_message("save_file", function(payload) {
  path <- app$dialog_save(
    title   = "Save Results",
    filters = "CSV files (*.csv)|*.csv",
    default = "results.csv"
  )
  if (is.null(path)) return(list(cancelled = TRUE))
  
  write.csv(reconstruct_df(payload$data), path, row.names = FALSE)
  list(cancelled = FALSE, saved_to = basename(path))
})
```

### Toasts and notifications

```r
app$toast("Operation complete", type = "success")  # green
app$toast("File not found",     type = "error")    # red
app$toast("Update available",   type = "info")     # blue
app$toast("Check your inputs",  type = "warning")  # amber
```

### Loading overlay

```r
# Manual control (use async() instead when possible)
app$loading_start("Processing data...", cancellable = TRUE, job_id = "job_1")
app$loading_progress(45)
app$loading_progress(90, message = "Almost done...")
app$loading_done()
```

### Native menus

```r
app$set_menu(list(
  File = list(
    "New..."     = function() app$send("new_doc", list()),
    "Open..."    = function() app$send("open_file", list()),
    "---",
    "Exit"       = app$quit
  ),
  View = list(
    "Refresh"    = function() app$send("refresh", list()),
    "Full screen"= function() app$maximize()
  ),
  Help = list(
    "Documentation" = function() {
      shell.exec("https://janakiraman-311.github.io/RDesk/")
    },
    "About"         = function() {
      app$toast("MyApp v1.0.0 -- built with RDesk", type = "info")
    }
  )
))
```

---

## SECTION 9 — BUILD AND DISTRIBUTION

### Build a distributable

```r
RDesk::build_app(
  app_dir         = "path/to/MyApp",
  app_name        = "MyApp",
  out_dir         = tempdir(),        # or your dist folder
  build_installer = TRUE              # creates .exe installer
)
# Output: dist/MyApp-1.0.0-setup.exe (68-200MB depending on packages)
```

### Detect bundle vs development mode

```r
if (rdesk_is_bundle()) {
  # Running as distributed exe
  config_path <- file.path(getwd(), "config.json")
} else {
  # Running in development via source("app.R")
  config_path <- file.path(app_dir, "config.json")
}
```

### Auto-update

```r
# In app.R, before app$run()
rdesk_auto_update(
  current_version = "1.0.0",
  version_url     = "https://example.com/myapp/latest.txt",
  download_url    = "https://example.com/myapp/MyApp-setup.exe",
  app             = app
)
```

Host a plain text file at version_url containing only the version 
string e.g. "1.1.0". RDesk checks silently on launch and downloads 
and installs the update if a newer version is available.

---

## SECTION 10 — SHINY MIGRATION REFERENCE

### Pattern mapping

| Shiny pattern         | RDesk equivalent                        |
|-----------------------|-----------------------------------------|
| input$x               | payload$x inside on_message()           |
| reactive({...})       | plain R function called explicitly      |
| renderPlot({...})     | rdesk_plot_to_base64() + return list    |
| renderTable({...})    | rdesk_df_to_list() + return list        |
| observe({...})        | app$on_message() handler                |
| observeEvent(input$x) | app$on_message("x_changed", ...)        |
| showNotification()    | app$toast()                             |
| withProgress()        | async() with loading_message=           |
| downloadHandler()     | app$dialog_save() in on_message()       |
| fileInput()           | app$dialog_open() in on_message()       |
| updateSelectInput()   | app$send("update_select", list(...))    |
| session$sendMessage() | app$send("type", payload)               |
| shinyjs::show()       | app$send("show_element", list(id=...)) |

### Migration sequence

1. Copy all pure R functions (data loading, modelling, helpers) unchanged
2. Map every input$x to a JS rdesk.send("x_changed", {value: ...})
3. Map every render*() to an on_message() that returns the data
4. Rewrite ui.R as www/index.html using plain HTML
5. Replace renderPlot with rdesk_plot_to_base64() in handlers
6. Replace renderTable with rdesk_df_to_list() in handlers
7. Replace fileInput/downloadHandler with dialog_open/dialog_save
8. Wrap slow handlers in async()
9. Add native menu with app$set_menu() in on_ready()

### Complete migration example

```r
# SHINY server.R
server <- function(input, output, session) {
  filtered <- reactive({
    mtcars[mtcars$cyl == input$cyl_filter, ]
  })
  output$scatter <- renderPlot({
    ggplot2::ggplot(filtered(), ggplot2::aes(wt, mpg)) +
      ggplot2::geom_point()
  })
  output$table <- renderTable({ filtered() })
}
```

```r
# RDESK R/server.R
init_handlers <- function(app) {

  app$on_ready(function() {
    app$send("data_ready", rdesk_df_to_list(mtcars))
  })

  app$on_message("filter_changed", async(function(payload) {
    df <- mtcars[mtcars$cyl == payload$cyl_filter, ]
    p  <- ggplot2::ggplot(df, ggplot2::aes(wt, mpg)) +
            ggplot2::geom_point()
    list(
      chart = rdesk_plot_to_base64(p),
      table = rdesk_df_to_list(df)
    )
  }, app = app))

}
```

```javascript
// RDESK www/js/app.js
rdesk.ready(function() {
  rdesk.send("get_data", {});
});

document.getElementById("cyl-filter").addEventListener("change", function() {
  rdesk.send("filter_changed", { cyl_filter: parseInt(this.value) });
});

rdesk.on("filter_changed_result", function(data) {
  document.getElementById("chart").src =
    "data:image/png;base64," + data.chart;
  renderTable(data.table.rows, data.table.cols);
});
```

---

## SECTION 11 — COMMON MISTAKES AND FIXES

### Mistake 1 — calling app$ inside async() worker

```r
# WRONG
app$on_message("task", async(function(payload) {
  result <- compute(payload)
  app$toast("Done!")      # ERROR: app not available in worker
  app$send("done", result) # ERROR: same reason
}, app = app))

# CORRECT
app$on_message("task", async(function(payload) {
  result <- compute(payload)
  list(result = result)   # return value is sent automatically
}, app = app))
# toast after completion: use on_done callback in rdesk_async()
```

### Mistake 2 — forgetting flush(stdout())

```r
# WRONG - in bundled mode, messages may buffer
cat(jsonlite::toJSON(msg), "\n")

# CORRECT - always flush after writing to stdout
cat(jsonlite::toJSON(msg), "\n")
flush(stdout())

# Note: app$send() handles this automatically.
# Only matters if you write raw cat() calls.
```

### Mistake 3 — sourcing individual R files by name

```r
# WRONG - breaks hot reload and is fragile
source("R/server.R")
source("R/data.R")

# CORRECT - sources all files, order-safe
lapply(
  list.files(file.path(app_dir, "R"), pattern = "\\.R$", full.names = TRUE),
  source
)
```

### Mistake 4 — writing to getwd() in examples or defaults

```r
# WRONG - CRAN policy violation
build_app(app_dir = "MyApp", out_dir = "dist")

# CORRECT - always write to tempdir() in examples
build_app(app_dir = "MyApp", out_dir = file.path(tempdir(), "dist"))
```

### Mistake 5 — using installed.packages()

```r
# WRONG - slow, CRAN policy violation
if ("ggplot2" %in% installed.packages()[,"Package"]) { ... }

# CORRECT
if (requireNamespace("ggplot2", quietly = TRUE)) { ... }
```

### Mistake 6 — no on.exit() after setwd()

```r
# WRONG
setwd(build_dir)
# ... do work ...
setwd(original_dir)  # never runs if error occurs

# CORRECT
oldwd <- getwd()
on.exit(setwd(oldwd), add = TRUE)
setwd(build_dir)
```

---

## SECTION 12 — DEVELOPMENT WORKFLOW

### Daily development loop

```r
# 1. Open project in RStudio
# 2. Source the app
source("app.R")
# Window opens

# 3. Make changes to R/server.R or www/
# 4. Close window, re-source
# With hot reload (Phase 2):
# rdesk_watch(app)  # auto-reloads on file change
```

### Testing handlers without a window

```r
# Set CI mode to test handlers without launching a window
options(rdesk.ci_mode = TRUE)
library(RDesk)

# Handlers can be unit tested
source("R/server.R")
# test individual functions from data.R and plots.R directly
```

### Build verification before distribution

```r
# Always verify before sending to users
pkg <- build_app(
  app_dir  = "MyApp",
  app_name = "MyApp",
  out_dir  = tempdir()
)
# Test the output ZIP manually before building installer
```

---

## SECTION 13 — QUICK REFERENCE CARD

```r
# Create new app
RDesk::rdesk_create_app("MyApp")

# Core app setup
app <- App$new(title = "Title", width = 1100L, height = 740L, www = "www/")
app$on_ready(function() { ... })
app$on_message("type", function(payload) { list(...) })
app$run()

# Send data to UI
app$send("type", list(key = value))

# Async handler
app$on_message("type", async(function(payload) {
  list(result = compute(payload))
}, app = app, loading_message = "Working..."))

# Chart
rdesk_plot_to_base64(ggplot_object)

# Table
rdesk_df_to_list(data_frame)

# IPC message
rdesk_message("type", list(key = value))
rdesk_parse_message(json_string)

# Dialogs
app$dialog_open(title = "Open", filters = "CSV|*.csv")
app$dialog_save(title = "Save", filters = "CSV|*.csv")
app$dialog_folder(title = "Select folder")

# Notifications
app$toast("message", type = "success|error|info|warning")
app$notify("title", "body")

# Loading
app$loading_start("message", cancellable = TRUE, job_id = "id")
app$loading_progress(50)
app$loading_done()

# Jobs
rdesk_async(task, args, on_done, on_error)
rdesk_cancel_job("job_id")
rdesk_jobs_pending()

# Build
build_app(app_dir, app_name, out_dir = tempdir(), build_installer = FALSE)

# Detect mode
rdesk_is_bundle()  # TRUE in distributed app, FALSE in development

# Updates
rdesk_auto_update(current_version, version_url, download_url, app)

# JavaScript
rdesk.send("type", payload)          # returns Promise
rdesk.on("type", function(data){})   # listen for messages
rdesk.ready(function(){})            # run when app ready
rdesk.off("type")                    # remove listener
```

---

## SECTION 14 — PACKAGE INFORMATION

```
Package:    RDesk
Version:    1.0.4
CRAN:       https://cran.r-project.org/package=RDesk
GitHub:     https://github.com/Janakiraman-311/RDesk
Docs:       https://janakiraman-311.github.io/RDesk/
Maintainer: Janakiraman G <janakiraman.bt@gmail.com>
License:    MIT
OS:         Windows 10 or later only (v1.0.x)
Requires:   Rtools44+, WebView2 Runtime

Install:    install.packages("RDesk")
Dev:        devtools::install_github("Janakiraman-311/RDesk")
```

---

END OF SKILL FILE

---
title: "Async & Background Tasks in RDesk"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Async & Background Tasks in RDesk}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
# RDesk async functions work safely in non-interactive mode
RDesk::rdesk_jobs_pending()  # returns 0 when no jobs running
```

## The three tiers of async in RDesk

RDesk provides three levels of async support. Start at Tier 1.
Move to Tier 2 or 3 only when you need something Tier 1 cannot do.

![RDesk Loading Overlay & Progress Hook](../man/figures/loading.png)

### Tier 1 -- async() wrapper (recommended)

Wrap any message handler with `async()`. RDesk handles everything else.

```r
app$on_message("calculate_total", async(function(payload) {
  # This runs in a background worker
  result <- sum(payload$values)
  list(total = result)
}, loading_message = "Calculating..."))
```

**What async() does for you automatically:**
- Runs your function in a background worker (non-blocking).
- Shows a loading overlay with a cancel button.
- Reloads your packages in the worker context.
- Sends the result back as `<type>_result`.
- Shows an error toast if something goes wrong.

### Tier 2 -- rdesk_async() (manual control)

Use `rdesk_async()` when you need manual control over the UI state or multiple progress updates:

```r
app$on_message("long_job", function(payload) {
  job_id <- rdesk_async(
    task = function(x) { 
       # Perform multiple steps, each reporting progress
       for(i in 1:5) { 
         Sys.sleep(1) 
         # Note: You can send progress manually or 
         # rely on the final on_done. 
       }
       x * 2 
    },
    args = list(x = 10),
    on_done = function(res) {
      # Hide overlay when task finishes
      app$send("__loading__", list(active = FALSE))
      app$send("job_finished", list(result = res))
    }
  )
  # Show overlay with explicit message and job_id for cancellation
  app$send("__loading__", list(
    active = TRUE, 
    message = "Starting long job...", 
    job_id = job_id,
    progress = 10 # Initial progress
  ))
})
```

### Tier 3 -- mirai direct (expert use)

Use `mirai::mirai()` directly when you need:
- Custom daemon counts per task type.
- Task chaining or pipelines.
- Integration with other mirai-aware packages.

## One rule that applies to all three tiers

Background workers are completely isolated from the main R session.
They cannot see your variables, your loaded data, or your functions
unless you pass them explicitly through `args`.

This is not a limitation -- it makes your code reproducible and safe.

---
title: "RDesk IPC Message Contract"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{RDesk IPC Message Contract}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
# Parse a valid IPC message
raw <- '{"id":"msg_001","type":"get_data","version":"1.0","payload":{},"timestamp":1234567890}'
msg <- RDesk::rdesk_parse_message(raw)
msg$type
```

# RDesk IPC Message Contract (v1.0)

This document defines the standard JSON envelope used for all communication between the R backend and the HTML/JS frontend in RDesk applications.

## The Message Envelope

Every message sent across the native IPC bridge (WebView2 to stdin/stdout) must be a JSON object with the following structure:

```json
{
  "id": "msg_abc123",
  "type": "action_name",
  "version": "1.0",
  "payload": { 
    "key": "value" 
  },
  "timestamp": 1742123456.789
}
```

### Field Definitions

*   **`id`**: A unique string identifier for the message (e.g., for tracing or request/response correlation).
*   **`type`**: The internal name of the message or action. This triggers handlers on the receiving end.
*   **`version`**: The version of the message contract (currently "1.0").
*   **`payload`**: A JSON object containing the actual data for the message.
*   **`timestamp`**: Seconds since the Unix epoch (including decimal fractions).

## Usage in R

When sending from R to the UI, use `app$send()`:

```r
app$send("update_ui", list(status = "Success"))
```

RDesk automatically wraps your payload in the standard envelope before transmission.

## Usage in JavaScript

When sending from the UI to R, use `rdesk.send()`:

```javascript
rdesk.send("button_click", { id: "submit_btn" });
```

The `rdesk.js` library automatically constructs the envelope with a unique ID and current timestamp.

## Error Handling

The parser `rdesk_parse_message()` is designed to be defensive. It will return `NULL` if:
1. The input is not valid JSON.
2. The envelope is missing required fields (`type` or `payload`).

In these cases, a `warning()` is emitted with the specific failure reason.

### Defensive Pattern for Developers

If you are building custom extensions or handlers, always check for `NULL` before processing:

```r
msg <- rdesk_parse_message(raw_json)
if (is.null(msg)) {
  # Log error or silently ignore malformed traffic
  return(invisible(NULL))
}

# Safely access msg$type and msg$payload here
```

## The async result convention

When using `async()`, the result is automatically sent back to the UI
as a message of type `<original_type>_result`.

For example:

| R registers | JS sends | JS receives |
|---|---|---|
| `on_message("filter_cars", async(...))` | `rdesk.send("filter_cars", payload)` | `rdesk.on("filter_cars_result", fn)` |
| `on_message("run_model", async(...))` | `rdesk.send("run_model", payload)` | `rdesk.on("run_model_result", fn)` |

This convention means the JS developer always knows which event to
listen for without reading the R code. It is consistent and predictable.

### Overriding the result type

If you need a different result type, use `rdesk_async()` directly
(Tier 2) and call `app$send()` explicitly in your `on_done` callback.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{rdesk_resolve_www}
\alias{rdesk_resolve_www}
\title{Resolve the www directory for an app}
\usage{
rdesk_resolve_www(www_dir)
}
\arguments{
\item{www_dir}{User-provided path to www directory (character)
Passing an explicit absolute path is the most reliable option and skips
the best-effort call-stack search.}
}
\value{
Normalized absolute path to a valid www directory
}
\description{
Resolve the www directory for an app
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/async.R
\name{rdesk_cancel_job}
\alias{rdesk_cancel_job}
\title{Cancel a running background job}
\usage{
rdesk_cancel_job(job_id)
}
\arguments{
\item{job_id}{The ID of the job to cancel.}
}
\value{
Invisible TRUE if cancelled, FALSE if not found.
}
\description{
Cancel a running background job
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/async.R
\name{rdesk_start_daemons}
\alias{rdesk_start_daemons}
\title{Start the mirai daemon pool}
\usage{
rdesk_start_daemons()
}
\value{
Invisible number of workers started.
}
\description{
Called once at App$run() startup when rdesk.async_backend is "mirai".
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/window.R
\name{rdesk_req_id}
\alias{rdesk_req_id}
\title{Generate a unique request ID for dialog round-trips}
\usage{
rdesk_req_id()
}
\value{
A character string ID
}
\description{
Generate a unique request ID for dialog round-trips
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scaffold.R
\name{rdesk_scaffold_success_msg}
\alias{rdesk_scaffold_success_msg}
\title{Internal success message}
\usage{
rdesk_scaffold_success_msg(name, app_dir, data_source, viz_type, use_async)
}
\description{
Internal success message
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/window.R
\name{rdesk_close_window}
\alias{rdesk_close_window}
\title{Close the native window process}
\usage{
rdesk_close_window(proc)
}
\arguments{
\item{proc}{The processx process object returned by rdesk_open_window}
}
\description{
Close the native window process
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scaffold.R
\name{rdesk_create_app}
\alias{rdesk_create_app}
\title{Create a new RDesk application}
\usage{
rdesk_create_app(
  name,
  path = tempdir(),
  data_source = NULL,
  viz_type = NULL,
  use_async = NULL,
  theme = "light",
  open = TRUE
)
}
\arguments{
\item{name}{App name. Must be a valid directory name.}

\item{path}{Directory to create the app in. Default is current directory.}

\item{data_source}{Internal use. Defaults to "builtin".}

\item{viz_type}{Internal use. Defaults to "mixed".}

\item{use_async}{Internal use. Defaults to TRUE.}

\item{theme}{One of "light", "dark", "system". Default "system".}

\item{open}{Logical. If TRUE and in RStudio, opens the new project in a new session.}
}
\value{
Path to the created app directory, invisibly.
}
\description{
Scaffolds a professional RDesk application with a modern dashboard layout.
The app includes a sidebar for filters, KPI cards, and an asynchronous
ggplot2 charting engine fueled by mtcars (default).
}
\examples{
if (interactive()) {
  # Create the Professional Hero Dashboard in a temporary directory
  rdesk_create_app("MyDashboard", path = tempdir())
}

# The following demonstrates just the return value without opening a window
# (Fast and safe - no \dontrun needed for this specific logical check)
path <- file.path(tempdir(), "TestLogic")
if (!dir.exists(path)) {
  # This is just a placeholder example of how to call the function safely
  message("Scaffold path will be: ", path)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{rdesk_log_dir}
\alias{rdesk_log_dir}
\title{Resolve the bundled log directory for an app}
\usage{
rdesk_log_dir(app_name = Sys.getenv("R_APP_NAME", "RDeskApp"))
}
\description{
Resolve the bundled log directory for an app
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ipc.R
\name{rdesk_message}
\alias{rdesk_message}
\title{Construct a standard RDesk IPC message envelope}
\usage{
rdesk_message(
  type,
  payload = list(),
  version = getOption("rdesk.ipc_version", "1.0")
)
}
\arguments{
\item{type}{The message type/action name}

\item{payload}{A list representing the message data}

\item{version}{The contract version (default "1.0")}
}
\value{
A list representing the standard JSON envelope
}
\description{
Construct a standard RDesk IPC message envelope
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/App.R
\name{App}
\alias{App}
\title{Create and launch a native desktop application window from R.}
\description{
Provides bidirectional native pipe communication between R and the UI.
}
\examples{
# Safe logical check (unwrapped)
app_dir <- system.file("templates/hello", package = "RDesk")
if (nzchar(app_dir)) {
  message("Built-in app directory: ", app_dir)
}

if (interactive()) {
  app <- App$new(title = "Car Visualizer", width = 1200, height = 800)
  
  app$on_ready(function() {
    message("App is ready!")
  })
  
  # Handle messages from UI
  app$on_message("get_data", function(payload) {
    list(cars = mtcars[1:5, ])
  })
  
  # Start the app
  app$run()
}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-App-new}{\code{App$new()}}
\item \href{#method-App-on_ready}{\code{App$on_ready()}}
\item \href{#method-App-on_close}{\code{App$on_close()}}
\item \href{#method-App-check_update}{\code{App$check_update()}}
\item \href{#method-App-register_hotkey}{\code{App$register_hotkey()}}
\item \href{#method-App-set_tray_menu}{\code{App$set_tray_menu()}}
\item \href{#method-App-clipboard_write}{\code{App$clipboard_write()}}
\item \href{#method-App-clipboard_read}{\code{App$clipboard_read()}}
\item \href{#method-App-on_message}{\code{App$on_message()}}
\item \href{#method-App-send}{\code{App$send()}}
\item \href{#method-App-load_ui}{\code{App$load_ui()}}
\item \href{#method-App-set_size}{\code{App$set_size()}}
\item \href{#method-App-set_position}{\code{App$set_position()}}
\item \href{#method-App-set_title}{\code{App$set_title()}}
\item \href{#method-App-minimize}{\code{App$minimize()}}
\item \href{#method-App-maximize}{\code{App$maximize()}}
\item \href{#method-App-restore}{\code{App$restore()}}
\item \href{#method-App-fullscreen}{\code{App$fullscreen()}}
\item \href{#method-App-always_on_top}{\code{App$always_on_top()}}
\item \href{#method-App-set_menu}{\code{App$set_menu()}}
\item \href{#method-App-dialog_open}{\code{App$dialog_open()}}
\item \href{#method-App-dialog_save}{\code{App$dialog_save()}}
\item \href{#method-App-dialog_folder}{\code{App$dialog_folder()}}
\item \href{#method-App-message_box}{\code{App$message_box()}}
\item \href{#method-App-dialog_color}{\code{App$dialog_color()}}
\item \href{#method-App-notify}{\code{App$notify()}}
\item \href{#method-App-loading_start}{\code{App$loading_start()}}
\item \href{#method-App-loading_progress}{\code{App$loading_progress()}}
\item \href{#method-App-loading_done}{\code{App$loading_done()}}
\item \href{#method-App-toast}{\code{App$toast()}}
\item \href{#method-App-set_tray}{\code{App$set_tray()}}
\item \href{#method-App-remove_tray}{\code{App$remove_tray()}}
\item \href{#method-App-service}{\code{App$service()}}
\item \href{#method-App-quit}{\code{App$quit()}}
\item \href{#method-App-get_dir}{\code{App$get_dir()}}
\item \href{#method-App-run}{\code{App$run()}}
\item \href{#method-App-clone}{\code{App$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-new"></a>}}
\if{latex}{\out{\hypertarget{method-App-new}{}}}
\subsection{Method \code{new()}}{
Create a new RDesk application
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$new(title, width = 1200L, height = 800L, www = NULL, icon = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{title}}{Window title string}

\item{\code{width}}{Window width in pixels (default 1200)}

\item{\code{height}}{Window height in pixels (default 800)}

\item{\code{www}}{Directory containing HTML/CSS/JS assets (default: built-in template)}

\item{\code{icon}}{Path to window icon file}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A new App instance
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-on_ready"></a>}}
\if{latex}{\out{\hypertarget{method-App-on_ready}{}}}
\subsection{Method \code{on_ready()}}{
Register a callback to fire when the window is ready
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$on_ready(fn)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{fn}}{A zero-argument function called after the server starts and window opens}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The App instance (invisible)
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-on_close"></a>}}
\if{latex}{\out{\hypertarget{method-App-on_close}{}}}
\subsection{Method \code{on_close()}}{
Register a callback to fire when the user attempts to close the window
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$on_close(fn)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{fn}}{A zero-argument function. Should return TRUE to allow closing, FALSE to cancel.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The App instance (invisible)
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-check_update"></a>}}
\if{latex}{\out{\hypertarget{method-App-check_update}{}}}
\subsection{Method \code{check_update()}}{
Check for application updates from a remote URL
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$check_update(version_url, current_version = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{version_url}}{URL to a JSON metadata file (e.g. \verb{\{"version": "1.1.0", "url": "http://..."\}})}

\item{\code{current_version}}{Optional version string to compare against. Defaults to app description version.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
A list with update status and metadata
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-register_hotkey"></a>}}
\if{latex}{\out{\hypertarget{method-App-register_hotkey}{}}}
\subsection{Method \code{register_hotkey()}}{
Register a global keyboard shortcut (hotkey)
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$register_hotkey(keys, fn)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{keys}}{Character string representing the key combination (e.g., "Ctrl+Shift+A")}

\item{\code{fn}}{A zero-argument function to be called when the hotkey is pressed}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The App instance (invisible)
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-set_tray_menu"></a>}}
\if{latex}{\out{\hypertarget{method-App-set_tray_menu}{}}}
\subsection{Method \code{set_tray_menu()}}{
Set the native system tray menu
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$set_tray_menu(items)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{items}}{A named list of lists defining the menu structure}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The App instance (invisible)
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-clipboard_write"></a>}}
\if{latex}{\out{\hypertarget{method-App-clipboard_write}{}}}
\subsection{Method \code{clipboard_write()}}{
Write text to the system clipboard
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$clipboard_write(text)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{text}}{Character string to copy}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The App instance (invisible)
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-clipboard_read"></a>}}
\if{latex}{\out{\hypertarget{method-App-clipboard_read}{}}}
\subsection{Method \code{clipboard_read()}}{
Read text from the system clipboard
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$clipboard_read()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
Character string from clipboard or NULL
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-on_message"></a>}}
\if{latex}{\out{\hypertarget{method-App-on_message}{}}}
\subsection{Method \code{on_message()}}{
Register a handler for a UI -> R message type
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$on_message(type, fn)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{type}}{Unique message identifier string}

\item{\code{fn}}{A function(payload) called when this message type arrives}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The App instance (invisible)
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-send"></a>}}
\if{latex}{\out{\hypertarget{method-App-send}{}}}
\subsection{Method \code{send()}}{
Send a message from R to the UI
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$send(type, payload = list())}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{type}}{Character string message type (received by rdesk.on() in JS)}

\item{\code{payload}}{A list or data.frame to serialise as JSON payload}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The App instance (invisible)
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-load_ui"></a>}}
\if{latex}{\out{\hypertarget{method-App-load_ui}{}}}
\subsection{Method \code{load_ui()}}{
Load an HTML file into the window
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$load_ui(path = "index.html")}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{path}}{Path relative to the www directory (e.g. "index.html")}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The App instance (invisible)
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-set_size"></a>}}
\if{latex}{\out{\hypertarget{method-App-set_size}{}}}
\subsection{Method \code{set_size()}}{
Set the window size dynamically
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$set_size(width, height)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{width}}{New width (pixels)}

\item{\code{height}}{New height (pixels)}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The App instance (invisible)
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-set_position"></a>}}
\if{latex}{\out{\hypertarget{method-App-set_position}{}}}
\subsection{Method \code{set_position()}}{
Set the window position dynamically
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$set_position(x, y)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{x}}{Horizontal position from left (pixels)}

\item{\code{y}}{Vertical position from top (pixels)}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The App instance (invisible)
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-set_title"></a>}}
\if{latex}{\out{\hypertarget{method-App-set_title}{}}}
\subsection{Method \code{set_title()}}{
Set the window title dynamically
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$set_title(title)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{title}}{New title}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The App instance (invisible)
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-minimize"></a>}}
\if{latex}{\out{\hypertarget{method-App-minimize}{}}}
\subsection{Method \code{minimize()}}{
Minimize the window to the taskbar
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$minimize()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The App instance (invisible)
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-maximize"></a>}}
\if{latex}{\out{\hypertarget{method-App-maximize}{}}}
\subsection{Method \code{maximize()}}{
Maximize the window to fill the screen
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$maximize()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The App instance (invisible)
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-restore"></a>}}
\if{latex}{\out{\hypertarget{method-App-restore}{}}}
\subsection{Method \code{restore()}}{
Restore the window from minimize/maximize
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$restore()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The App instance (invisible)
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-fullscreen"></a>}}
\if{latex}{\out{\hypertarget{method-App-fullscreen}{}}}
\subsection{Method \code{fullscreen()}}{
Toggle fullscreen mode
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$fullscreen(enabled = TRUE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{enabled}}{If TRUE, enters fullscreen. If FALSE, exits.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The App instance (invisible)
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-always_on_top"></a>}}
\if{latex}{\out{\hypertarget{method-App-always_on_top}{}}}
\subsection{Method \code{always_on_top()}}{
Set the window to stay always on top of others
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$always_on_top(enabled = TRUE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{enabled}}{If TRUE, always on top.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The App instance (invisible)
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-set_menu"></a>}}
\if{latex}{\out{\hypertarget{method-App-set_menu}{}}}
\subsection{Method \code{set_menu()}}{
Set the native window menu
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$set_menu(items)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{items}}{A named list of lists defining the menu structure}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The App instance (invisible)
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-dialog_open"></a>}}
\if{latex}{\out{\hypertarget{method-App-dialog_open}{}}}
\subsection{Method \code{dialog_open()}}{
Open a native file-open dialog
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$dialog_open(title = "Open File", filters = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{title}}{Dialog title}

\item{\code{filters}}{List of file filters, e.g. list("CSV files" = "*.csv")}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Selected file path (character) or NULL if cancelled
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-dialog_save"></a>}}
\if{latex}{\out{\hypertarget{method-App-dialog_save}{}}}
\subsection{Method \code{dialog_save()}}{
Open a native file-save dialog
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$dialog_save(title = "Save File", default_name = "", filters = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{title}}{Dialog title}

\item{\code{default_name}}{Initial filename}

\item{\code{filters}}{List of file filters}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Selected file path (character) or NULL if cancelled
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-dialog_folder"></a>}}
\if{latex}{\out{\hypertarget{method-App-dialog_folder}{}}}
\subsection{Method \code{dialog_folder()}}{
Open a native folder selection dialog
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$dialog_folder(title = "Select Folder")}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{title}}{Dialog title}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Selected directory path (character) or NULL if cancelled
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-message_box"></a>}}
\if{latex}{\out{\hypertarget{method-App-message_box}{}}}
\subsection{Method \code{message_box()}}{
Show a native message box / alert
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$message_box(message, title = "RDesk", type = "ok", icon = "info")}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{message}}{The message text}

\item{\code{title}}{The dialog title}

\item{\code{type}}{One of "ok", "okcancel", "yesno", "yesnocancel"}

\item{\code{icon}}{One of "info", "warning", "error", "question"}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The button pressed (character: "ok", "cancel", "yes", "no")
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-dialog_color"></a>}}
\if{latex}{\out{\hypertarget{method-App-dialog_color}{}}}
\subsection{Method \code{dialog_color()}}{
Open a native color selection dialog
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$dialog_color(initial_color = "#FFFFFF")}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{initial_color}}{Optional hex color to start with (e.g. "#FF0000")}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Selected hex color code or NULL if cancelled
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-notify"></a>}}
\if{latex}{\out{\hypertarget{method-App-notify}{}}}
\subsection{Method \code{notify()}}{
Send a native desktop notification
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$notify(title, body = "")}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{title}}{Notification title}

\item{\code{body}}{Notification body text}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The App instance (invisible)
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-loading_start"></a>}}
\if{latex}{\out{\hypertarget{method-App-loading_start}{}}}
\subsection{Method \code{loading_start()}}{
Show a loading state in the UI
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$loading_start(
  message = "Loading...",
  progress = NULL,
  cancellable = FALSE,
  job_id = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{message}}{Text shown under the spinner}

\item{\code{progress}}{Optional numeric 0-100 for a progress bar}

\item{\code{cancellable}}{If TRUE, shows a cancel button in the UI}

\item{\code{job_id}}{Optional job_id from rdesk_async() to wire cancel button}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-loading_progress"></a>}}
\if{latex}{\out{\hypertarget{method-App-loading_progress}{}}}
\subsection{Method \code{loading_progress()}}{
Update progress on an active loading state
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$loading_progress(value, message = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{value}}{Numeric 0-100}

\item{\code{message}}{Optional updated message}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-loading_done"></a>}}
\if{latex}{\out{\hypertarget{method-App-loading_done}{}}}
\subsection{Method \code{loading_done()}}{
Hide the loading state in the UI
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$loading_done()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-toast"></a>}}
\if{latex}{\out{\hypertarget{method-App-toast}{}}}
\subsection{Method \code{toast()}}{
Show a non-blocking toast notification in the UI
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$toast(message, type = "info", duration_ms = 3000L)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{message}}{Text to show}

\item{\code{type}}{One of "info", "success", "warning", "error"}

\item{\code{duration_ms}}{How long to show it (default 3000ms)}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-set_tray"></a>}}
\if{latex}{\out{\hypertarget{method-App-set_tray}{}}}
\subsection{Method \code{set_tray()}}{
Set or update the system tray icon
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$set_tray(label = "RDesk App", icon = NULL, on_click = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{label}}{Tooltip text for the tray icon}

\item{\code{icon}}{Path to .ico file (optional)}

\item{\code{on_click}}{Character "left" or "right" or callback function(button)}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
The App instance (invisible)
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-remove_tray"></a>}}
\if{latex}{\out{\hypertarget{method-App-remove_tray}{}}}
\subsection{Method \code{remove_tray()}}{
Remove the system tray icon
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$remove_tray()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The App instance (invisible)
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-service"></a>}}
\if{latex}{\out{\hypertarget{method-App-service}{}}}
\subsection{Method \code{service()}}{
Service this app's pending native events
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$service()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The App instance (invisible)
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-quit"></a>}}
\if{latex}{\out{\hypertarget{method-App-quit}{}}}
\subsection{Method \code{quit()}}{
Close the window and stop the app's event loop.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$quit()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
The App instance (invisible)
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-get_dir"></a>}}
\if{latex}{\out{\hypertarget{method-App-get_dir}{}}}
\subsection{Method \code{get_dir()}}{
Get the application root directory (where www/ and R/ are located).
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$get_dir()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
Character string path.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-run"></a>}}
\if{latex}{\out{\hypertarget{method-App-run}{}}}
\subsection{Method \code{run()}}{
Start the application - opens the window
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$run(block = TRUE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{block}}{If TRUE (default), blocks with an event loop until the window is closed.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-App-clone"></a>}}
\if{latex}{\out{\hypertarget{method-App-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{App$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/async.R
\name{rdesk_stop_daemons}
\alias{rdesk_stop_daemons}
\title{Stop the mirai daemon pool}
\usage{
rdesk_stop_daemons()
}
\description{
Called at App$cleanup().
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{rdesk_parse_hotkey}
\alias{rdesk_parse_hotkey}
\title{Parse a hotkey string into modifiers and virtual key codes}
\usage{
rdesk_parse_hotkey(keys)
}
\arguments{
\item{keys}{String like "Ctrl+Shift+A" or "Alt+F4"}
}
\value{
List with modifiers and vk
}
\description{
Parse a hotkey string into modifiers and virtual key codes
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/window.R
\name{rdesk_read_events}
\alias{rdesk_read_events}
\title{Read all pending stdout lines from the launcher without blocking}
\usage{
rdesk_read_events(proc)
}
\arguments{
\item{proc}{Process object}
}
\value{
A list of parsed JSON events
}
\description{
Read all pending stdout lines from the launcher without blocking
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/window.R
\name{rdesk_open_window}
\alias{rdesk_open_window}
\title{Open a native window pointing to a URL}
\usage{
rdesk_open_window(
  url,
  title = "RDesk",
  width = 1200,
  height = 800,
  www_path = ""
)
}
\arguments{
\item{url}{The target URL to load}

\item{title}{Window title}

\item{width}{Window width}

\item{height}{Window height}

\item{www_path}{Path to the local assets directory}
}
\value{
A processx process object
}
\description{
Open a native window pointing to a URL
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/App.R
\name{rdesk_auto_update}
\alias{rdesk_auto_update}
\title{Automatically check for and install app updates}
\usage{
rdesk_auto_update(
  version_url,
  download_url,
  current_version,
  silent = FALSE,
  app = NULL
)
}
\arguments{
\item{version_url}{URL to a plain text file containing the latest version string (e.g., "1.1.0")}

\item{download_url}{URL to the latest installer .exe}

\item{current_version}{Current app version string e.g. "1.0.0"}

\item{silent}{If TRUE, downloads and installs without prompting. Default FALSE.}

\item{app}{Optional App instance for showing toast notifications.}
}
\value{
Invisible TRUE if update was applied, FALSE otherwise.
}
\description{
\code{rdesk_auto_update} is a high-level function designed for bundled (standalone)
applications. It checks a remote version string, compares it with the current
version, and if a newer version is found, it downloads and executes the installer
silently before quitting the current application.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build.R
\name{build_app}
\alias{build_app}
\title{Build a self-contained distributable from an RDesk application}
\usage{
build_app(
  app_dir = ".",
  out_dir = file.path(tempdir(), "dist"),
  app_name = NULL,
  version = NULL,
  r_version = NULL,
  include_packages = character(0),
  portable_r_method = c("extract_only", "installer"),
  runtime_dir = NULL,
  overwrite = FALSE,
  build_installer = FALSE,
  publisher = "RDesk User",
  website = "https://github.com/Janakiraman-311/RDesk",
  license_file = NULL,
  icon_file = NULL,
  prune_runtime = TRUE,
  dry_run = FALSE
)
}
\arguments{
\item{app_dir}{Path to the app directory (must contain app.R and www/)}

\item{out_dir}{Output directory for the zip file (created if not exists)}

\item{app_name}{Name of the application. Defaults to name in DESCRIPTION or "MyRDeskApp".}

\item{version}{Version string. Defaults to version in DESCRIPTION or "1.0.0".}

\item{r_version}{R version to bundle e.g. "4.4.2". Defaults to current R version.}

\item{include_packages}{Character vector of extra CRAN packages to bundle.
RDesk's own dependencies are always included automatically.}

\item{portable_r_method}{How to provision the bundled R runtime when
\code{runtime_dir = "download"} is used explicitly. \code{"extract_only"} requires
standalone 7-Zip and never launches the R installer. \code{"installer"} allows
the legacy silent installer path explicitly.}

\item{runtime_dir}{Controls which R runtime is bundled with the app.
\itemize{
\item \code{NULL} (default) -- copies the developer's currently running R
installation. Guarantees the bundled packages and the runtime are the
same R version, eliminating renv version-mismatch crashes.
\item A filesystem path -- copies that R installation root directly.
\item \code{"download"} -- downloads a portable R installer from CRAN
(legacy behaviour; requires network; risks version mismatch with renv).
}}

\item{overwrite}{If TRUE, overwrite existing output. Default FALSE.}

\item{build_installer}{If TRUE, also build a Windows installer (.exe) using InnoSetup.}

\item{publisher}{Documentation for the application publisher (used in installer).}

\item{website}{URL for the application website (used in installer).}

\item{license_file}{Path to a license file (.txt or .rtf) to include in the installer.}

\item{icon_file}{Path to an .ico file for the installer and application shortcut.}

\item{prune_runtime}{If TRUE, remove unnecessary files (Tcl/Tk, docs, tests) from
the bundled R runtime to reduce size (~15-20MB saving). Default TRUE.}

\item{dry_run}{If TRUE, performs a quick validation of the app structure and
environment without performing the full build. Default FALSE.}
}
\value{
Path to the created zip file, invisibly.
}
\description{
Build a self-contained distributable from an RDesk application
}
\examples{
# Prepare an app directory (following scaffold example)
app_path <- file.path(tempdir(), "MyApp")
rdesk_create_app("MyApp", path = tempdir())

# Perform a dry-run build (fast, no external binaries downloaded)
build_app(app_path, out_dir = tempdir(), dry_run = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{rdesk_is_bundle}
\alias{rdesk_is_bundle}
\title{Check if the app is running in a bundled (standalone) environment}
\usage{
rdesk_is_bundle()
}
\value{
TRUE if running in a bundle, FALSE otherwise
}
\description{
Check if the app is running in a bundled (standalone) environment
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ipc.R
\name{rdesk_parse_message}
\alias{rdesk_parse_message}
\title{Parse and validate an incoming RDesk IPC message}
\usage{
rdesk_parse_message(raw_json)
}
\arguments{
\item{raw_json}{The raw JSON string from the frontend}
}
\value{
A list containing the validated message components, or NULL if invalid
}
\description{
Parse and validate an incoming RDesk IPC message
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/async.R
\name{rdesk_jobs_pending}
\alias{rdesk_jobs_pending}
\title{Check if any background jobs are pending}
\usage{
rdesk_jobs_pending()
}
\value{
Number of pending jobs.
}
\description{
Check if any background jobs are pending
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{rdesk_error_plot}
\alias{rdesk_error_plot}
\title{Generate a base64-encoded error plot}
\usage{
rdesk_error_plot(message = "Error generating plot")
}
\arguments{
\item{message}{Error message to display (optional)}
}
\value{
A base64-encoded PNG string
}
\description{
Generate a base64-encoded error plot
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{rdesk_plot_to_base64}
\alias{rdesk_plot_to_base64}
\title{Convert a ggplot2 object to a base64-encoded PNG string}
\usage{
rdesk_plot_to_base64(plot, width = 6, height = 4, dpi = 96)
}
\arguments{
\item{plot}{A ggplot2 object}

\item{width}{Width in inches (default 6)}

\item{height}{Height in inches (default 4)}

\item{dpi}{DPI resolution (default 96)}
}
\value{
A base64-encoded PNG string or a fallback error plot
}
\description{
Convert a ggplot2 object to a base64-encoded PNG string
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/async.R
\name{async}
\alias{async}
\title{Wrap a message handler to run asynchronously with zero configuration}
\usage{
async(
  fn,
  app = NULL,
  loading_message = "Working...",
  cancellable = TRUE,
  error_message = "Error: "
)
}
\arguments{
\item{fn}{The handler function, taking a \code{payload} argument.}

\item{app}{The RDesk \code{App} instance. If NULL, tries to resolve from the global registry.}

\item{loading_message}{Message to display in the UI overlay while working.}

\item{cancellable}{Whether the UI should show a 'Cancel' button.}

\item{error_message}{Prefix for toast notifications if the task fails.}
}
\value{
A wrapped handler function suitable for \code{app$on_message()}.
}
\description{
\code{async()} is the simplest way to make an RDesk message handler
non-blocking. Wrap any handler function with \code{async()} and RDesk
automatically handles background execution, loading states, error toasts,
and result routing.

\code{async()} transforms a standard RDesk message handler into a background task.
The UI remains responsive while the task runs. When finished, a result
message (e.g., \code{get_data_result}) is automatically sent back to the UI.
}
\details{
To ensure the background worker has access to all application logic, RDesk
automatically sources every \code{.R} file in the application's \verb{R/} directory
before executing the task. It also snapshots currently loaded packages
(excluding system packages) to recreate the environment.
}
\examples{
if (interactive()) {
  app$on_message("filter_cars", async(function(payload) {
    mtcars[mtcars$cyl == payload$cylinders, ]
  }, app = app))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/App.R
\name{rdesk_service}
\alias{rdesk_service}
\title{Service all active RDesk applications}
\usage{
rdesk_service()
}
\description{
Processes native OS events for all open windows.
Call this periodically if you are running apps with \code{block = FALSE}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/async.R
\name{rdesk_async}
\alias{rdesk_async}
\title{Run a task in the background}
\usage{
rdesk_async(
  task,
  args = list(),
  on_done = NULL,
  on_error = NULL,
  timeout_sec = NULL,
  app_id = NULL
)
}
\arguments{
\item{task}{A function to run in the background.}

\item{args}{A list of arguments to pass to the task.}

\item{on_done}{Callback function(result) called when the task finishes successfully.}

\item{on_error}{Callback function(error) called if the task fails.}

\item{timeout_sec}{Optional timeout in seconds. If exceeded, the job is
cancelled and \code{on_error()} receives a timeout error.}

\item{app_id}{Optional App ID used to associate a job with a specific app.}
}
\value{
Invisible job ID.
}
\description{
Automatically switches between 'mirai' (persistent daemons) and 'callr' (on-demand processes).
}
\examples{
# Fast, non-interactive task check (safe to unwrap)
rdesk_jobs_pending()

if (interactive()) {
  # Run a long-running computation in the background
  rdesk_async(
    task = function(n) { Sys.sleep(2); sum(runif(n)) },
    args = list(n = 1e6),
    on_done = function(res) message("Task finished: ", res),
    on_error = function(err) message("Task failed: ", err$message)
  )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/async.R
\name{rdesk_jobs_list}
\alias{rdesk_jobs_list}
\title{List currently pending background jobs}
\usage{
rdesk_jobs_list()
}
\value{
A data.frame with job ID, started time, backend, and app ID.
}
\description{
List currently pending background jobs
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ipc.R
\name{rdesk_make_router}
\alias{rdesk_make_router}
\title{Create an IPC message router}
\usage{
rdesk_make_router()
}
\value{
A list with register() and dispatch() methods
}
\description{
Create an IPC message router
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{rdesk_df_to_list}
\alias{rdesk_df_to_list}
\title{Convert a data frame to a list suitable for JSON serialization}
\usage{
rdesk_df_to_list(df)
}
\arguments{
\item{df}{Data frame to convert}
}
\value{
A list with 'rows' (list of lists) and 'cols' (character vector)
}
\description{
Convert a data frame to a list suitable for JSON serialization
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build.R
\name{rdesk_copy_r_runtime}
\alias{rdesk_copy_r_runtime}
\title{Copy an R installation into the bundle staging directory}
\usage{
rdesk_copy_r_runtime(r_home, dest_dir)
}
\description{
Copies \verb{bin/}, \verb{library/}, \verb{etc/}, \verb{modules/}, and \verb{include/} from
\code{r_home} into \code{dest_dir}. Skips heavyweight directories
(Tcl/Tk, docs, tests) that are already pruned by \code{rdesk_prune_runtime()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build.R
\name{rdesk_validate_build_inputs}
\alias{rdesk_validate_build_inputs}
\title{Validate build inputs before starting the process}
\usage{
rdesk_validate_build_inputs(
  app_dir,
  extra_pkgs,
  build_installer = FALSE,
  portable_r_method = c("extract_only", "installer"),
  runtime_dir = NULL,
  use_download = FALSE
)
}
\arguments{
\item{app_dir}{Path to app directory.}

\item{extra_pkgs}{Character vector of packages.}

\item{build_installer}{Logical.}

\item{portable_r_method}{Method for R portability.}

\item{runtime_dir}{Path to pre-existing runtime.}
}
\description{
Validate build inputs before starting the process
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/async.R
\name{rdesk_poll_jobs}
\alias{rdesk_poll_jobs}
\title{Poll background jobs}
\usage{
rdesk_poll_jobs()
}
\description{
This is called internally by the main event loop to check if any
background tasks have finished. Handles both mirai and callr backends.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build.R
\name{rdesk_resolve_launcher_bin_dir}
\alias{rdesk_resolve_launcher_bin_dir}
\title{Resolve a source-tree launcher binary directory}
\usage{
rdesk_resolve_launcher_bin_dir(project_root)
}
\description{
Resolve a source-tree launcher binary directory
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{rdesk_sanitize_log_component}
\alias{rdesk_sanitize_log_component}
\title{Sanitize an app name for filesystem-safe bundled log paths}
\usage{
rdesk_sanitize_log_component(x)
}
\description{
Sanitize an app name for filesystem-safe bundled log paths
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/window.R
\name{rdesk_send_cmd}
\alias{rdesk_send_cmd}
\title{Send a JSON command to the launcher process over stdin}
\usage{
rdesk_send_cmd(proc, cmd, payload = list(), id = NULL)
}
\arguments{
\item{proc}{Process object}

\item{cmd}{Command string (e.g., "QUIT", "SET_MENU")}

\item{payload}{Data to send as JSON}

\item{id}{Optional request ID for async responses}
}
\description{
Send a JSON command to the launcher process over stdin
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build.R
\name{rdesk_detect_r_home}
\alias{rdesk_detect_r_home}
\title{Auto-detect the current R installation root}
\usage{
rdesk_detect_r_home()
}
\description{
Auto-detect the current R installation root
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{rdesk_log}
\alias{rdesk_log}
\title{Log a message to the app's log file}
\usage{
rdesk_log(
  message,
  level = "INFO",
  app_name = Sys.getenv("R_APP_NAME", "RDeskApp")
)
}
\arguments{
\item{message}{Message to log}

\item{level}{Log level ("INFO", "WARN", "ERROR")}

\item{app_name}{Optional app name to determine log file}
}
\description{
Log a message to the app's log file
}
\keyword{internal}
